@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ class CControllerMediatypeEnable extends CController {
 	}
 
 	protected function checkPermissions() {
-		if ($this->getUserType() != USER_TYPE_SUPER_ADMIN) {
+		if (!$this->checkAccess(CRoleHelper::UI_ADMINISTRATION_MEDIA_TYPES)) {
 			return false;
 		}
 
@@ -62,13 +62,17 @@ class CControllerMediatypeEnable extends CController {
 
 		$updated = count($mediatypes);
 
-		$response = new CControllerResponseRedirect('zabbix.php?action=mediatype.list&uncheck=1');
+		$response = new CControllerResponseRedirect((new CUrl('zabbix.php'))
+			->setArgument('action', 'mediatype.list')
+			->setArgument('page', CPagerHelper::loadPage('mediatype.list', null))
+		);
 
 		if ($result) {
-			$response->setMessageOk(_n('Media type enabled', 'Media types enabled', $updated));
+			$response->setFormData(['uncheck' => '1']);
+			CMessageHelper::setSuccessTitle(_n('Media type enabled', 'Media types enabled', $updated));
 		}
 		else {
-			$response->setMessageError(_n('Cannot enable media type', 'Cannot enable media types', $updated));
+			CMessageHelper::setErrorTitle(_n('Cannot enable media type', 'Cannot enable media types', $updated));
 		}
 		$this->setResponse($response);
 	}
