@@ -23,24 +23,82 @@ class CControllerMenuPopup extends CController {
 
 	protected function checkInput() {
 		$fields = [
-			'type' => 'required|in history,host,item,item_configuration,item_prototype_configuration,map_element,refresh,trigger,trigger_macro,widget_actions',
+			'type' => 'required|in history,host,item,item_configuration,item_prototype_configuration,map_element,trigger,trigger_macro',
 			'data' => 'array'
 		];
 
-		$ret = $this->validateInput($fields);
+		$ret = $this->validateInput($fields) && $this->validateInputData();
 
 		if (!$ret) {
-			$output = [];
-			if (($messages = getMessages()) !== null) {
-				$output['errors'] = $messages->toString();
-			}
-
-			$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
+			$this->setResponse(
+				new CControllerResponseData(['main_block' => json_encode([
+					'error' => [
+						'messages' => array_column(get_and_clear_messages(), 'message')
+					]
+				])])
+			);
 		}
 
 		return $ret;
 	}
 
+	protected function validateInputData(): bool {
+		$type = $this->getInput('type');
+
+		switch ($type) {
+			case 'host':
+				$rules = [
+					'hostid' => 'required|db hosts.hostid',
+					'has_goto' => 'in 0'
+				];
+				break;
+
+			case 'history':
+			case 'item':
+				$rules = [
+					'itemid' => 'required|db items.itemid'
+				];
+				break;
+
+			case 'item_configuration':
+			case 'item_prototype_configuration':
+				$rules = [
+					'itemid' => 'required|db items.itemid',
+					'backurl' => 'required|string'
+				];
+				break;
+
+			case 'map_element':
+				$rules = [
+					'sysmapid' => 'required|db sysmaps.sysmapid',
+					'selementid' => 'required|db sysmaps_elements.selementid',
+					'unique_id' => 'string',
+					'severity_min' => 'in '.implode(',', [TRIGGER_SEVERITY_NOT_CLASSIFIED, TRIGGER_SEVERITY_INFORMATION, TRIGGER_SEVERITY_WARNING, TRIGGER_SEVERITY_AVERAGE, TRIGGER_SEVERITY_HIGH, TRIGGER_SEVERITY_DISASTER]),
+					'hostid' => 'db hosts.hostid'
+				];
+				break;
+
+			case 'trigger':
+				$rules = [
+					'triggerid' => 'required|db triggers.triggerid',
+					'eventid' => 'db events.eventid',
+					'acknowledge' => 'in 0,1'
+				];
+				break;
+
+			case 'trigger_macro':
+				$rules = [];
+				break;
+		}
+
+		$this->input['data'] = array_intersect_key($this->getInput('data', []), $rules);
+
+		$validator = new CNewValidator($this->input['data'], $rules);
+		array_map('error', $validator->getAllErrors());
+
+		return !$validator->isError() && !$validator->isErrorFatal();
+	}
+
 	protected function checkPermissions() {
 		return true;
 	}
@@ -240,13 +298,23 @@ class CControllerMenuPopup extends CController {
 		if ($db_items) {
 			$db_item = $db_items[0];
 			$is_writable = false;
+			$is_executable = false;
 
-			if ($db_item['type'] != ITEM_TYPE_HTTPTEST && CWebUser::getType() > USER_TYPE_ZABBIX_USER) {
-				$is_writable = (bool) API::Host()->get([
-					'output' => [],
-					'hostids' => $db_item['hostid'],
-					'editable' => true
-				]);
+			if ($db_item['type'] != ITEM_TYPE_HTTPTEST) {
+				if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
+					$is_writable = true;
+				}
+				elseif (CWebUser::getType() == USER_TYPE_ZABBIX_ADMIN) {
+					$is_writable = (bool) API::Host()->get([
+						'output' => [],
+						'hostids' => $db_item['hostid'],
+						'editable' => true
+					]);
+				}
+			}
+
+			if (in_array($db_item['type'], checkNowAllowedTypes())) {
+				$is_executable = $is_writable ? true : CWebUser::checkAccess(CRoleHelper::ACTIONS_INVOKE_EXECUTE_NOW);
 			}
 
 			return [
@@ -259,6 +327,7 @@ class CControllerMenuPopup extends CController {
 				'history' => $db_item['history'] != 0,
 				'trends' => $db_item['trends'] != 0,
 				'isWriteable' => $is_writable,
+				'isExecutable' => $is_executable,
 				'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
 			];
 		}
@@ -422,7 +491,6 @@ class CControllerMenuPopup extends CController {
 	 * @param array   $data
 	 * @param string  $data['sysmapid']
 	 * @param string  $data['selementid']
-	 * @param array   $data['options']       (optional)
 	 * @param int     $data['severity_min']  (optional)
 	 * @param int     $data['unique_id']     (optional)
 	 * @param string  $data['hostid']        (optional)
@@ -565,13 +633,8 @@ class CControllerMenuPopup extends CController {
 	 *
 	 * @param array  $data
 	 * @param string $data['triggerid']
-	 * @param string $data['eventid']                 (optional) Mandatory for Acknowledge menu.
-	 * @param bool   $data['acknowledge']             (optional) Whether to show Acknowledge menu.
-	 * @param int    $data['severity_min']            (optional)
-	 * @param bool   $data['show_suppressed']         (optional)
-	 * @param array  $data['urls']                    (optional)
-	 * @param string $data['urls']['name']
-	 * @param string $data['urls']['url']
+	 * @param string $data['eventid']      (optional) Mandatory for Acknowledge menu.
+	 * @param bool   $data['acknowledge']  (optional) Whether to show Acknowledge menu.
 	 *
 	 * @return mixed
 	 */
@@ -663,16 +726,8 @@ class CControllerMenuPopup extends CController {
 				if ($events) {
 					$event = $events[0];
 
-					if ($event['r_eventid'] != 0) {
-						$can_be_closed = false;
-					}
-					else {
-						foreach ($event['acknowledges'] as $acknowledge) {
-							if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
-								$can_be_closed = false;
-								break;
-							}
-						}
+					if ($can_be_closed) {
+						$can_be_closed = !isEventClosed($event);
 					}
 
 					foreach ($event['urls'] as $url) {
@@ -704,6 +759,7 @@ class CControllerMenuPopup extends CController {
 							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY)
 							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS)
 							|| $can_be_closed
+							|| CWebUser::checkAccess(CRoleHelper::ACTIONS_SUPPRESS_PROBLEMS)
 						)
 				);
 			}
@@ -826,8 +882,8 @@ class CControllerMenuPopup extends CController {
 			$output['data'] = $menu_data;
 		}
 
-		if (($messages = getMessages()) !== null) {
-			$output['errors'] = $messages->toString();
+		if ($messages = get_and_clear_messages()) {
+			$output['error']['messages'] = array_column($messages, 'message');
 		}
 
 		$this->setResponse(new CControllerResponseData(['main_block' => json_encode($output)]));
