@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,29 +28,28 @@ class CControllerWidgetMapView extends CControllerWidget {
 		$this->setType(WIDGET_MAP);
 		$this->setValidationRules([
 			'name' => 'string',
-			'uniqueid' => 'required|string',
 			'initial_load' => 'in 0,1',
 			'fields' => 'json',
-			'storage' => 'array'
+			'current_sysmapid' => 'db sysmaps.sysmapid',
+			'unique_id' => 'string',
+			'previous_maps' => 'array'
 		]);
 	}
 
 	protected function doAction() {
 		$fields = $this->getForm()->getFieldsData();
-		$storage = $this->getInput('storage', []);
-		$uniqueid = $this->getInput('uniqueid');
 		$sysmap_data = null;
 		$previous_map = null;
 		$sysmapid = null;
 		$error = null;
 
 		// Get previous map.
-		if (array_key_exists('previous_maps', $storage)) {
-			$previous_map = array_filter(explode(',', $storage['previous_maps']), 'is_numeric');
+		if ($this->hasInput('previous_maps')) {
+			$previous_maps = array_filter($this->getInput('previous_maps'), 'is_numeric');
 
-			if ($previous_map) {
+			if ($previous_maps) {
 				$previous_map = API::Map()->get([
-					'sysmapids' => [array_pop($previous_map)],
+					'sysmapids' => [array_pop($previous_maps)],
 					'output' => ['sysmapid', 'name']
 				]);
 
@@ -58,11 +57,14 @@ class CControllerWidgetMapView extends CControllerWidget {
 			}
 		}
 
-		$sysmapid = array_key_exists('current_sysmapid', $storage)
-			? $storage['current_sysmapid']
-			: (array_key_exists('sysmapid', $fields) ? $fields['sysmapid'] : null);
+		if ($this->hasInput('current_sysmapid')) {
+			$sysmapid = $this->getInput('current_sysmapid');
+		}
+		elseif (array_key_exists('sysmapid', $fields)) {
+			$sysmapid =  $fields['sysmapid'];
+		}
 
-		$sysmap_data = CMapHelper::get(($sysmapid === null) ? [] : [$sysmapid]);
+		$sysmap_data = CMapHelper::get(($sysmapid == null) ? [] : [$sysmapid], ['unique_id' => $this->getInput('unique_id')]);
 
 		if ($sysmapid === null || $sysmap_data['id'] < 0) {
 			$error = _('No permissions to referred object or it does not exist!');
@@ -70,15 +72,14 @@ class CControllerWidgetMapView extends CControllerWidget {
 
 		// Rewrite actions to force Submaps be opened in same widget, instead of separate window.
 		foreach ($sysmap_data['elements'] as &$element) {
-			$actions = CJs::decodeJson($element['actions']);
-			$actions['data']['widget_uniqueid'] = $uniqueid;
-			$element['actions'] = CJs::encodeJson($actions);
+			$actions = json_decode($element['actions'], true);
+			$element['actions'] = json_encode($actions);
 		}
 		unset($element);
 
 		// Pass variables to view.
 		$this->setResponse(new CControllerResponseData([
-			'name' => $this->getInput('name', $this->getDefaultHeader()),
+			'name' => $this->getInput('name', $this->getDefaultName()),
 			'sysmap_data' => $sysmap_data ?: [],
 			'widget_settings' => [
 				'current_sysmapid' => $sysmapid,
@@ -88,7 +89,6 @@ class CControllerWidgetMapView extends CControllerWidget {
 				'source_type' => $fields['source_type'],
 				'previous_map' => $previous_map,
 				'initial_load' => $this->getInput('initial_load', 1),
-				'uniqueid' => $uniqueid,
 				'error' => $error
 			],
 			'user' => [
