@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,11 +25,14 @@
 
 $this->includeJsFile('administration.usergroup.edit.js.php');
 
-$widget = (new CWidget())->setTitle(_('User groups'));
+$widget = (new CWidget())
+	->setTitle(_('User groups'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::ADMINISTRATION_USERGROUP_EDIT));
 
 $form = (new CForm())
+	->setId('user-group-form')
 	->setName('user_group_form')
-	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE);
+	->setAttribute('aria-labelledby', ZBX_STYLE_PAGE_TITLE);
 
 if ($data['usrgrpid'] != 0) {
 	$form->addVar('usrgrpid', $data['usrgrpid']);
@@ -73,8 +76,16 @@ if ($data['can_update_group']) {
 			GROUP_GUI_ACCESS_DISABLED => user_auth_type2str(GROUP_GUI_ACCESS_DISABLED)
 		]));
 
+	$userdirectory = (new CSelect('userdirectoryid'))
+		->setValue($data['userdirectoryid'])
+		->setFocusableElementId('userdirectoryid')
+		->addOption((new CSelectOption(0, _('Default')))->addClass(ZBX_STYLE_DEFAULT_OPTION))
+		->addOptions(CSelect::createOptionsFromArray($data['userdirectories']))
+		->setAdaptiveWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
+
 	$form_list
 		->addRow((new CLabel(_('Frontend access'), $select_gui_access->getFocusableElementId())), $select_gui_access)
+		->addRow((new CLabel(_('LDAP Server'), $userdirectory->getFocusableElementId())), $userdirectory)
 		->addRow(_('Enabled'), (new CCheckBox('users_status', GROUP_STATUS_ENABLED))
 			->setUncheckedValue(GROUP_STATUS_DISABLED)
 			->setChecked($data['users_status'] == GROUP_STATUS_ENABLED)
@@ -100,14 +111,69 @@ $form_list->addRow(_('Debug mode'),
 		->setChecked($data['debug_mode'] == GROUP_DEBUG_MODE_ENABLED)
 );
 
-$permissions_form_list = new CFormList('permissions_form_list');
-$permissions_form_list->addRow(_('Permissions'),
-	(new CDiv(new CPartial('administration.usergroup.grouprights.html', [
+$template_permissions_form_grid = (new CFormGrid())->addItem([
+	new CLabel(_('Permissions')),
+	new CFormField((new CDiv(new CPartial('administration.usergroup.templategrouprights.html', [
+		'templategroup_rights' => $data['templategroup_rights']
+	])))
+		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;'))
+]);
+
+$new_templategroup_right_table = (new CTable())
+	->setId('new-templategroup-right-table')
+	->addRow([
+		(new CMultiSelect([
+			'name' => 'new_templategroup_right[groupids][]',
+			'object_name' => 'templateGroup',
+			'data' => array_intersect_key($data['template_groups_ms'],
+				array_flip($data['new_templategroup_right']['groupids'])
+			),
+			'popup' => [
+				'parameters' => [
+					'srctbl' => 'template_groups',
+					'srcfld1' => 'groupid',
+					'dstfrm' => $form->getName(),
+					'dstfld1' => 'new_templategroup_right_groupids_'
+				]
+			]
+		]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+		(new CCol(
+			(new CRadioButtonList('new_templategroup_right[permission]',
+				(int) $data['new_templategroup_right']['permission']
+			))
+				->addValue(_('Read-write'), PERM_READ_WRITE)
+				->addValue(_('Read'), PERM_READ)
+				->addValue(_('Deny'), PERM_DENY)
+				->addValue(_('None'), PERM_NONE)
+				->setModern(true)
+		))->setAttribute('style', 'vertical-align: top')
+	])
+	->addRow((new CCheckBox('new_templategroup_right[include_subgroups]'))
+		->setChecked((bool) $data['new_templategroup_right']['include_subgroups'])
+		->setLabel(_('Include subgroups'))
+	)
+	->addRow([
+		(new CSimpleButton(_('Add')))
+			->onClick('javascript: usergroups.submitNewTemplateGroupRight("usergroup.templategroupright.add");')
+			->addClass(ZBX_STYLE_BTN_LINK)
+	]);
+
+$template_permissions_form_grid
+	->addItem([
+		new CFormField((new CDiv($new_templategroup_right_table))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;'))
+	]);
+
+$host_permissions_form_grid = (new CFormGrid())->addItem([
+	new CLabel(_('Permissions')),
+	new CFormField((new CDiv(new CPartial('administration.usergroup.grouprights.html', [
 		'group_rights' => $data['group_rights']
 	])))
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-);
+		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;'))
+]);
 
 $new_group_right_table = (new CTable())
 	->setId('new-group-right-table')
@@ -144,11 +210,12 @@ $new_group_right_table = (new CTable())
 			->addClass(ZBX_STYLE_BTN_LINK)
 	]);
 
-$permissions_form_list->addRow(null,
-	(new CDiv($new_group_right_table))
-		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-		->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-);
+$host_permissions_form_grid
+	->addItem([
+		new CFormField((new CDiv($new_group_right_table))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;'))
+	]);
 
 $tag_filter_form_list = new CFormList('tagFilterFormList');
 
@@ -203,8 +270,9 @@ $tag_filter_form_list->addRow(null,
 
 $tabs = (new CTabView())
 	->addTab('user_group_tab', _('User group'), $form_list)
-	->addTab('permissions_tab', _('Permissions'), $permissions_form_list)
-	->addTab('tag_filter_tab', _('Tag filter'), $tag_filter_form_list);
+	->addTab('template_permissions_tab', _('Template permissions'), $template_permissions_form_grid, TAB_INDICATOR_TEMPLATE_PERMISSIONS)
+	->addTab('permissions_tab', _('Host permissions'), $host_permissions_form_grid, TAB_INDICATOR_HOST_PERMISSIONS)
+	->addTab('tag_filter_tab', _('Problem tag filter'), $tag_filter_form_list, TAB_INDICATOR_TAG_FILTER);
 if (!$data['form_refresh']) {
 	$tabs->setSelected(0);
 }
@@ -240,5 +308,3 @@ else {
 $form->addItem($tabs);
 $widget->addItem($form);
 $widget->show();
-
-$this->addJsFile('multiselect.js');
