@@ -61,7 +61,7 @@
 
 			this._app = new ChartList( $('#charts'), this._data.timeline, this._data.config, this._container);
 			this._app.setCharts(this._data.charts);
-			this._app.onResize();
+			this._app.refresh();
 
 			this._resize_observer = new ResizeObserver(this._app.onResize.bind(this._app));
 			this._resize_observer.observe(this._container);
@@ -245,27 +245,6 @@
 		return promise;
 	};
 
-	/**
-	 * Start or pause timeout based Chart refresh.
-	 *
-	 * @param {number} seconds  Seconds to wait before reschedule. Zero seconds will pause schedule.
-	 * @param {number} delay_loading  (optional) Add "loading indicator" only when request exceeds delay.
-	 */
-	Chart.prototype.scheduleRefresh = function(seconds, delay_loading) {
-		if (this._timeoutid) {
-			clearTimeout(this._timeoutid);
-		}
-
-		if (!seconds) {
-			return;
-		}
-
-		this.refresh(delay_loading)
-			.finally(_ => {
-				this._timeoutid = setTimeout(() => this.scheduleRefresh(seconds, 0), seconds * 1000);
-			});
-	};
-
 	/**
 	 * @param {jQuery} $el       A container where charts are maintained.
 	 * @param {object} timeline  Time control object.
@@ -372,10 +351,6 @@
 			clearTimeout(this._timeoutid);
 		}
 
-		for (const chart of this.charts) {
-			chart.scheduleRefresh(0);
-		}
-
 		this.updateListAndCharts(delay_loading)
 			.finally(_ => {
 				if (refresh_interval) {
@@ -430,7 +405,6 @@
 
 		if (this._prev_width === undefined) {
 			this._prev_width = width;
-			this.updateCharts();
 
 			return;
 		}
