@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,34 +19,47 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $this->addJsFile('gtlc.js');
 $this->addJsFile('flickerfreescreen.js');
 $this->addJsFile('class.svg.canvas.js');
 $this->addJsFile('class.svg.map.js');
 $this->addJsFile('layout.mode.js');
+$this->includeJsFile('monitoring.map.view.js.php');
+
+$this->enableLayoutModes();
+$web_layout_mode = $this->getLayoutMode();
 
 (new CWidget())
 	->setTitle(_('Maps'))
-	->setWebLayoutMode(CView::getLayoutMode())
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::MONITORING_MAP_VIEW))
+	->setWebLayoutMode($web_layout_mode)
 	->setControls(new CList([
 		(new CForm('get'))
 			->cleanItems()
+			->setName('map.view')
 			->addVar('action', 'map.view')
 			->addVar('sysmapid', $data['map']['sysmapid'])
 			->setAttribute('aria-label', _('Main filter'))
 			->addItem((new CList())
 				->addItem([
-					new CLabel(_('Minimum severity'), 'severity_min'),
+					new CLabel(_('Minimum severity'), 'label-severity-min'),
 					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					$data['pageFilter']->getSeveritiesMinCB()
+					(new CSelect('severity_min'))
+						->setFocusableElementId('label-severity-min')
+						->setValue($data['severity_min'])
+						->addOptions(CSelect::createOptionsFromArray($data['severities']))
 				])
 			),
 		(new CTag('nav', true, (new CList())
 			->addItem($data['map']['editable']
-				? new CRedirectButton(_('Edit map'), (new CUrl('sysmap.php'))
+				? (new CRedirectButton(_('Edit map'), (new CUrl('sysmap.php'))
 					->setArgument('sysmapid', $data['map']['sysmapid'])
 					->getUrl()
-				)
+				))->setEnabled($data['allowed_edit'])
 				: null
 			)
 			->addItem(get_icon('favourite', [
@@ -54,13 +67,11 @@ $this->addJsFile('layout.mode.js');
 				'elname' => 'sysmapid',
 				'elid' => $data['map']['sysmapid']
 			]))
-			->addItem(get_icon('fullscreen'))
+			->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]))
 		))
 			->setAttribute('aria-label', _('Content controls'))
 	]))
-	->setBreadcrumbs(
-		get_header_sysmap_table($data['map']['sysmapid'], $data['map']['name'], $data['severity_min'])
-	)
+	->setNavigation(getSysmapNavigation($data['map']['sysmapid'], $data['map']['name'], $data['severity_min']))
 	->addItem(
 		(new CDiv())
 			->addClass(ZBX_STYLE_TABLE_FORMS_CONTAINER)
@@ -82,3 +93,7 @@ $this->addJsFile('layout.mode.js');
 			)
 	)
 	->show();
+
+(new CScriptTag('view.init();'))
+	->setOnDocumentReady()
+	->show();
