@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,25 +27,28 @@ $table = (new CTableInfo())->setNoDataMessage(_('No maps added.'));
 
 foreach ($data['maps'] as $map) {
 	$table->addRow([
-		new CLink($map['label'], (new CUrl('zabbix.php'))
-			->setArgument('action', 'map.view')
-			->setArgument('sysmapid', $map['sysmapid'])
-		),
+		$data['allowed_ui_maps']
+			? new CLink($map['label'], (new CUrl('zabbix.php'))
+				->setArgument('action', 'map.view')
+				->setArgument('sysmapid', $map['sysmapid'])
+			)
+			: $map['label'],
 		(new CButton())
-			->onClick("rm4favorites('sysmapid','".$map['sysmapid']."')")
-			->addClass(ZBX_STYLE_REMOVE_BTN)
+			->setAttribute('data-sysmapid', $map['sysmapid'])
+			->onClick('rm4favorites("sysmapid", this.dataset.sysmapid);')
+			->addClass(ZBX_STYLE_BTN_REMOVE)
 			->setAttribute('aria-label', _xs('Remove, %1$s', 'screen reader', $map['label']))
 			->removeId()
 	]);
 }
 
 $output = [
-	'header' => $data['name'],
+	'name' => $data['name'],
 	'body' => $table->toString()
 ];
 
-if (($messages = getMessages()) !== null) {
-	$output['messages'] = $messages->toString();
+if ($messages = get_and_clear_messages()) {
+	$output['messages'] = array_column($messages, 'message');
 }
 
 if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
