@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,13 +26,13 @@
 $item = new CDashboardWidgetMap($data['sysmap_data'], $data['widget_settings']);
 
 $output = [
-	'header' => $data['name'],
+	'name' => $data['name'],
 	'body' => $item->toString(),
-	'script_inline' => $item->getScriptRun()
+	'sysmap_data' => $item->getScriptData()
 ];
 
-if (($messages = getMessages()) !== null) {
-	$output['messages'] = $messages->toString();
+if ($messages = get_and_clear_messages()) {
+	$output['messages'] = array_column($messages, 'message');
 }
 
 if ($this->data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
