@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,19 +21,24 @@
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 // indicator of sort field
 $sort_div = (new CSpan())->addClass(($data['sortorder'] === ZBX_SORT_DOWN) ? ZBX_STYLE_ARROW_DOWN : ZBX_STYLE_ARROW_UP);
 
-$url_details = (new CUrl('tr_events.php'))
-	->setArgument('triggerid', '')
-	->setArgument('eventid', '');
+$url_details = $data['allowed_ui_problems']
+	? (new CUrl('tr_events.php'))
+		->setArgument('triggerid', '')
+		->setArgument('eventid', '')
+	: null;
 
 $show_timeline = ($data['sortfield'] === 'clock' && $data['fields']['show_timeline']);
 $show_recovery_data = in_array($data['fields']['show'], [TRIGGERS_OPTION_RECENT_PROBLEM, TRIGGERS_OPTION_ALL]);
 
-$header_time = new CColHeader(($data['sortfield'] === 'clock') ? [_('Time'), $sort_div] : _('Time'));
+$header_time = new CColHeader(($data['sortfield'] === 'clock')
+	? [_x('Time', 'compact table header'), $sort_div]
+	: _x('Time', 'compact table header'));
 
 if ($show_timeline) {
 	$header = [
@@ -50,20 +55,32 @@ $show_opdata = $data['fields']['show_opdata'];
 
 $table = (new CTableInfo())
 	->setHeader(array_merge($header, [
-		$show_recovery_data ? _('Recovery time') : null,
-		$show_recovery_data ? _('Status') : null,
-		_('Info'),
-		($data['sortfield'] === 'host') ? [_('Host'), $sort_div] : _('Host'),
+		$show_recovery_data
+			? _x('Recovery time', 'compact table header')
+			: null,
+		$show_recovery_data
+			? _x('Status', 'compact table header')
+			: null,
+		_x('Info', 'compact table header'),
+		($data['sortfield'] === 'host')
+			? [_x('Host', 'compact table header'), $sort_div]
+			: _x('Host', 'compact table header'),
 		[
-			($data['sortfield'] === 'name') ? [_('Problem'), $sort_div] : _('Problem'),
+			($data['sortfield'] === 'name')
+				? [_x('Problem', 'compact table header'), $sort_div]
+				: _x('Problem', 'compact table header'),
 			' &bullet; ',
-			($data['sortfield'] === 'severity') ? [_('Severity'), $sort_div] : _('Severity')
+			($data['sortfield'] === 'severity')
+				? [_x('Severity', 'compact table header'), $sort_div]
+				: _x('Severity', 'compact table header')
 		],
-		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) ? _('Operational data') : null,
-		_('Duration'),
-		_('Ack'),
-		_('Actions'),
-		$data['fields']['show_tags'] ? _('Tags') : null
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY)
+			? _x('Operational data', 'compact table header')
+			: null,
+			_x('Duration', 'compact table header'),
+			_x('Ack', 'compact table header'),
+			_x('Actions', 'compact table header'),
+		$data['fields']['show_tags'] ? _x('Tags', 'compact table header') : null
 	]));
 
 $today = strtotime('today');
@@ -73,6 +90,14 @@ if ($data['data']['problems']) {
 	$triggers_hosts = makeTriggersHostsList($data['data']['triggers_hosts']);
 }
 
+$allowed = [
+	'ui_problems' => $data['allowed_ui_problems'],
+	'add_comments' => $data['allowed_add_comments'],
+	'change_severity' => $data['allowed_change_severity'],
+	'acknowledge' => $data['allowed_acknowledge'],
+	'suppress' => $data['allowed_suppress']
+];
+
 foreach ($data['data']['problems'] as $eventid => $problem) {
 	$trigger = $data['data']['triggers'][$problem['objectid']];
 
@@ -80,30 +105,31 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$value = TRIGGER_VALUE_FALSE;
 		$value_str = _('RESOLVED');
 		$value_clock = $problem['r_clock'];
+		$can_be_closed = false;
 	}
 	else {
-		$in_closing = false;
-
-		foreach ($problem['acknowledges'] as $acknowledge) {
-			if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
-				$in_closing = true;
-				break;
-			}
-		}
-
+		$in_closing = hasEventCloseAction($problem['acknowledges']);
+		$can_be_closed = ($trigger['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED && $data['allowed_close']
+			&& !$in_closing
+		);
 		$value = $in_closing ? TRIGGER_VALUE_FALSE : TRIGGER_VALUE_TRUE;
 		$value_str = $in_closing ? _('CLOSING') : _('PROBLEM');
 		$value_clock = $in_closing ? time() : $problem['clock'];
 	}
 
-	$url_details
-		->setArgument('triggerid', $problem['objectid'])
-		->setArgument('eventid', $problem['eventid']);
-
 	$cell_clock = ($problem['clock'] >= $today)
 		? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
 		: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-	$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+
+	if ($url_details !== null) {
+		$url_details
+			->setArgument('triggerid', $problem['objectid'])
+			->setArgument('eventid', $problem['eventid']);
+		$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+	}
+	else {
+		$cell_clock = new CCol($cell_clock);
+	}
 
 	$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 
@@ -112,7 +138,7 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 			$cell_r_clock = ($problem['r_clock'] >= $today)
 				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
 				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-			$cell_r_clock = (new CCol(new CLink($cell_r_clock, $url_details)))
+			$cell_r_clock = (new CCol($url_details !== null ? new CLink($cell_r_clock, $url_details) : $cell_r_clock))
 				->addClass(ZBX_STYLE_NOWRAP)
 				->addClass(ZBX_STYLE_RIGHT);
 		}
@@ -147,8 +173,32 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		}
 	}
 
-	if (array_key_exists('suppression_data', $problem) && $problem['suppression_data']) {
-		$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data']);
+	if (array_key_exists('suppression_data', $problem)) {
+		if (count($problem['suppression_data']) == 1
+				&& $problem['suppression_data'][0]['maintenanceid'] == 0
+				&& isEventRecentlyUnsuppressed($problem['acknowledges'], $unsuppression_action)) {
+			// Show blinking button if the last manual suppression was recently revoked.
+			$user_unsuppressed = array_key_exists($unsuppression_action['userid'], $data['data']['users'])
+				? getUserFullname($data['data']['users'][$unsuppression_action['userid']])
+				: _('Inaccessible user');
+
+			$info_icons[] = (new CSimpleButton())
+				->addClass(ZBX_STYLE_ACTION_ICON_UNSUPPRESS)
+				->addClass('blink')
+				->setHint(_s('Unsuppressed by: %1$s', $user_unsuppressed));
+		}
+		elseif ($problem['suppression_data']) {
+			$info_icons[] = makeSuppressedProblemIcon($problem['suppression_data'], false);
+		}
+		elseif (isEventRecentlySuppressed($problem['acknowledges'], $suppression_action)) {
+			// Show blinking button if suppression was made but is not yet processed by server.
+			$info_icons[] = makeSuppressedProblemIcon([[
+				'suppress_until' => $suppression_action['suppress_until'],
+				'username' => array_key_exists($suppression_action['userid'], $data['data']['users'])
+					? getUserFullname($data['data']['users'][$suppression_action['userid']])
+					: _('Inaccessible user')
+			]], true);
+		}
 	}
 
 	$opdata = null;
@@ -175,21 +225,16 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 			);
 
 			if ($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY) {
-				$opdata = (new CCol($opdata))
-					->addClass('opdata')
-					->addClass(ZBX_STYLE_WORDWRAP);
+				$opdata = (new CCol($opdata))->addClass('opdata');
 			}
 		}
 	}
 
 	$problem_link = [
 		(new CLinkAction($problem['name']))
-			->setAjaxHint(CHintBoxHelper::getEventList($trigger['triggerid'], $eventid, $show_timeline,
-				$data['fields']['show_tags'], $data['fields']['tags'], $data['fields']['tag_name_format'],
-				$data['fields']['tag_priority']
-			))
+			->setMenuPopup(CMenuPopupHelper::getTrigger($trigger['triggerid'], $problem['eventid']))
 			->setAttribute('aria-label', _xs('%1$s, Severity, %2$s', 'screen reader',
-				$problem['name'], getSeverityName($problem['severity'], $data['config'])
+				$problem['name'], CSeverityHelper::getName((int) $problem['severity'])
 			))
 	];
 
@@ -197,22 +242,25 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		$problem_link = array_merge($problem_link, [' (', $opdata, ')']);
 	}
 
-	$description = (new CCol($problem_link));
+	$description = (new CCol($problem_link))->addClass(ZBX_STYLE_WORDBREAK);
 
-	$description_style = getSeverityStyle($problem['severity']);
+	$description_style = CSeverityHelper::getStyle((int) $problem['severity']);
 
 	if ($value == TRIGGER_VALUE_TRUE) {
 		$description->addClass($description_style);
 	}
 
-	if (!$show_recovery_data && $data['config'][$is_acknowledged ? 'problem_ack_style' : 'problem_unack_style']) {
+	if (!$show_recovery_data
+			&& (($is_acknowledged && $data['config']['problem_ack_style'])
+				|| (!$is_acknowledged && $data['config']['problem_unack_style']))) {
 		// blinking
 		$duration = time() - $problem['clock'];
+		$blink_period = timeUnitToSeconds($data['config']['blink_period']);
 
-		if ($data['config']['blink_period'] != 0 && $duration < $data['config']['blink_period']) {
+		if ($blink_period != 0 && $duration < $blink_period) {
 			$description
 				->addClass('blink')
-				->setAttribute('data-time-to-blink', $data['config']['blink_period'] - $duration)
+				->setAttribute('data-time-to-blink', $blink_period - $duration)
 				->setAttribute('data-toggle-class', ZBX_STYLE_BLINK_HIDDEN);
 		}
 	}
@@ -240,10 +288,16 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 	}
 
 	// Create acknowledge link.
-	$problem_update_link = (new CLink($is_acknowledged ? _('Yes') : _('No')))
-		->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-		->addClass(ZBX_STYLE_LINK_ALT)
-		->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);');
+	$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+			|| $can_be_closed || $allowed['suppress'])
+		? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+			->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+			->addClass(ZBX_STYLE_LINK_ALT)
+			->setAttribute('data-eventid', $problem['eventid'])
+			->onClick('acknowledgePopUp({eventids: [this.dataset.eventid]}, this);')
+		: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+			$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+		);
 
 	$table->addRow(array_merge($row, [
 		$show_recovery_data ? $cell_r_clock : null,
@@ -251,13 +305,16 @@ foreach ($data['data']['problems'] as $eventid => $problem) {
 		makeInformationList($info_icons),
 		$triggers_hosts[$trigger['triggerid']],
 		$description,
-		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY ) ? $opdata : null,
-		(new CCol(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
-			->addClass(ZBX_STYLE_NOWRAP),
+		($show_opdata == OPERATIONAL_DATA_SHOW_SEPARATELY ) ? $opdata->addClass(ZBX_STYLE_WORDBREAK) : null,
+		(new CCol(
+			(new CLinkAction(zbx_date2age($problem['clock'], ($problem['r_eventid'] != 0) ? $problem['r_clock'] : 0)))
+				->setAjaxHint(CHintBoxHelper::getEventList($trigger['triggerid'], $eventid, $show_timeline,
+					$data['fields']['show_tags'], $data['fields']['tags'], $data['fields']['tag_name_format'],
+					$data['fields']['tag_priority']
+				))
+		))->addClass(ZBX_STYLE_NOWRAP),
 		$problem_update_link,
-		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['mediatypes'],
-			$data['data']['users'], $data['config']
-		),
+		makeEventActionsIcons($problem['eventid'], $data['data']['actions'], $data['data']['users']),
 		$data['fields']['show_tags'] ? $data['data']['tags'][$problem['eventid']] : null
 	]));
 }
@@ -271,24 +328,12 @@ if ($data['info'] !== '') {
 }
 
 $output = [
-	'aria_label' => _xs('%1$s widget', 'screen reader', $data['name']).', '.$data['info'],
-	'header' => $data['name'],
+	'name' => $data['name'],
 	'body' => $table->toString()
 ];
 
-if ($data['initial_load']) {
-	$output['script_inline'] =
-		'if (typeof refreshProblemsWidget !== typeof(Function)) {'.
-			'function refreshProblemsWidget(event, response, overlay) {'.
-				'refreshWidgetOnAcknowledgeCreate("problems", response, overlay);'.
-			'}'.
-
-			'$.subscribe("acknowledge.create", refreshProblemsWidget);'.
-		'}';
-}
-
-if (($messages = getMessages()) !== null) {
-	$output['messages'] = $messages->toString();
+if ($messages = get_and_clear_messages()) {
+	$output['messages'] = array_column($messages, 'message');
 }
 
 if ($data['user']['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
