@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Host inventory');
 $page['file'] = 'hostinventories.php';
-$page['scripts'] = ['layout.mode.js', 'multiselect.js'];
 
 $hostId = getRequest('hostid', 0);
 
@@ -73,7 +72,12 @@ CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR
  * Display
  */
 if ($hostId > 0) {
-	$data = [];
+	$data = [
+		'allowed_ui_hosts' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_HOSTS),
+		'allowed_ui_problems' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_PROBLEMS),
+		'allowed_ui_latest_data' => CWebUser::checkAccess(CRoleHelper::UI_MONITORING_LATEST_DATA),
+		'allowed_ui_conf_hosts' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+	];
 
 	// inventory info
 	$data['tableTitles'] = getHostInventories();
@@ -88,7 +92,6 @@ if ($hostId > 0) {
 		'selectTriggers' => API_OUTPUT_COUNT,
 		'selectInventory' => $inventoryFields,
 		'selectGraphs' => API_OUTPUT_COUNT,
-		'selectApplications' => API_OUTPUT_COUNT,
 		'selectDiscoveries' => API_OUTPUT_COUNT,
 		'selectHttpTests' => API_OUTPUT_COUNT,
 		'hostids' => $hostId,
@@ -131,7 +134,6 @@ if ($hostId > 0) {
 }
 else {
 	$data = [
-		'config' => select_config(),
 		'hosts' => [],
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
@@ -201,7 +203,7 @@ else {
 		$options = [
 			'output' => ['hostid', 'name', 'status'],
 			'selectInventory' => ['name', 'type', 'os', 'serialno_a', 'tag', 'macaddress_a', $data['filter']['field']],
-			'selectGroups' => API_OUTPUT_EXTEND,
+			'selectHostGroups' => ['name'],
 			'groupids' => $filter_groupids,
 			'filter' => ['inventory_mode' => [HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC]]
 		];
@@ -244,7 +246,7 @@ else {
 			unset($host);
 		}
 
-		$limit = $data['config']['search_limit'] + 1;
+		$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 
 		order_result($data['hosts'], $sortField, $sortOrder);
 
