@@ -78,7 +78,7 @@ class CAuditLog extends CApiService {
 		$actions = [
 			CAudit::ACTION_ADD, CAudit::ACTION_UPDATE, CAudit::ACTION_DELETE, CAudit::ACTION_LOGOUT,
 			CAudit::ACTION_EXECUTE, CAudit::ACTION_LOGIN_SUCCESS, CAudit::ACTION_LOGIN_FAILED,
-			CAudit::ACTION_HISTORY_CLEAR
+			CAudit::ACTION_HISTORY_CLEAR, CAudit::ACTION_CONFIG_REFRESH
 		];
 		$resourcetype = [
 			CAudit::RESOURCE_ACTION, CAudit::RESOURCE_AUTHENTICATION, CAudit::RESOURCE_AUTH_TOKEN,
@@ -90,9 +90,10 @@ class CAuditLog extends CApiService {
 			CAudit::RESOURCE_MACRO, CAudit::RESOURCE_MAINTENANCE, CAudit::RESOURCE_MAP, CAudit::RESOURCE_MEDIA_TYPE,
 			CAudit::RESOURCE_MODULE, CAudit::RESOURCE_PROXY, CAudit::RESOURCE_REGEXP, CAudit::RESOURCE_SCENARIO,
 			CAudit::RESOURCE_SCHEDULED_REPORT, CAudit::RESOURCE_SCRIPT, CAudit::RESOURCE_SETTINGS, CAudit::RESOURCE_SLA,
-			CAudit::RESOURCE_TEMPLATE, CAudit::RESOURCE_TEMPLATE_DASHBOARD, CAudit::RESOURCE_TRIGGER,
-			CAudit::RESOURCE_TRIGGER_PROTOTYPE, CAudit::RESOURCE_USER, CAudit::RESOURCE_USER_GROUP,
-			CAudit::RESOURCE_USER_ROLE, CAudit::RESOURCE_VALUE_MAP, CAudit::RESOURCE_HA_NODE
+			CAudit::RESOURCE_TEMPLATE, CAudit::RESOURCE_TEMPLATE_GROUP, CAudit::RESOURCE_TEMPLATE_DASHBOARD,
+			CAudit::RESOURCE_TRIGGER, CAudit::RESOURCE_TRIGGER_PROTOTYPE, CAudit::RESOURCE_USER,
+			CAudit::RESOURCE_USER_GROUP, CAudit::RESOURCE_USER_ROLE, CAudit::RESOURCE_VALUE_MAP,
+			CAudit::RESOURCE_HA_NODE, CAudit::RESOURCE_USERDIRECTORY
 		];
 
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
