@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ class CGraphPrototype extends CGraphGeneral {
 
 	protected $tableName = 'graphs';
 	protected $tableAlias = 'g';
-	protected $sortColumns = ['graphid', 'name', 'graphtype'];
+	protected $sortColumns = ['graphid', 'name', 'graphtype', 'discover'];
 
 	public function __construct() {
 		parent::__construct();
@@ -80,6 +80,8 @@ class CGraphPrototype extends CGraphGeneral {
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectGroups'				=> null,
+			'selectHostGroups'			=> null,
+			'selectTemplateGroups'		=> null,
 			'selectTemplates'			=> null,
 			'selectHosts'				=> null,
 			'selectItems'				=> null,
@@ -94,6 +96,8 @@ class CGraphPrototype extends CGraphGeneral {
 		];
 		$options = zbx_array_merge($defOptions, $options);
 
+		$this->checkDeprecatedParam($options, 'selectGroups');
+
 		// editable + PERMISSION CHECK
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
@@ -114,36 +118,6 @@ class CGraphPrototype extends CGraphGeneral {
 					' OR MIN(permission) IS NULL'.
 					' OR MIN(permission)='.PERM_DENY.
 				')';
-			// check permissions by Y min item
-			$sqlParts['where'][] = 'NOT EXISTS ('.
-				'SELECT NULL'.
-				' FROM items i,hosts_groups hgg'.
-					' LEFT JOIN rights r'.
-						' ON r.id=hgg.groupid'.
-							' AND '.dbConditionInt('r.groupid', $userGroups).
-				' WHERE g.ymin_type='.GRAPH_YAXIS_TYPE_ITEM_VALUE.
-					' AND g.ymin_itemid=i.itemid'.
-					' AND i.hostid=hgg.hostid'.
-				' GROUP BY i.hostid'.
-				' HAVING MAX(permission)<'.$permission.
-					' OR MIN(permission) IS NULL'.
-					' OR MIN(permission)='.PERM_DENY.
-				')';
-			// check permissions by Y max item
-			$sqlParts['where'][] = 'NOT EXISTS ('.
-				'SELECT NULL'.
-				' FROM items i,hosts_groups hgg'.
-					' LEFT JOIN rights r'.
-						' ON r.id=hgg.groupid'.
-							' AND '.dbConditionInt('r.groupid', $userGroups).
-				' WHERE g.ymax_type='.GRAPH_YAXIS_TYPE_ITEM_VALUE.
-					' AND g.ymax_itemid=i.itemid'.
-					' AND i.hostid=hgg.hostid'.
-				' GROUP BY i.hostid'.
-				' HAVING MAX(permission)<'.$permission.
-					' OR MIN(permission) IS NULL'.
-					' OR MIN(permission)='.PERM_DENY.
-				')';
 		}
 
 		// groupids
@@ -184,6 +158,7 @@ class CGraphPrototype extends CGraphGeneral {
 			$sqlParts['from']['graphs_items'] = 'graphs_items gi';
 			$sqlParts['from']['items'] = 'items i';
 			$sqlParts['where'][] = dbConditionInt('i.hostid', $options['hostids']);
+			$sqlParts['where'][] = dbConditionInt('i.flags', (array) ZBX_FLAG_DISCOVERY_PROTOTYPE);
 			$sqlParts['where']['gig'] = 'gi.graphid=g.graphid';
 			$sqlParts['where']['igi'] = 'i.itemid=gi.itemid';
 
@@ -293,7 +268,7 @@ class CGraphPrototype extends CGraphGeneral {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($graph = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -324,282 +299,65 @@ class CGraphPrototype extends CGraphGeneral {
 		return $result;
 	}
 
-	protected function inherit($graph, $hostids = null) {
-		$graphTemplates = API::Template()->get([
-			'itemids' => zbx_objectValues($graph['gitems'], 'itemid'),
-			'output' => ['templateid'],
-			'nopermissions' => true
-		]);
-
-		if (empty($graphTemplates)) {
-			return true;
-		}
-
-		$graphTemplate = reset($graphTemplates);
-
-		$chdHosts = API::Host()->get([
-			'templateids' => $graphTemplate['templateid'],
-			'output' => ['hostid', 'host'],
-			'preservekeys' => true,
-			'hostids' => $hostids,
-			'nopermissions' => true,
-			'templated_hosts' => true
-		]);
-
-		$graph = $this->get([
-			'graphids' => $graph['graphid'],
-			'nopermissions' => true,
-			'filter' => ['flags' => null],
-			'selectGraphItems' => API_OUTPUT_EXTEND,
-			'output' => API_OUTPUT_EXTEND
-		]);
-		$graph = reset($graph);
-
-		foreach ($chdHosts as $chdHost) {
-			$tmpGraph = $graph;
-			$tmpGraph['templateid'] = $graph['graphid'];
-
-			$tmpGraph['gitems'] = getSameGraphItemsForHost($tmpGraph['gitems'], $chdHost['hostid']);
-			if (!$tmpGraph['gitems']) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" cannot inherit. No required items on "%2$s".', $tmpGraph['name'], $chdHost['host']));
-			}
-
-			if ($tmpGraph['ymax_itemid'] > 0) {
-				$ymaxItemid = getSameGraphItemsForHost([['itemid' => $tmpGraph['ymax_itemid']]], $chdHost['hostid']);
-				if (!$ymaxItemid) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" cannot inherit. No required items on "%2$s" (Ymax value item).', $tmpGraph['name'], $chdHost['host']));
-				}
-				$ymaxItemid = reset($ymaxItemid);
-				$tmpGraph['ymax_itemid'] = $ymaxItemid['itemid'];
-			}
-			if ($tmpGraph['ymin_itemid'] > 0) {
-				$yminItemid = getSameGraphItemsForHost([['itemid' => $tmpGraph['ymin_itemid']]], $chdHost['hostid']);
-				if (!$yminItemid) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Graph "%1$s" cannot inherit. No required items on "%2$s" (Ymin value item).',
-							$tmpGraph['name'],
-							$chdHost['host']
-					));
-				}
-				$yminItemid = reset($yminItemid);
-				$tmpGraph['ymin_itemid'] = $yminItemid['itemid'];
-			}
-
-			// check if templated graph exists
-			$chdGraphs = $this->get([
-				'filter' => [
-					'templateid' => $tmpGraph['graphid'],
-					'flags' => [ZBX_FLAG_DISCOVERY_PROTOTYPE, ZBX_FLAG_DISCOVERY_NORMAL]
-				],
-				'output' => API_OUTPUT_EXTEND,
-				'selectGraphItems' => API_OUTPUT_EXTEND,
-				'preservekeys' => true,
-				'hostids' => $chdHost['hostid']
-			]);
-
-			if ($chdGraph = reset($chdGraphs)) {
-				if ($tmpGraph['name'] !== $chdGraph['name']) {
-					$graphExists = $this->get([
-						'output' => ['graphid'],
-						'hostids' => $chdHost['hostid'],
-						'filter' => [
-							'name' => $tmpGraph['name'],
-							'flags' => null
-						],
-						'nopermissions' => true,
-						'limit' => 1
-					]);
-					if ($graphExists) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-							'Graph "%1$s" already exists on "%2$s".',
-							$tmpGraph['name'],
-							$chdHost['host']
-						));
-					}
-				}
-				elseif ($chdGraph['flags'] != $tmpGraph['flags']) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('Graph with same name but other type exist.'));
-				}
-
-				$tmpGraph['graphid'] = $chdGraph['graphid'];
-				$this->updateReal($tmpGraph, $chdGraph);
-			}
-			// check if graph with same name and items exists
-			else {
-				$chdGraph = $this->get([
-					'filter' => ['name' => $tmpGraph['name'], 'flags' => null],
-					'output' => API_OUTPUT_EXTEND,
-					'selectGraphItems' => API_OUTPUT_EXTEND,
-					'preservekeys' => true,
-					'nopermissions' => true,
-					'hostids' => $chdHost['hostid']
-				]);
-				if ($chdGraph = reset($chdGraph)) {
-					if ($chdGraph['templateid'] != 0) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" already exists on "%2$s" (inherited from another template).', $tmpGraph['name'], $chdHost['host']));
-					}
-					elseif ($chdGraph['flags'] != $tmpGraph['flags']) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('Graph with same name but other type exist.'));
-					}
-
-					$chdGraphItemItems = [];
-
-					foreach([API::Item(), API::ItemPrototype()] as $api) {
-						$chdGraphItemItems += $api->get([
-							'output' => ['key_', 'hostid', 'itemid'],
-							'itemids' => zbx_objectValues($chdGraph['gitems'], 'itemid'),
-							'preservekeys' => true
-						]);
-					}
-
-					if (count($chdGraph['gitems']) == count($tmpGraph['gitems'])) {
-						foreach ($tmpGraph['gitems'] as $gitem) {
-							foreach ($chdGraph['gitems'] as $chdGraphItem) {
-								$chdGraphItemItem = $chdGraphItemItems[$chdGraphItem['itemid']];
-								if ($gitem['key_'] == $chdGraphItemItem['key_']
-										&& bccomp($chdHost['hostid'], $chdGraphItemItem['hostid']) == 0) {
-									continue 2;
-								}
-							}
-
-							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" already exists on "%2$s" (items are not identical).', $tmpGraph['name'], $chdHost['host']));
-						}
-
-						$tmpGraph['graphid'] = $chdGraph['graphid'];
-						$this->updateReal($tmpGraph, $chdGraph);
-					}
-					else {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Graph "%1$s" already exists on "%2$s" (items are not identical).', $tmpGraph['name'], $chdHost['host']));
-					}
-				}
-				else {
-					$graphid = $this->createReal($tmpGraph);
-					$tmpGraph['graphid'] = $graphid;
-				}
-			}
-			$this->inherit($tmpGraph);
-		}
-	}
-
-	/**
-	 * Inherit template graphs from template to host.
-	 *
-	 * @param array $data
-	 *
-	 * @return bool
-	 */
-	public function syncTemplates($data) {
-		$data['templateids'] = zbx_toArray($data['templateids']);
-		$data['hostids'] = zbx_toArray($data['hostids']);
-
-		$dbLinks = DBSelect(
-			'SELECT ht.hostid,ht.templateid'.
-			' FROM hosts_templates ht'.
-			' WHERE '.dbConditionInt('ht.hostid', $data['hostids']).
-				' AND '.dbConditionInt('ht.templateid', $data['templateids'])
-		);
-		$linkage = [];
-		while ($link = DBfetch($dbLinks)) {
-			if (!isset($linkage[$link['templateid']])) {
-				$linkage[$link['templateid']] = [];
-			}
-			$linkage[$link['templateid']][$link['hostid']] = 1;
-		}
-
-		$graphs = $this->get([
-			'hostids' => $data['templateids'],
-			'preservekeys' => true,
-			'output' => API_OUTPUT_EXTEND,
-			'selectGraphItems' => API_OUTPUT_EXTEND,
-			'selectHosts' => ['hostid'],
-			'filter' => ['flags' => null]
-		]);
-
-		foreach ($graphs as $graph) {
-			foreach ($data['hostids'] as $hostid) {
-				if (isset($linkage[$graph['hosts'][0]['hostid']][$hostid])) {
-					$this->inherit($graph, $hostid);
-				}
-			}
-		}
-
-		return true;
-	}
-
 	/**
 	 * Delete GraphPrototype.
 	 *
 	 * @param array $graphids
-	 * @param bool  $nopermissions
 	 *
 	 * @return array
 	 */
-	public function delete(array $graphids, $nopermissions = false) {
-		if (empty($graphids)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
-		}
+	public function delete(array $graphids) {
+		$this->validateDelete($graphids, $db_graphs);
 
-		$delGraphs = $this->get([
-			'graphids' => $graphids,
-			'editable' => true,
-			'output' => API_OUTPUT_EXTEND,
-			'preservekeys' => true
-		]);
+		CGraphPrototypeManager::delete($graphids);
 
-		if (!$nopermissions) {
-			foreach ($graphids as $graphid) {
-				if (!isset($delGraphs[$graphid])) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
-				}
-				if ($delGraphs[$graphid]['templateid'] != 0) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS, _('Cannot delete templated graphs.'));
-				}
-			}
-		}
-
-		$parentGraphids = $graphids;
-		do {
-			$dbGraphs = DBselect('SELECT g.graphid FROM graphs g WHERE '.dbConditionInt('g.templateid', $parentGraphids));
-			$parentGraphids = [];
-			while ($dbGraph = DBfetch($dbGraphs)) {
-				$parentGraphids[] = $dbGraph['graphid'];
-				$graphids[] = $dbGraph['graphid'];
-			}
-		} while (!empty($parentGraphids));
-
-		$graphids = array_unique($graphids);
-		$createdGraphs = [];
-
-		$dbGraphs = DBselect('SELECT gd.graphid FROM graph_discovery gd WHERE '.dbConditionInt('gd.parent_graphid', $graphids));
-		while ($graph = DBfetch($dbGraphs)) {
-			$createdGraphs[$graph['graphid']] = $graph['graphid'];
-		}
-		if (!empty($createdGraphs)) {
-			$result = API::Graph()->delete($createdGraphs, true);
-			if (!$result) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete graphs created by low level discovery.'));
-			}
-		}
-
-		DB::delete('screens_items', [
-			'resourceid' => $graphids,
-			'resourcetype' => SCREEN_RESOURCE_LLD_GRAPH
-		]);
-
-		DB::delete('graphs', ['graphid' => $graphids]);
-
-		foreach ($delGraphs as $graph) {
-			info(_s('Graph prototype "%s" deleted.', $graph['name']));
-		}
+		$this->addAuditBulk(CAudit::ACTION_DELETE, CAudit::RESOURCE_GRAPH_PROTOTYPE, $db_graphs);
 
 		return ['graphids' => $graphids];
 	}
 
-	protected function createReal($graph) {
-		// mark the graph as a graph prototype
-		$graph['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
+	/**
+	 * Validates the input parameters for the delete() method.
+	 *
+	 * @param array $graphids   [IN/OUT]
+	 * @param array $db_graphs  [OUT]
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateDelete(array &$graphids, array &$db_graphs = null) {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
+		if (!CApiInputValidator::validate($api_input_rules, $graphids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
 
-		return parent::createReal($graph);
+		$db_graphs = $this->get([
+			'output' => ['graphid', 'name', 'templateid'],
+			'graphids' => $graphids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		foreach ($graphids as $graphid) {
+			if (!array_key_exists($graphid, $db_graphs)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
+			}
+
+			if ($db_graphs[$graphid]['templateid'] != 0) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Cannot delete templated graph prototype.'));
+			}
+		}
+	}
+
+	protected function createReal(array &$graphs) {
+		foreach ($graphs as &$graph) {
+			// mark the graph as a graph prototype
+			$graph['flags'] = ZBX_FLAG_DISCOVERY_PROTOTYPE;
+		}
+		unset($graph);
+
+		return parent::createReal($graphs);
 	}
 
 	protected function addRelatedObjects(array $options, array $result) {
@@ -653,7 +411,7 @@ class CGraphPrototype extends CGraphGeneral {
 	 *
 	 * @param array $graphs
 	 */
-	protected function validateCreate(array $graphs) {
+	protected function validateCreate(array &$graphs) {
 		$itemIds = $this->validateItemsCreate($graphs);
 
 		$allowedItems = API::Item()->get([
@@ -716,7 +474,7 @@ class CGraphPrototype extends CGraphGeneral {
 		}
 		unset($graph);
 
-		$itemIds = $this->validateItemsUpdate($graphs);
+		$itemIds = $this->validateItemsUpdate($graphs, $dbGraphs);
 
 		$allowedItems = API::Item()->get([
 			'itemids' => $itemIds,
