@@ -1,4 +1,4 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2022 Zabbix SIA
@@ -46,11 +46,11 @@ class CReport extends CApiService {
 	protected $tableAlias = 'r';
 	protected $sortColumns = ['reportid', 'name', 'status'];
 
-	protected $output_fields = ['reportid', 'userid', 'name', 'description', 'status', 'dashboardid', 'period', 'cycle',
+	private $output_fields = ['reportid', 'userid', 'name', 'description', 'status', 'dashboardid', 'period', 'cycle',
 		'weekdays', 'start_time', 'active_since', 'active_till', 'state', 'lastsent', 'info', 'subject', 'message'
 	];
-	protected $user_output_fields = ['userid', 'access_userid', 'exclude'];
-	protected $usrgrp_output_fields = ['usrgrpid', 'access_userid'];
+	private $user_output_fields = ['userid', 'access_userid', 'exclude'];
+	private $usrgrp_output_fields = ['usrgrpid', 'access_userid'];
 
 	/**
 	 * @param array $options
@@ -134,7 +134,7 @@ class CReport extends CApiService {
 	 * @return array
 	 */
 	public function create(array $reports): array {
-		$this->validateCreate($reports);
+		self::validateCreate($reports);
 
 		$reportids = DB::insert('report', $reports);
 
@@ -143,9 +143,9 @@ class CReport extends CApiService {
 		}
 		unset($report);
 
-		$this->updateParams($reports, __FUNCTION__);
-		$this->updateUsers($reports, __FUNCTION__);
-		$this->updateUserGroups($reports, __FUNCTION__);
+		self::updateParams($reports, __FUNCTION__);
+		self::updateUsers($reports);
+		self::updateUserGroups($reports);
 
 		self::addAuditLog(CAudit::ACTION_ADD, CAudit::RESOURCE_SCHEDULED_REPORT, $reports);
 
@@ -155,9 +155,9 @@ class CReport extends CApiService {
 	/**
 	 * @param array $reports
 	 *
-	 * @throws APIException if no permissions or the input is invalid.
+	 * @throws APIException
 	 */
-	protected function validateCreate(array &$reports): void {
+	private static function validateCreate(array &$reports): void {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
 			'userid' =>				['type' => API_ID, 'default' => self::$userData['userid']],
 			'name' =>				['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('report', 'name')],
@@ -232,39 +232,90 @@ class CReport extends CApiService {
 		}
 		unset($report);
 
-		$this->checkDuplicates(array_column($reports, 'name'));
-		$this->checkDashboards(array_unique(array_column($reports, 'dashboardid')));
-		$this->checkUsers($reports);
-		$this->checkUserGroups($reports);
+		self::checkDuplicates($reports);
+		self::checkDashboards($reports);
+		self::checkUsers($reports);
+		self::checkUserGroups($reports);
 	}
 
 	/**
-	 * Check for duplicated reports.
+	 * @param array      $reports
+	 * @param array|null $db_reports
 	 *
-	 * @param array $names
-	 *
-	 * @throws APIException if report already exists.
+	 * @throws APIException
 	 */
-	protected function checkDuplicates(array $names): void {
-		$db_reports = DB::select('report', [
+	private static function checkDuplicates(array $reports, array $db_reports = null): void {
+		$names = [];
+
+		foreach ($reports as $report) {
+			if (!array_key_exists('name', $report)) {
+				continue;
+			}
+
+			if ($db_reports === null || $report['name'] !== $db_reports[$report['reportid']]['name']) {
+				$names[] = $report['name'];
+			}
+		}
+
+		if (!$names) {
+			return;
+		}
+
+		$duplicates = DB::select('report', [
 			'output' => ['name'],
 			'filter' => ['name' => $names],
 			'limit' => 1
 		]);
 
-		if ($db_reports) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Report "%1$s" already exists.', $db_reports[0]['name']));
+		if ($duplicates) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Report "%1$s" already exists.', $duplicates[0]['name']));
 		}
 	}
 
 	/**
-	 * Check for valid dashboards.
+	 * @param array      $reports
+	 * @param array|null $db_reports
 	 *
-	 * @param array $dashboardids
-	 *
-	 * @throws APIException if dashboard is not valid.
+	 * @throws APIException
 	 */
-	protected function checkDashboards(array $dashboardids): void {
+	private static function checkDashboards(array $reports, array $db_reports = null): void {
+		$dashboardids = [];
+
+		foreach ($reports as $i => $report) {
+			if ($db_reports === null) {
+				$dashboardids[$report['dashboardid']] = true;
+
+				continue;
+			}
+
+			if (!array_key_exists('dashboardid', $report)
+					|| bccomp($report['dashboardid'], $db_reports[$report['reportid']]['dashboardid']) == 0) {
+				continue;
+			}
+
+			// When changing the dashboard, new lists of users and user groups must be provided.
+
+			if (!array_key_exists('users', $report)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1),
+					_s('the parameter "%1$s" is missing', 'users')
+				));
+			}
+
+			if (!array_key_exists('user_groups', $report)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1),
+					_s('the parameter "%1$s" is missing', 'user_groups')
+				));
+			}
+
+			$dashboardids[$report['dashboardid']] = true;
+		}
+
+		if (!$dashboardids) {
+			return;
+		}
+
+		$dashboardids = array_keys($dashboardids);
+
 		$db_dashboards = API::Dashboard()->get([
 			'output' => [],
 			'dashboardids' => $dashboardids,
@@ -281,44 +332,22 @@ class CReport extends CApiService {
 	}
 
 	/**
-	 * Check for valid users.
+	 * @param array      $reports
+	 * @param array|null $db_reports
 	 *
-	 * @param array  $reports
-	 * @param string $reports[]['userid']                          (optional)
-	 * @param string $reports[]['dashboardid']                     (optional)
-	 * @param array  $reports[]['users']                           (optional)
-	 * @param string $reports[]['users'][]['userid']
-	 * @param string $reports[]['users'][]['access_userid']        (optional)
-	 * @param string $reports[]['users'][]['exclude']
-	 * @param array  $reports[]['user_groups']                     (optional)
-	 * @param string $reports[]['user_groups'][]['access_userid']  (optional)
-	 * @param array  $db_reports                                   (optional)
-	 * @param string $db_reports[]['reportid']
-	 * @param string $db_reports[]['userid']
-	 * @param string $db_reports[]['dashboardid']
-	 * @param array  $db_reports[]['users']
-	 * @param string $db_reports[]['users'][]['userid']
-	 * @param string $db_reports[]['users'][]['access_userid']
-	 * @param string $db_reports[]['users'][]['exclude']
-	 * @param array  $db_reports[]['user_groups']
-	 * @param string $db_reports[]['user_groups'][]['access_userid']
-	 *
-	 * @throws APIException if user is not valid.
+	 * @throws APIException
 	 */
-	protected function checkUsers(array $reports, array $db_reports = []): void {
+	private static function checkUsers(array $reports, array $db_reports = null): void {
 		$userids = [];
 
 		foreach ($reports as $report) {
 			$db_report = [];
-			$dashboardid_has_changed = false;
 			$users = array_key_exists('users', $report) ? $report['users'] : [];
 			$user_groups = array_key_exists('user_groups', $report) ? $report['user_groups'] : [];
 
-			if ($db_reports) {
+			if ($db_reports !== null) {
 				$db_report = $db_reports[$report['reportid']];
-				$dashboardid_has_changed = (array_key_exists('dashboardid', $report)
-					&& $report['dashboardid'] != $db_report['dashboardid']
-				);
+
 				if (!array_key_exists('users', $report)) {
 					$users = $db_report['users'];
 				}
@@ -359,13 +388,12 @@ class CReport extends CApiService {
 				}
 
 				foreach ($report['users'] as $user) {
-					if ($dashboardid_has_changed || !array_key_exists($user['userid'], $db_userids)) {
+					if (!array_key_exists($user['userid'], $db_userids)) {
 						$userids[$user['userid']] = true;
 					}
 
 					if (array_key_exists('access_userid', $user) && $user['access_userid'] != 0
-							&& ($dashboardid_has_changed
-								|| !array_key_exists($user['access_userid'], $db_access_userids))) {
+							&& !array_key_exists($user['access_userid'], $db_access_userids)) {
 						$userids[$user['access_userid']] = true;
 					}
 				}
@@ -378,8 +406,7 @@ class CReport extends CApiService {
 
 				foreach ($report['user_groups'] as $usrgrp) {
 					if (array_key_exists('access_userid', $usrgrp) && $usrgrp['access_userid'] != 0
-							&& ($dashboardid_has_changed
-								|| !array_key_exists($usrgrp['access_userid'], $db_access_userids))) {
+							&& !array_key_exists($usrgrp['access_userid'], $db_access_userids)) {
 						$userids[$usrgrp['access_userid']] = true;
 					}
 				}
@@ -408,38 +435,22 @@ class CReport extends CApiService {
 	}
 
 	/**
-	 * Check for valid user groups.
+	 * @param array      $reports
+	 * @param array|null $db_reports
 	 *
-	 * @param array  $reports
-	 * @param string $reports[]['dashboarid']                    (optional)
-	 * @param array  $reports[]['user_groups']                   (optional)
-	 * @param string $reports[]['user_groups'][]['usrgrpid']
-	 * @param array  $db_reports                                 (optional)
-	 * @param string $db_reports[]['reportid']
-	 * @param string $db_reports[]['dashboarid']
-	 * @param array  $db_reports[]['user_groups']
-	 * @param string $db_reports[]['user_groups'][]['usrgrpid']
-	 *
-	 * @throws APIException if user group is not valid.
+	 * @throws APIException
 	 */
-	protected function checkUserGroups(array $reports, array $db_reports = []): void {
+	private static function checkUserGroups(array $reports, array $db_reports = null): void {
 		$usrgrpids = [];
 
 		foreach ($reports as $report) {
 			if (array_key_exists('user_groups', $report) && $report['user_groups']) {
-				$db_usrgrpids = [];
-				$dashboardid_has_changed = false;
-
-				if ($db_reports) {
-					$db_report = $db_reports[$report['reportid']];
-					$db_usrgrpids = array_flip(array_column($db_report['user_groups'], 'usrgrpid'));
-					$dashboardid_has_changed = (array_key_exists('dashboarid', $report)
-						&& $report['dashboarid'] != $db_report['dashboarid']
-					);
-				}
+				$db_usrgrpids = $db_reports !== null
+					? array_flip(array_column($db_reports[$report['reportid']]['user_groups'], 'usrgrpid'))
+					: [];
 
 				foreach ($report['user_groups'] as $usrgrp) {
-					if ($dashboardid_has_changed || !array_key_exists($usrgrp['usrgrpid'], $db_usrgrpids)) {
+					if (!array_key_exists($usrgrp['usrgrpid'], $db_usrgrpids)) {
 						$usrgrpids[$usrgrp['usrgrpid']] = true;
 					}
 				}
@@ -490,9 +501,9 @@ class CReport extends CApiService {
 			DB::update('report', $upd_reports);
 		}
 
-		$this->updateParams($reports, __FUNCTION__);
-		$this->updateUsers($reports, __FUNCTION__, $db_reports);
-		$this->updateUserGroups($reports, __FUNCTION__, $db_reports);
+		self::updateParams($reports, __FUNCTION__);
+		self::updateUsers($reports, $db_reports);
+		self::updateUserGroups($reports, $db_reports);
 
 		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_SCHEDULED_REPORT, $reports, $db_reports);
 
@@ -503,9 +514,9 @@ class CReport extends CApiService {
 	 * @param array      $reports
 	 * @param array|null $db_reports
 	 *
-	 * @throws APIException if no permissions or the input is invalid.
+	 * @throws APIException
 	 */
-	protected function validateUpdate(array &$reports, array &$db_reports = null): void {
+	private function validateUpdate(array &$reports, ?array &$db_reports): void {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
 			'reportid' =>			['type' => API_ID, 'flags' => API_REQUIRED],
 			'userid' =>				['type' => API_ID],
@@ -561,32 +572,9 @@ class CReport extends CApiService {
 				array_diff_key($db_report_active_fields, array_flip(['reportid']));
 		}
 
-		$names = [];
-		$dashboardids = [];
-
 		foreach ($reports as $i => &$report) {
 			$db_report = $db_reports[$report['reportid']];
 
-			if (array_key_exists('name', $report) && $report['name'] !== $db_report['name']) {
-				$names[] = $report['name'];
-			}
-
-			if (array_key_exists('dashboardid', $report) && $report['dashboardid'] != $db_report['dashboardid']) {
-				if (!array_key_exists('users', $report)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1),
-						_s('the parameter "%1$s" is missing', 'users')
-					));
-				}
-
-				if (!array_key_exists('user_groups', $report)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1),
-						_s('the parameter "%1$s" is missing', 'user_groups')
-					));
-				}
-
-				$dashboardids[$report['dashboardid']] = true;
-			}
-
 			if (array_key_exists('cycle', $report) || array_key_exists('weekdays', $report)) {
 				$cycle = array_key_exists('cycle', $report) ? $report['cycle'] : $db_report['cycle'];
 				$weekdays = array_key_exists('weekdays', $report) ? $report['weekdays'] : $db_report['weekdays'];
@@ -640,17 +628,13 @@ class CReport extends CApiService {
 		}
 		unset($report);
 
-		if ($names) {
-			$this->checkDuplicates($names);
-		}
-		if ($dashboardids) {
-			$this->checkDashboards(array_keys($dashboardids));
-		}
+		self::checkDuplicates($reports, $db_reports);
+		self::checkDashboards($reports, $db_reports);
 
 		self::addAffectedObjects($reports, $db_reports);
 
-		$this->checkUsers($reports, $db_reports);
-		$this->checkUserGroups($reports, $db_reports);
+		self::checkUsers($reports, $db_reports);
+		self::checkUserGroups($reports, $db_reports);
 	}
 
 	/**
@@ -659,7 +643,7 @@ class CReport extends CApiService {
 	 * @param array  $reports
 	 * @param string $method
 	 */
-	protected function updateParams(array $reports, string $method): void {
+	private static function updateParams(array $reports, string $method): void {
 		$params_by_name = [
 			'subject' => 'subject',
 			'body' => 'message'
@@ -740,13 +724,10 @@ class CReport extends CApiService {
 	}
 
 	/**
-	 * Update table "report_user" and populate report.users by "reportuserid" property.
-	 *
 	 * @param array      $reports
-	 * @param string     $method
 	 * @param array|null $db_reports
 	 */
-	protected function updateUsers(array &$reports, string $method, array $db_reports = null): void {
+	private static function updateUsers(array &$reports, array $db_reports = null): void {
 		$ins_report_users = [];
 		$upd_report_users = [];
 		$del_reportuserids = [];
@@ -756,7 +737,7 @@ class CReport extends CApiService {
 				continue;
 			}
 
-			$db_report_users = ($method === 'update')
+			$db_report_users = $db_reports !== null
 				? array_column($db_reports[$report['reportid']]['users'], null, 'userid')
 				: [];
 
@@ -813,13 +794,10 @@ class CReport extends CApiService {
 	}
 
 	/**
-	 * Update table "report_usrgrp" and populate report.user_groups by "reportusrgrpid" property.
-	 *
 	 * @param array      $reports
-	 * @param string     $method
 	 * @param array|null $db_reports
 	 */
-	protected function updateUserGroups(array &$reports, string $method, array $db_reports = null): void {
+	private static function updateUserGroups(array &$reports, array $db_reports = null): void {
 		$ins_report_usrgrps = [];
 		$upd_report_usrgrps = [];
 		$del_reportusrgrpids = [];
@@ -829,7 +807,7 @@ class CReport extends CApiService {
 				continue;
 			}
 
-			$db_report_usrgrps = ($method === 'update')
+			$db_report_usrgrps = $db_reports !== null
 				? array_column($db_reports[$report['reportid']]['user_groups'], null, 'usrgrpid')
 				: [];
 
@@ -890,6 +868,8 @@ class CReport extends CApiService {
 	/**
 	 * @param array $reportids
 	 *
+	 * @throws APIException
+	 *
 	 * @return array
 	 */
 	public function delete(array $reportids): array {
@@ -1038,22 +1018,24 @@ class CReport extends CApiService {
 	}
 
 	/**
-	 * Add existing users and user groups to $db_reports, regardless of whether they will be affected by the update.
-	 *
-	 * @static
-	 *
 	 * @param array $reports
 	 * @param array $db_reports
 	 */
 	private static function addAffectedObjects(array $reports, array &$db_reports): void {
+		self::addAffectedUsers($reports, $db_reports);
+		self::addAffectedUserGroups($reports, $db_reports);
+	}
+
+	/**
+	 * @param array $reports
+	 * @param array $db_reports
+	 */
+	private static function addAffectedUsers(array $reports, array &$db_reports): void {
 		$reportids = [];
 
 		foreach ($reports as $report) {
 			$reportids[] = $report['reportid'];
-
-			foreach (['users', 'user_groups'] as $param) {
-				$db_reports[$report['reportid']][$param] = [];
-			}
+			$db_reports[$report['reportid']]['users'] = [];
 		}
 
 		$options = [
@@ -1066,6 +1048,19 @@ class CReport extends CApiService {
 			$db_reports[$db_report_user['reportid']]['users'][$db_report_user['reportuserid']] =
 				array_diff_key($db_report_user, array_flip(['reportid']));
 		}
+	}
+
+	/**
+	 * @param array $reports
+	 * @param array $db_reports
+	 */
+	private static function addAffectedUserGroups(array $reports, array &$db_reports): void {
+		$reportids = [];
+
+		foreach ($reports as $report) {
+			$reportids[] = $report['reportid'];
+			$db_reports[$report['reportid']]['user_groups'] = [];
+		}
 
 		$options = [
 			'output' => ['reportusrgrpid', 'reportid', 'usrgrpid', 'access_userid'],
