@@ -1194,6 +1194,10 @@ class CUser extends CApiService {
 			'values' => ['creator_userid' => null],
 			'where' => ['creator_userid' => $userids]
 		]);
+		DB::update('event_suppress', [
+			'values' => ['userid' => null],
+			'where' => ['userid' => $userids]
+		]);
 
 		$tokenids = DB::select('token', [
 			'output' => [],
@@ -1380,54 +1384,6 @@ class CUser extends CApiService {
 		}
 	}
 
-	/**
-	 * Authenticate a user using LDAP.
-	 *
-	 * The $user array must have the following attributes:
-	 * - username   - user name
-	 * - password   - user password
-	 *
-	 * @param array $user
-	 *
-	 * @return bool
-	 */
-	protected function ldapLogin(array $user) {
-		$cnf = [];
-		$auth_params = [
-			CAuthenticationHelper::LDAP_CASE_SENSITIVE,
-			CAuthenticationHelper::LDAP_CONFIGURED,
-			CAuthenticationHelper::LDAP_HOST,
-			CAuthenticationHelper::LDAP_PORT,
-			CAuthenticationHelper::LDAP_BASE_DN,
-			CAuthenticationHelper::LDAP_BIND_DN,
-			CAuthenticationHelper::LDAP_SEARCH_ATTRIBUTE,
-			CAuthenticationHelper::LDAP_BIND_PASSWORD
-		];
-
-		foreach ($auth_params as $param) {
-			$cnf[str_replace('ldap_', '', $param)] = CAuthenticationHelper::get($param);
-		}
-
-		$ldap_status = (new CFrontendSetup())->checkPhpLdapModule();
-
-		if ($ldap_status['result'] != CFrontendSetup::CHECK_OK) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, $ldap_status['error']);
-		}
-
-		$ldapValidator = new CLdapAuthValidator(['conf' => $cnf]);
-
-		if ($ldapValidator->validate($user)) {
-			return true;
-		}
-		else {
-			self::exception($ldapValidator->isConnectionError()
-					? ZBX_API_ERROR_PARAMETERS
-					: ZBX_API_ERROR_PERMISSIONS,
-				$ldapValidator->getError()
-			);
-		}
-	}
-
 	public function logout($user) {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => []];
 		if (!CApiInputValidator::validate($api_input_rules, $user, '/', $error)) {
@@ -1535,11 +1491,38 @@ class CUser extends CApiService {
 		try {
 			switch ($group_to_auth_map[$db_user['gui_access']]) {
 				case ZBX_AUTH_LDAP:
-					$this->ldapLogin($user);
+					if (CAuthenticationHelper::get(CAuthenticationHelper::LDAP_CONFIGURED) == ZBX_AUTH_LDAP_DISABLED) {
+						self::exception(ZBX_API_ERROR_INTERNAL, _('LDAP authentication is disabled.'));
+					}
+
+					$id = $db_user['userdirectoryid'] != 0
+						? $db_user['userdirectoryid']
+						: CAuthenticationHelper::get(CAuthenticationHelper::LDAP_USERDIRECTORYID);
+					$userdirectory = [];
+
+					if ($id != 0) {
+						$userdirectory = API::UserDirectory()->get([
+							'output' => ['userdirectoryid', 'host', 'port', 'base_dn', 'bind_dn', 'search_attribute',
+								'search_filter', 'start_tls'
+							],
+							'userdirectoryids' => $id
+						]);
+						$userdirectory = reset($userdirectory);
+					}
+
+					if (!$userdirectory) {
+						self::exception(ZBX_API_ERROR_INTERNAL, _('Cannot find user directory for LDAP.'));
+					}
+
+					$userdirectory += [
+						'test_username' => $user['username'],
+						'test_password' => $user['password']
+					];
+					API::UserDirectory()->test($userdirectory);
 					break;
 
 				case ZBX_AUTH_INTERNAL:
-					if (!self::verifyPassword($user['password'], $db_user)) {
+					if (!password_verify($user['password'], $db_user['passwd'])) {
 						self::exception(ZBX_API_ERROR_PERMISSIONS,
 							_('Incorrect user name or password or account is temporarily blocked.')
 						);
@@ -1548,7 +1531,6 @@ class CUser extends CApiService {
 
 				default:
 					self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions for system access.'));
-					break;
 			}
 		}
 		catch (APIException $e) {
@@ -1593,31 +1575,6 @@ class CUser extends CApiService {
 		return array_key_exists('userData', $user) && $user['userData'] ? $db_user : $db_user['sessionid'];
 	}
 
-	/**
-	 * @param string $password           User-specified password.
-	 * @param array  $db_user            Saved user profile.
-	 * @param string $db_user['passwd']  Saved password hash.
-	 * @param int    $db_user['userid']  User id.
-	 *
-	 * @return bool
-	 */
-	private static function verifyPassword($password, array $db_user) {
-		if (strlen($db_user['passwd']) > ZBX_MD5_SIZE) {
-			return password_verify($password, $db_user['passwd']);
-		}
-
-		if (hash_equals($db_user['passwd'], md5($password))) {
-			DB::update('users', [
-				'values' => ['passwd' => password_hash($password, PASSWORD_BCRYPT, ['cost' => ZBX_BCRYPT_COST])],
-				'where' => ['userid' => $db_user['userid']]
-			]);
-
-			return true;
-		}
-
-		return false;
-	}
-
 	/**
 	 * Method is ONLY for internal use!
 	 * Login user by username. Return array with user data.
@@ -1633,9 +1590,7 @@ class CUser extends CApiService {
 	public function loginByUsername($username, $case_sensitive = null, $default_auth = null) {
 		// Check whether the method is called via an API call or from a local php file.
 		if ($case_sensitive === null || $default_auth === null) {
-			return self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Incorrect method "%1$s.%2$s".', 'user', 'loginByUsername')
-			);
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect method "%1$s.%2$s".', 'user', 'loginByUsername'));
 		}
 
 		$user_data = $this->findAccessibleUser($username, $case_sensitive, $default_auth, false);
@@ -1811,25 +1766,65 @@ class CUser extends CApiService {
 		$permissions = [
 			'debug_mode' => GROUP_DEBUG_MODE_DISABLED,
 			'users_status' => GROUP_STATUS_ENABLED,
-			'gui_access' => GROUP_GUI_ACCESS_SYSTEM
+			'gui_access' => GROUP_GUI_ACCESS_SYSTEM,
+			'userdirectoryid' => 0
 		];
 
 		$db_usrgrps = DBselect(
-			'SELECT g.debug_mode,g.users_status,g.gui_access'.
+			'SELECT g.debug_mode,g.users_status,g.gui_access,g.userdirectoryid'.
 			' FROM usrgrp g,users_groups ug'.
 			' WHERE g.usrgrpid=ug.usrgrpid'.
 				' AND ug.userid='.$userid
 		);
 
+		$userdirectoryids = [];
+
 		while ($db_usrgrp = DBfetch($db_usrgrps)) {
 			if ($db_usrgrp['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 				$permissions['debug_mode'] = GROUP_DEBUG_MODE_ENABLED;
 			}
+
 			if ($db_usrgrp['users_status'] == GROUP_STATUS_DISABLED) {
 				$permissions['users_status'] = GROUP_STATUS_DISABLED;
 			}
+
 			if ($db_usrgrp['gui_access'] > $permissions['gui_access']) {
 				$permissions['gui_access'] = $db_usrgrp['gui_access'];
+				$userdirectoryids = [];
+			}
+
+			if ($permissions['gui_access'] === $db_usrgrp['gui_access']
+					&& ($db_usrgrp['gui_access'] == GROUP_GUI_ACCESS_LDAP
+						|| ($db_usrgrp['gui_access'] == GROUP_GUI_ACCESS_SYSTEM
+							&& CAuthenticationHelper::get(CAuthenticationHelper::AUTHENTICATION_TYPE) == ZBX_AUTH_LDAP
+			))) {
+				$userdirectoryids[$db_usrgrp['userdirectoryid']] = true;
+			}
+		}
+
+		if ($userdirectoryids) {
+			if (array_key_exists(0, $userdirectoryids)) {
+				unset($userdirectoryids[0]);
+				$default_userdirectoryid = CAuthenticationHelper::get(CAuthenticationHelper::LDAP_USERDIRECTORYID);
+
+				if ($default_userdirectoryid != 0) {
+					$userdirectoryids[$default_userdirectoryid] = true;
+				}
+			}
+
+			if (count($userdirectoryids) === 1) {
+				$permissions['userdirectoryid'] = array_keys($userdirectoryids)[0];
+			}
+			elseif (count($userdirectoryids) > 1) {
+				$db_userdirectoryids = API::UserDirectory()->get([
+					'output' => [],
+					'userdirectoryids' => array_keys($userdirectoryids),
+					'sortfield' => ['name'],
+					'limit' => 1,
+					'preservekeys' => true
+				]);
+
+				$permissions['userdirectoryid'] = array_keys($db_userdirectoryids)[0];
 			}
 		}
 
@@ -2064,6 +2059,7 @@ class CUser extends CApiService {
 	 * @param array  $permissions
 	 * @param string $permissions['debug_mode']
 	 * @param string $permissions['gui_access']
+	 * @param string $permissions['userdirectoryid']
 	 */
 	private function addExtraFields(array $db_user, array $permissions): array {
 		$db_user['type'] = $this->getUserType($db_user['roleid']);
@@ -2071,6 +2067,7 @@ class CUser extends CApiService {
 
 		$db_user['debug_mode'] = $permissions['debug_mode'];
 		$db_user['gui_access'] = $permissions['gui_access'];
+		$db_user['userdirectoryid'] = $permissions['userdirectoryid'];
 
 		if ($db_user['lang'] === LANG_DEFAULT) {
 			$db_user['lang'] = CSettingsHelper::getGlobal(CSettingsHelper::DEFAULT_LANG);
@@ -2086,10 +2083,10 @@ class CUser extends CApiService {
 	/**
 	 * Sets the default user timezone used by all date/time functions.
 	 *
-	 * @param string $timezone
+	 * @param string|null $timezone
 	 */
-	private function setTimezone(string $timezone): void {
-		if ($timezone !== ZBX_DEFAULT_TIMEZONE) {
+	private function setTimezone(?string $timezone): void {
+		if ($timezone !== null && $timezone !== ZBX_DEFAULT_TIMEZONE) {
 			date_default_timezone_set($timezone);
 		}
 	}
