@@ -77,6 +77,8 @@ class CUserGroup extends CApiService {
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectUsers'				=> null,
 			'selectRights'				=> null,
+			'selectHostGroupRights'		=> null,
+			'selectTemplateGroupRights'	=> null,
 			'selectTagFilters'			=> null,
 			'countOutput'				=> false,
 			'preservekeys'				=> false,
@@ -87,6 +89,8 @@ class CUserGroup extends CApiService {
 
 		$options = zbx_array_merge($defOptions, $options);
 
+		$this->checkDeprecatedParam($options, 'selectRights');
+
 		// permissions
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
 			if (!$options['editable']) {
@@ -182,7 +186,9 @@ class CUserGroup extends CApiService {
 		$ins_usrgrps = [];
 
 		foreach ($usrgrps as $usrgrp) {
-			unset($usrgrp['rights'], $usrgrp['tag_filters'], $usrgrp['users']);
+			unset($usrgrp['hostgroup_rights'], $usrgrp['templategroup_rights'], $usrgrp['tag_filters'],
+				$usrgrp['users']
+			);
 			$ins_usrgrps[] = $usrgrp;
 		}
 		$usrgrpids = DB::insert('usrgrp', $ins_usrgrps);
@@ -208,22 +214,34 @@ class CUserGroup extends CApiService {
 	 */
 	private function validateCreate(array &$usrgrps) {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
-			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
-			'debug_mode' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
-			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED])],
-			'users_status' =>	['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
-			'rights' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
-				'id' =>				['type' => API_ID, 'flags' => API_REQUIRED],
-				'permission' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			'name' =>					['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
+			'debug_mode' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
+			'gui_access' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED]), 'default' => DB::getDefault('usrgrp', 'gui_access')],
+			'users_status' =>			['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
+			'userdirectoryid' =>		['type' => API_MULTIPLE, 'rules' => [
+											['if' => ['field' => 'gui_access', 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_LDAP])], 'type' => API_ID],
+											['else' => true, 'type' => API_UNEXPECTED]
 			]],
-			'tag_filters' =>	['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
-				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
-				'tag' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
-				'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
+			'rights' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'replacement' => 'hostgroup_rights', 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
 			]],
-			'userids' =>		['type' => API_IDS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'uniq' => true],
-			'users' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['userid']], 'fields' => [
-				'userid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			'hostgroup_rights' =>		['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			]],
+			'templategroup_rights' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			]],
+			'tag_filters' =>			['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
+				'groupid' =>				['type' => API_ID, 'flags' => API_REQUIRED],
+				'tag' =>					['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
+				'value' =>					['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
+			]],
+			'userids' =>				['type' => API_IDS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'uniq' => true],
+			'users' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['userid']], 'fields' => [
+				'userid' =>					['type' => API_ID, 'flags' => API_REQUIRED]
 			]]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $usrgrps, '/', $error)) {
@@ -245,8 +263,10 @@ class CUserGroup extends CApiService {
 		$this->checkDuplicates(array_column($usrgrps, 'name'));
 		$this->checkUsers($usrgrps);
 		$this->checkHimself($usrgrps, __FUNCTION__);
+		$this->checkTemplateGroups($usrgrps);
 		$this->checkHostGroups($usrgrps);
 		$this->checkTagFilters($usrgrps);
+		self::checkUserDirectories($usrgrps);
 	}
 
 	/**
@@ -275,47 +295,65 @@ class CUserGroup extends CApiService {
 	 * @throws APIException if the input is invalid.
 	 */
 	private function validateUpdate(array &$usrgrps, array &$db_usrgrps = null) {
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['usrgrpid'], ['name']], 'fields' => [
-			'usrgrpid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
-			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
-			'debug_mode' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
-			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED])],
-			'users_status' =>	['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
-			'rights' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
-				'id' =>				['type' => API_ID, 'flags' => API_REQUIRED],
-				'permission' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
-			]],
-			'tag_filters' =>	['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
-				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
-				'tag' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
-				'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
-			]],
-			'userids' =>		['type' => API_IDS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'uniq' => true],
-			'users' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['userid']], 'fields' => [
-				'userid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
-			]]
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE | API_ALLOW_UNEXPECTED, 'uniq' => [['usrgrpid']], 'fields' => [
+			'usrgrpid' =>	['type' => API_ID, 'flags' => API_REQUIRED]
 		]];
+
 		if (!CApiInputValidator::validate($api_input_rules, $usrgrps, '/', $error)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		// Check user group names.
-		$db_usrgrps = DB::select('usrgrp', [
+		$usrgrpids = array_column($usrgrps, 'usrgrpid');
+		$db_usrgrps = API::UserGroup()->get([
 			'output' => ['usrgrpid', 'name', 'debug_mode', 'gui_access', 'users_status'],
-			'usrgrpids' => array_column($usrgrps, 'usrgrpid'),
+			'usrgrpids' => $usrgrpids,
 			'preservekeys' => true
 		]);
 
+		if (count($usrgrpids) != count($db_usrgrps)) {
+			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		}
+
 		$names = [];
+		$usrgrps = $this->extendObjectsByKey($usrgrps, $db_usrgrps, 'usrgrpid', ['gui_access']);
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+			'usrgrpid' =>				['type' => API_ID],
+			'name' =>					['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
+			'debug_mode' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
+			'gui_access' =>				['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED])],
+			'users_status' =>			['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
+			'userdirectoryid' =>		['type' => API_MULTIPLE, 'rules' => [
+											['if' => ['field' => 'gui_access', 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_LDAP])], 'type' => API_ID],
+											['else' => true, 'type' => API_UNEXPECTED]
+			]],
+			'rights' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'replacement' => 'hostgroup_rights', 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			]],
+			'hostgroup_rights' =>		['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			]],
+			'templategroup_rights' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
+				'id' =>						['type' => API_ID, 'flags' => API_REQUIRED],
+				'permission' =>				['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
+			]],
+			'tag_filters' =>			['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
+				'groupid' =>				['type' => API_ID, 'flags' => API_REQUIRED],
+				'tag' =>					['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
+				'value' =>					['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
+			]],
+			'userids' =>				['type' => API_IDS, 'flags' => API_NORMALIZE | API_DEPRECATED, 'uniq' => true],
+			'users' =>					['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['userid']], 'fields' => [
+				'userid' =>					['type' => API_ID, 'flags' => API_REQUIRED]
+			]]
+		]];
+
+		if (!CApiInputValidator::validate($api_input_rules, $usrgrps, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
 
 		foreach ($usrgrps as &$usrgrp) {
-			// Check if this user group exists.
-			if (!array_key_exists($usrgrp['usrgrpid'], $db_usrgrps)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
 			if (array_key_exists('userids', $usrgrp)) {
 				if (array_key_exists('users', $usrgrp)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Parameter "%1$s" is deprecated.', 'userids'));
@@ -330,6 +368,12 @@ class CUserGroup extends CApiService {
 			if (array_key_exists('name', $usrgrp) && $usrgrp['name'] !== $db_usrgrp['name']) {
 				$names[] = $usrgrp['name'];
 			}
+
+			if (array_key_exists('gui_access', $usrgrp) && $usrgrp['gui_access'] != $db_usrgrp['gui_access']
+					&& $usrgrp['gui_access'] != GROUP_GUI_ACCESS_LDAP
+					&& $usrgrp['gui_access'] != GROUP_GUI_ACCESS_SYSTEM) {
+				$usrgrp['userdirectoryid'] = 0;
+			}
 		}
 		unset($usrgrp);
 
@@ -341,8 +385,10 @@ class CUserGroup extends CApiService {
 		$this->checkUsers($usrgrps);
 		$this->checkHimself($usrgrps, __FUNCTION__, $db_usrgrps);
 		$this->checkUsersWithoutGroups($usrgrps);
+		$this->checkTemplateGroups($usrgrps);
 		$this->checkHostGroups($usrgrps);
 		$this->checkTagFilters($usrgrps);
+		self::checkUserDirectories($usrgrps);
 	}
 
 	/**
@@ -404,10 +450,52 @@ class CUserGroup extends CApiService {
 	}
 
 	/**
-	 * Check for valid host grups.
+	 * Check for valid template groups.
 	 *
 	 * @param array  $usrgrps
-	 * @param array  $usrgrps[]['rights']   (optional)
+	 * @param array  $usrgrps[]['templategroup_rights']  (optional)
+	 *
+	 * @throws APIException
+	 */
+	private function checkTemplateGroups(array $usrgrps) {
+		$groupids = [];
+
+		foreach ($usrgrps as $usrgrp) {
+			if (array_key_exists('templategroup_rights', $usrgrp)) {
+				foreach ($usrgrp['templategroup_rights'] as $right) {
+					$groupids[$right['id']] = true;
+				}
+			}
+		}
+
+		if (!$groupids) {
+			return;
+		}
+
+		$groupids = array_keys($groupids);
+
+		$db_groups = DB::select('hstgrp', [
+			'output' => [],
+			'groupids' => $groupids,
+			'filter' => ['type' => HOST_GROUP_TYPE_TEMPLATE_GROUP],
+			'preservekeys' => true
+		]);
+
+		foreach ($groupids as $groupid) {
+			if (!array_key_exists($groupid, $db_groups)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Template group with ID "%1$s" is not available.', $groupid)
+				);
+			}
+		}
+	}
+
+	/**
+	 * Check for valid host groups.
+	 *
+	 * @param array  $usrgrps
+	 * @param array  $usrgrps[]['hostgroup_rights']  (optional)
+	 * @param array  $usrgrps[]['tag_filters']       (optional)
 	 *
 	 * @throws APIException
 	 */
@@ -415,11 +503,12 @@ class CUserGroup extends CApiService {
 		$groupids = [];
 
 		foreach ($usrgrps as $usrgrp) {
-			if (array_key_exists('rights', $usrgrp)) {
-				foreach ($usrgrp['rights'] as $right) {
+			if (array_key_exists('hostgroup_rights', $usrgrp)) {
+				foreach ($usrgrp['hostgroup_rights'] as $right) {
 					$groupids[$right['id']] = true;
 				}
 			}
+
 			if (array_key_exists('tag_filters', $usrgrp)) {
 				foreach ($usrgrp['tag_filters'] as $tag_filter) {
 					$groupids[$tag_filter['groupid']] = true;
@@ -436,6 +525,7 @@ class CUserGroup extends CApiService {
 		$db_groups = DB::select('hstgrp', [
 			'output' => [],
 			'groupids' => $groupids,
+			'filter' => ['type' => HOST_GROUP_TYPE_HOST_GROUP],
 			'preservekeys' => true
 		]);
 
@@ -662,44 +752,46 @@ class CUserGroup extends CApiService {
 		$upd_rights = [];
 		$del_rightids = [];
 
-		foreach ($usrgrps as &$usrgrp) {
-			if (!array_key_exists('rights', $usrgrp)) {
-				continue;
-			}
+		foreach (['hostgroup_rights', 'templategroup_rights'] as $parameter) {
+			foreach ($usrgrps as &$usrgrp) {
+				if (!array_key_exists($parameter, $usrgrp)) {
+					continue;
+				}
 
-			$db_rights = ($method === 'update')
-				? array_column($db_usrgrps[$usrgrp['usrgrpid']]['rights'], null, 'id')
-				: [];
+				$db_rights = ($method === 'update')
+					? array_column($db_usrgrps[$usrgrp['usrgrpid']][$parameter], null, 'id')
+					: [];
 
-			foreach ($usrgrp['rights'] as &$right) {
-				if (array_key_exists($right['id'], $db_rights)) {
-					$db_right = $db_rights[$right['id']];
-					unset($db_rights[$right['id']]);
+				foreach ($usrgrp[$parameter] as &$right) {
+					if (array_key_exists($right['id'], $db_rights)) {
+						$db_right = $db_rights[$right['id']];
+						unset($db_rights[$right['id']]);
 
-					$right['rightid'] = $db_right['rightid'];
+						$right['rightid'] = $db_right['rightid'];
 
-					$upd_right = DB::getUpdatedValues('rights', $right, $db_right);
+						$upd_right = DB::getUpdatedValues('rights', $right, $db_right);
 
-					if ($upd_right) {
-						$upd_rights[] = [
-							'values' => $upd_right,
-							'where' => ['rightid' => $db_right['rightid']]
+						if ($upd_right) {
+							$upd_rights[] = [
+								'values' => $upd_right,
+								'where' => ['rightid' => $db_right['rightid']]
+							];
+						}
+					}
+					else {
+						$ins_rights[] = [
+							'groupid' => $usrgrp['usrgrpid'],
+							'id' => $right['id'],
+							'permission' => $right['permission']
 						];
 					}
 				}
-				else {
-					$ins_rights[] = [
-						'groupid' => $usrgrp['usrgrpid'],
-						'id' => $right['id'],
-						'permission' => $right['permission']
-					];
-				}
-			}
-			unset($right);
+				unset($right);
 
-			$del_rightids = array_merge($del_rightids, array_column($db_rights, 'rightid'));
+				$del_rightids = array_merge($del_rightids, array_column($db_rights, 'rightid'));
+			}
+			unset($usrgrp);
 		}
-		unset($usrgrp);
 
 		if ($ins_rights) {
 			$rightids = DB::insertBatch('rights', $ins_rights);
@@ -713,19 +805,21 @@ class CUserGroup extends CApiService {
 			DB::delete('rights', ['rightid' => $del_rightids]);
 		}
 
-		foreach ($usrgrps as &$usrgrp) {
-			if (!array_key_exists('rights', $usrgrp)) {
-				continue;
-			}
-
-			foreach ($usrgrp['rights'] as &$right) {
-				if (!array_key_exists('rightid', $right)) {
-					$right['rightid'] = array_shift($rightids);
+		foreach (['hostgroup_rights', 'templategroup_rights'] as $parameter) {
+			foreach ($usrgrps as &$usrgrp) {
+				if (!array_key_exists($parameter, $usrgrp)) {
+					continue;
 				}
+
+				foreach ($usrgrp[$parameter] as &$right) {
+					if (!array_key_exists('rightid', $right)) {
+						$right['rightid'] = array_shift($rightids);
+					}
+				}
+				unset($right);
 			}
-			unset($right);
+			unset($usrgrp);
 		}
-		unset($usrgrp);
 	}
 
 	/**
@@ -1007,48 +1101,9 @@ class CUserGroup extends CApiService {
 			$result = $relationMap->mapMany($result, $dbUsers, 'users');
 		}
 
-		// adding usergroup rights
-		if ($options['selectRights'] !== null && $options['selectRights'] != API_OUTPUT_COUNT) {
-			$db_rights = [];
-			$relationMap = $this->createRelationMap($result, 'groupid', 'rightid', 'rights');
-			$related_ids = $relationMap->getRelatedIds();
-
-			if ($related_ids) {
-				if (is_array($options['selectRights'])) {
-					$pk_field = $this->pk('rights');
-
-					$output_fields = [
-						$pk_field => $this->fieldId($pk_field, 'r')
-					];
-
-					foreach ($options['selectRights'] as $field) {
-						if ($this->hasField($field, 'rights')) {
-							$output_fields[$field] = $this->fieldId($field, 'r');
-						}
-					}
-
-					$output_fields = implode(',', $output_fields);
-				}
-				else {
-					$output_fields = 'r.*';
-				}
-
-				$db_rights = DBfetchArray(DBselect(
-					'SELECT '.$output_fields.
-					' FROM rights r'.
-					' WHERE '.dbConditionInt('r.rightid', $related_ids).
-						((self::$userData['type'] == USER_TYPE_SUPER_ADMIN) ? '' : ' AND r.permission>'.PERM_DENY)
-				));
-				$db_rights = zbx_toHash($db_rights, 'rightid');
-
-				foreach ($db_rights as &$db_right) {
-					unset($db_right['rightid'], $db_right['groupid']);
-				}
-				unset($db_right);
-			}
-
-			$result = $relationMap->mapMany($result, $db_rights, 'rights');
-		}
+		self::addRelatedRights($options, $result, 'selectRights');
+		self::addRelatedRights($options, $result, 'selectHostGroupRights');
+		self::addRelatedRights($options, $result, 'selectTemplateGroupRights');
 
 		// Adding usergroup tag filters.
 		if ($options['selectTagFilters'] !== null && $options['selectTagFilters'] != API_OUTPUT_COUNT) {
@@ -1089,6 +1144,72 @@ class CUserGroup extends CApiService {
 		return $result;
 	}
 
+	/**
+	 * Adds related host or template groups rights requested by "select*" options to the resulting object set.
+	 *
+	 * @param array  $options [IN] Original input options.
+	 * @param array  $result  [IN/OUT] Result output.
+	 * @param string $option  [IN] Possible values:
+	 *                               - "selectGroups" (deprecated);
+	 *                               - "selectHostGroups";
+	 *                               - "selectTemplateGroups".
+	 */
+	private static function addRelatedRights(array $options, array &$result, string $option): void {
+		if ($options[$option] === null || $options[$option] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		switch ($option) {
+			case 'selectRights':
+				$output_tag = 'rights';
+				$types = [HOST_GROUP_TYPE_HOST_GROUP];
+				break;
+
+			case 'selectHostGroupRights':
+				$output_tag = 'hostgroup_rights';
+				$types = [HOST_GROUP_TYPE_HOST_GROUP];
+				break;
+
+			case 'selectTemplateGroupRights':
+				$output_tag = 'templategroup_rights';
+				$types = [HOST_GROUP_TYPE_TEMPLATE_GROUP];
+				break;
+		}
+
+		foreach ($result as &$row) {
+			$row[$output_tag] = [];
+		}
+		unset($row);
+
+		if (is_array($options[$option])) {
+			$output_fields = ['groupid'];
+
+			foreach ($options[$option] as $field) {
+				if (in_array($field, ['id', 'permission'])) {
+					$output_fields[] = $field;
+				}
+			}
+		}
+		else {
+			$output_fields = ['groupid', 'id', 'permission'];
+		}
+
+		$sql = 'SELECT r.'.implode(',r.', $output_fields).
+			' FROM rights r,hstgrp hg'.
+			' WHERE r.id=hg.groupid'.
+				' AND '.dbConditionInt('hg.type', $types).
+				' AND '.dbConditionId('r.groupid', array_keys($result));
+		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
+			$sql .= ' AND '.dbConditionId('r.permission', [PERM_READ_WRITE, PERM_READ]);
+		}
+
+		$db_rights = DBselect($sql);
+
+		while ($db_right = DBfetch($db_rights)) {
+			$result[$db_right['groupid']][$output_tag][] = array_diff_key($db_right, array_flip(['groupid']));
+		}
+	}
+
 	/**
 	 * Add the existing rights, tag_filters and userids to $db_usrgrps whether these are affected by the update.
 	 *
@@ -1098,12 +1219,17 @@ class CUserGroup extends CApiService {
 	 * @param array $db_usrgrps
 	 */
 	private static function addAffectedObjects(array $usrgrps, array &$db_usrgrps): void {
-		$usrgrpids = ['rights' => [], 'tag_filters' => [], 'users' => []];
+		$usrgrpids = ['hostgroup_rights' => [], 'templategroup_rights' => [], 'tag_filters' => [], 'users' => []];
 
 		foreach ($usrgrps as $usrgrp) {
-			if (array_key_exists('rights', $usrgrp)) {
-				$usrgrpids['rights'][] = $usrgrp['usrgrpid'];
-				$db_usrgrps[$usrgrp['usrgrpid']]['rights'] = [];
+			if (array_key_exists('hostgroup_rights', $usrgrp)) {
+				$usrgrpids['hostgroup_rights'][] = $usrgrp['usrgrpid'];
+				$db_usrgrps[$usrgrp['usrgrpid']]['hostgroup_rights'] = [];
+			}
+
+			if (array_key_exists('templategroup_rights', $usrgrp)) {
+				$usrgrpids['templategroup_rights'][] = $usrgrp['usrgrpid'];
+				$db_usrgrps[$usrgrp['usrgrpid']]['templategroup_rights'] = [];
 			}
 
 			if (array_key_exists('tag_filters', $usrgrp)) {
@@ -1117,15 +1243,32 @@ class CUserGroup extends CApiService {
 			}
 		}
 
-		if ($usrgrpids['rights']) {
-			$options = [
-				'output' => ['rightid', 'groupid', 'permission', 'id'],
-				'filter' => ['groupid' => $usrgrpids['rights']]
-			];
-			$db_rights = DBselect(DB::makeSql('rights', $options));
+		if ($usrgrpids['hostgroup_rights']) {
+			$db_rights = DBselect(
+				'SELECT r.rightid,r.groupid,r.permission,r.id'.
+				' FROM rights r,hstgrp hg'.
+				' WHERE r.id=hg.groupid'.
+					' AND '.dbConditionId('r.groupid', $usrgrpids['hostgroup_rights']).
+					' AND '.dbConditionInt('hg.type', [HOST_GROUP_TYPE_HOST_GROUP])
+			);
 
 			while ($db_right = DBfetch($db_rights)) {
-				$db_usrgrps[$db_right['groupid']]['rights'][$db_right['rightid']] =
+				$db_usrgrps[$db_right['groupid']]['hostgroup_rights'][$db_right['rightid']] =
+					array_diff_key($db_right, array_flip(['groupid']));
+			}
+		}
+
+		if ($usrgrpids['templategroup_rights']) {
+			$db_rights = DBselect(
+				'SELECT r.rightid,r.groupid,r.permission,r.id'.
+				' FROM rights r,hstgrp hg'.
+				' WHERE r.id=hg.groupid'.
+					' AND '.dbConditionId('r.groupid', $usrgrpids['templategroup_rights']).
+					' AND '.dbConditionInt('hg.type', [HOST_GROUP_TYPE_TEMPLATE_GROUP])
+			);
+
+			while ($db_right = DBfetch($db_rights)) {
+				$db_usrgrps[$db_right['groupid']]['templategroup_rights'][$db_right['rightid']] =
 					array_diff_key($db_right, array_flip(['groupid']));
 			}
 		}
@@ -1156,4 +1299,37 @@ class CUserGroup extends CApiService {
 			}
 		}
 	}
+
+	/**
+	 * Check if user directories exist.
+	 *
+	 * @param array  $usrgrps
+	 * @param string $usrgrps['userdirectoryid']
+	 *
+	 * @throws APIException
+	 */
+	private static function checkUserDirectories(array $usrgrps): void {
+		$userdirectoryids = array_filter(array_column($usrgrps, 'userdirectoryid'));
+
+		if (!$userdirectoryids) {
+			return;
+		}
+
+		$db_userdirectories = API::UserDirectory()->get([
+			'output' => [],
+			'userdirectoryids' => $userdirectoryids,
+			'preservekeys' => true
+		]);
+
+		foreach ($usrgrps as $i => $usrgrp) {
+			if (array_key_exists('userdirectoryid', $usrgrp) && $usrgrp['userdirectoryid'] != 0
+					&& !array_key_exists($usrgrp['userdirectoryid'], $db_userdirectories)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Invalid parameter "%1$s": %2$s.', '/'.($i + 1).'/userdirectoryid',
+						_('referred object does not exist')
+					)
+				);
+			}
+		}
+	}
 }
