@@ -1,4 +1,4 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
 ** Copyright (C) 2001-2022 Zabbix SIA
@@ -57,9 +57,12 @@ final class CItemData {
 			'perf_instance.discovery[object]',
 			'perf_instance_en.discovery[object]',
 			'proc.cpu.util[<name>,<user>,<type>,<cmdline>,<mode>,<zone>]',
+			'proc.get[<name>,<user>,<cmdline>,<mode>]',
 			'proc.mem[<name>,<user>,<mode>,<cmdline>,<memtype>]',
 			'proc.num[<name>,<user>,<state>,<cmdline>,<zone>]',
 			'proc_info[process,<attribute>,<type>]',
+			'registry.data[key,<value name>]',
+			'registry.get[key,<mode>,<name regexp>]',
 			'sensor[device,sensor,<mode>]',
 			'service.info[service,<param>]',
 			'services[<type>,<state>,<exclude>]',
@@ -156,9 +159,12 @@ final class CItemData {
 			'perf_instance.discovery[object]',
 			'perf_instance_en.discovery[object]',
 			'proc.cpu.util[<name>,<user>,<type>,<cmdline>,<mode>,<zone>]',
+			'proc.get[<name>,<user>,<cmdline>,<mode>]',
 			'proc.mem[<name>,<user>,<mode>,<cmdline>,<memtype>]',
 			'proc.num[<name>,<user>,<state>,<cmdline>,<zone>]',
 			'proc_info[process,<attribute>,<type>]',
+			'registry.data[key,<value name>]',
+			'registry.get[key,<mode>,<name regexp>]',
 			'sensor[device,sensor,<mode>]',
 			'service.info[service,<param>]',
 			'services[<type>,<state>,<exclude>]',
@@ -224,18 +230,31 @@ final class CItemData {
 			'net.tcp.service[service,<ip>,<port>]',
 			'net.udp.service.perf[service,<ip>,<port>]',
 			'net.udp.service[service,<ip>,<port>]',
+			'vmware.alarms.get[<url>]',
 			'vmware.cl.perfcounter[<url>,<id>,<path>,<instance>]',
+			'vmware.cluster.alarms.get[<url>,<id>]',
 			'vmware.cluster.discovery[<url>]',
+			'vmware.cluster.property[<url>,<id>,<prop>]',
 			'vmware.cluster.status[<url>,<name>]',
+			'vmware.cluster.tags.get[<url>,<id>]',
+			'vmware.datastore.alarms.get[<url>,<uuid>]',
 			'vmware.datastore.discovery[<url>]',
 			'vmware.datastore.hv.list[<url>,<datastore>]',
+			'vmware.datastore.property[<url>,<uuid>,<prop>]',
 			'vmware.datastore.read[<url>,<datastore>,<mode>]',
 			'vmware.datastore.size[<url>,<datastore>,<mode>]',
+			'vmware.datastore.tags.get[<url>,<uuid>]',
 			'vmware.datastore.write[<url>,<datastore>,<mode>]',
+			'vmware.dc.alarms.get[<url>,<id>]',
 			'vmware.dc.discovery[<url>]',
+			'vmware.dc.tags.get[<url>,<id>]',
+			'vmware.dvswitch.discovery[<url>]',
+			'vmware.dvswitch.fetchports.get[<url>,<filter>,<mode>]',
 			'vmware.eventlog[<url>,<mode>]',
 			'vmware.fullname[<url>]',
+			'vmware.hv.alarms.get[<url>,<uuid>]',
 			'vmware.hv.cluster.name[<url>,<uuid>]',
+			'vmware.hv.connectionstate[<url>,<uuid>]',
 			'vmware.hv.cpu.usage.perf[<url>,<uuid>]',
 			'vmware.hv.cpu.usage[<url>,<uuid>]',
 			'vmware.hv.cpu.utilization[<url>,<uuid>]',
@@ -254,23 +273,34 @@ final class CItemData {
 			'vmware.hv.hw.cpu.threads[<url>,<uuid>]',
 			'vmware.hv.hw.memory[<url>,<uuid>]',
 			'vmware.hv.hw.model[<url>,<uuid>]',
+			'vmware.hv.hw.sensors.get[<url>,<uuid>]',
+			'vmware.hv.hw.serialnumber[<url>,<uuid>]',
 			'vmware.hv.hw.uuid[<url>,<uuid>]',
 			'vmware.hv.hw.vendor[<url>,<uuid>]',
 			'vmware.hv.maintenance[<url>,<uuid>]',
 			'vmware.hv.memory.size.ballooned[<url>,<uuid>]',
 			'vmware.hv.memory.used[<url>,<uuid>]',
+			'vmware.hv.net.if.discovery[<url>,<uuid>]',
 			'vmware.hv.network.in[<url>,<uuid>,<mode>]',
+			'vmware.hv.network.linkspeed[<url>,<uuid>,<ifname>]',
 			'vmware.hv.network.out[<url>,<uuid>,<mode>]',
 			'vmware.hv.perfcounter[<url>,<uuid>,<path>,<instance>]',
 			'vmware.hv.power[<url>,<uuid>,<max>]',
+			'vmware.hv.property[<url>,<uuid>,<prop>]',
 			'vmware.hv.sensor.health.state[<url>,<uuid>]',
 			'vmware.hv.sensors.get[<url>,<uuid>]',
 			'vmware.hv.status[<url>,<uuid>]',
+			'vmware.hv.tags.get[<url>,<uuid>]',
 			'vmware.hv.uptime[<url>,<uuid>]',
 			'vmware.hv.version[<url>,<uuid>]',
 			'vmware.hv.vm.num[<url>,<uuid>]',
+			'vmware.rp.cpu.usage[<url>,<rpid>]',
+			'vmware.rp.memory[<url>,<rpid>,<mode>]',
 			'vmware.version[<url>]',
+			'vmware.vm.alarms.get[<url>,<uuid>]',
+			'vmware.vm.attribute[<url>,<uuid>,<name>]',
 			'vmware.vm.cluster.name[<url>,<uuid>]',
+			'vmware.vm.consolidationneeded[<url>,<uuid>]',
 			'vmware.vm.cpu.latency[<url>,<uuid>]',
 			'vmware.vm.cpu.num[<url>,<uuid>]',
 			'vmware.vm.cpu.readiness[<url>,<uuid>,<instance>]',
@@ -299,6 +329,9 @@ final class CItemData {
 			'vmware.vm.net.if.usage[<url>,<uuid>,<instance>]',
 			'vmware.vm.perfcounter[<url>,<uuid>,<path>,<instance>]',
 			'vmware.vm.powerstate[<url>,<uuid>]',
+			'vmware.vm.property[<url>,<uuid>,<prop>]',
+			'vmware.vm.snapshot.get[<url>,<uuid>]',
+			'vmware.vm.state[<url>,<uuid>]',
 			'vmware.vm.storage.committed[<url>,<uuid>]',
 			'vmware.vm.storage.readoio[<url>,<uuid>,<instance>]',
 			'vmware.vm.storage.totalreadlatency[<url>,<uuid>,<instance>]',
@@ -306,6 +339,8 @@ final class CItemData {
 			'vmware.vm.storage.uncommitted[<url>,<uuid>]',
 			'vmware.vm.storage.unshared[<url>,<uuid>]',
 			'vmware.vm.storage.writeoio[<url>,<uuid>,<instance>]',
+			'vmware.vm.tags.get[<url>,<uuid>]',
+			'vmware.vm.tools[<url>,<uuid>,<mode>]',
 			'vmware.vm.uptime[<url>,<uuid>]',
 			'vmware.vm.vfs.dev.discovery[<url>,<uuid>]',
 			'vmware.vm.vfs.dev.read[<url>,<uuid>,<instance>,<mode>]',
@@ -319,11 +354,6 @@ final class CItemData {
 		],
 		ITEM_TYPE_INTERNAL => [
 			'zabbix[boottime]',
-			'zabbix[history]',
-			'zabbix[history_log]',
-			'zabbix[history_str]',
-			'zabbix[history_text]',
-			'zabbix[history_uint]',
 			'zabbix[host,,items]',
 			'zabbix[host,,items_unsupported]',
 			'zabbix[host,,maintenance]',
@@ -344,8 +374,6 @@ final class CItemData {
 			'zabbix[stats,<ip>,<port>,queue,<from>,<to>]',
 			'zabbix[stats,<ip>,<port>]',
 			'zabbix[tcache, cache, <parameter>]',
-			'zabbix[trends]',
-			'zabbix[trends_uint]',
 			'zabbix[triggers]',
 			'zabbix[uptime]',
 			'zabbix[vcache,buffer,<mode>]',
@@ -369,6 +397,36 @@ final class CItemData {
 		]
 	];
 
+	/**
+	 * Generates an array used to generate item type lookups in the form: item_type => [key_names].
+	 *
+	 * @return array
+	 */
+	public static function getKeysByItemType(): array {
+		$keys_by_type = self::KEYS_BY_TYPE;
+		$keys_by_type_shortened = [];
+
+		foreach ($keys_by_type as $item_type => $available_keys) {
+			$available_keys_shortened = [];
+
+			foreach ($available_keys as $key) {
+				$param_start_pos = strpos($key, '[');
+
+				if ($param_start_pos !== false) {
+					$key = substr($key, 0, $param_start_pos);
+				}
+
+				if (!array_key_exists($key, $available_keys_shortened)) {
+					$available_keys_shortened[] = $key;
+				}
+			}
+
+			$keys_by_type_shortened[$item_type] = $available_keys_shortened;
+		}
+
+		return $keys_by_type_shortened;
+	}
+
 	/**
 	 * Returns items available for the given item type as an array of key => details.
 	 *
@@ -381,12 +439,12 @@ final class CItemData {
 	}
 
 	/**
-	 * Generate an array used for item type lookups in the form: key_name => value_type.
+	 * Generates an array used to generate item type of information lookups in the form: key_name => value_type.
 	 * Value type set to null if key return type varies based on parameters.
 	 *
 	 * @return array
 	 */
-	public static function getTypeSuggestionsByKey(): array {
+	public static function getValueTypeByKey(): array {
 		$type_suggestions = [];
 		$keys = self::get();
 
@@ -430,9 +488,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_DB_MONITOR => [
 					'js-item-delay-label',
@@ -448,19 +505,17 @@ final class CItemData {
 					'password',
 					'js-item-sql-query-label',
 					'js-item-sql-query-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
 					['id' => 'key', 'defaultValue' => $data['is_discovery_rule']
 						? ZBX_DEFAULT_KEY_DB_MONITOR_DISCOVERY
 						: ZBX_DEFAULT_KEY_DB_MONITOR
-					]
+					],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_DEPENDENT => [
 					'js-item-master-item-label',
 					'js-item-master-item-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_EXTERNAL => [
 					'js-item-interface-label',
@@ -471,9 +526,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_HTTPAGENT => [
 					'js-item-url-label',
@@ -522,13 +576,12 @@ final class CItemData {
 					'js-item-interface-label',
 					'js-item-interface-field',
 					'interfaceid',
-					'js-item-trends-label',
-					'js-item-trends-field',
 					'js-item-allow-traps-label',
 					'js-item-allow-traps-field',
 					'allow_traps',
 					'trapper_hosts',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_INTERNAL => [
 					'js-item-delay-label',
@@ -536,9 +589,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_IPMI => [
 					'js-item-interface-label',
@@ -552,9 +604,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_JMX => [
 					'js-item-interface-label',
@@ -574,9 +625,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_SCRIPT => [
 					'js-item-parameters-label',
@@ -590,9 +640,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_SIMPLE => [
 					'js-item-delay-label',
@@ -609,9 +658,8 @@ final class CItemData {
 					'js-item-password-label',
 					'js-item-password-field',
 					'password',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_SNMP => [
 					'js-item-interface-label',
@@ -625,17 +673,15 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_SNMPTRAP => [
 					'js-item-interface-label',
 					'js-item-interface-field',
 					'interfaceid',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_SSH => [
 					'js-item-interface-label',
@@ -657,10 +703,9 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
 					'params_script',
-					['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_SSH]
+					['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_SSH],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_TELNET => [
 					'js-item-interface-label',
@@ -679,18 +724,16 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
 					'params_script',
-					['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_TELNET]
+					['id' => 'key', 'defaultValue' => ZBX_DEFAULT_KEY_TELNET],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_TRAPPER => [
 					'js-item-trapper-hosts-label',
 					'js-item-trapper-hosts-field',
 					'trapper_hosts',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_ZABBIX => [
 					'js-item-interface-label',
@@ -701,9 +744,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-label',
-					'js-item-trends-field',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				],
 				ITEM_TYPE_ZABBIX_ACTIVE => [
 					'js-item-delay-label',
@@ -711,9 +753,8 @@ final class CItemData {
 					'delay',
 					'js-item-flex-intervals-label',
 					'js-item-flex-intervals-field',
-					'js-item-trends-field',
-					'js-item-trends-label',
-					['id' => 'key', 'defaultValue' => '']
+					['id' => 'key', 'defaultValue' => ''],
+					['id' => 'value_type', 'defaultValue' => '']
 				]
 			],
 			// Ids to toggle when the field 'authtype' is changed.
@@ -1001,6 +1042,10 @@ final class CItemData {
 				'description' => _('Process CPU utilization percentage. Returns float'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
 			],
+			'proc.get[<name>,<user>,<cmdline>,<mode>]' => [
+				'description' => _('List of OS processes with attributes. Returns JSON array'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'proc.mem[<name>,<user>,<mode>,<cmdline>,<memtype>]' => [
 				'description' => _('Memory used by process in bytes. Returns integer'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1013,6 +1058,14 @@ final class CItemData {
 				'description' => _('Various information about specific process(es). Returns float'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
 			],
+			'registry.data[key,<value name>]' => [
+				'description' => _('Value data for value name in Windows Registry key.'),
+				'value_type' => null
+			],
+			'registry.get[key,<mode>,<name regexp>]' => [
+				'description' => _('List of Windows Registry values or keys located at given key. Returns JSON.'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'sensor[device,sensor,<mode>]' => [
 				'description' => _('Hardware sensor reading. Returns float'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
@@ -1221,18 +1274,38 @@ final class CItemData {
 				'description' => _('Virtual space size in bytes or in percentage from total. Returns integer for bytes; float for percentage'),
 				'value_type' => null
 			],
+			'vmware.alarms.get[<url>]' => [
+				'description' => _('VMware virtual center alarms data, returns JSON, <url> - VMware service URL'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.cl.perfcounter[<url>,<id>,<path>,<instance>]' => [
 				'description' => _('VMware cluster performance counter, <url> - VMware service URL, <id> - VMware cluster id, <path> - performance counter path, <instance> - performance counter instance'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
 			],
+			'vmware.cluster.alarms.get[<url>,<id>]' => [
+				'description' => _('VMware cluster alarms data, returns JSON, <url> - VMware service URL, <id> - VMware cluster id'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.cluster.discovery[<url>]' => [
 				'description' => _('Discovery of VMware clusters, <url> - VMware service URL. Returns JSON'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
 			],
+			'vmware.cluster.property[<url>,<id>,<prop>]' => [
+				'description' => _('VMware cluster property, <url> - VMware service URL, <id> - VMware cluster id, <prop> - property path'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.cluster.status[<url>,<name>]' => [
 				'description' => _('VMware cluster status, <url> - VMware service URL, <name> - VMware cluster name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.cluster.tags.get[<url>,<id>]' => [
+				'description' => _('VMware cluster tags array, <url> - VMware service URL, <id> - VMware cluster id'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.datastore.alarms.get[<url>,<uuid>]' => [
+				'description' => _('VMware datastore alarms data, returns JSON, <url> - VMware service URL, <uuid> - VMware datastore name'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.datastore.discovery[<url>]' => [
 				'description' => _('Discovery of VMware datastores, <url> - VMware service URL. Returns JSON'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
@@ -1241,6 +1314,10 @@ final class CItemData {
 				'description' => _('VMware datastore hypervisors list, <url> - VMware service URL, <datastore> - datastore name'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
 			],
+			'vmware.datastore.property[<url>,<uuid>,<prop>]' => [
+				'description' => _('VMware datastore property, <url> - VMware service URL, <uuid> - datastore name, <prop> - property path'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.datastore.read[<url>,<datastore>,<mode>]' => [
 				'description' => _('VMware datastore read statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
@@ -1249,12 +1326,32 @@ final class CItemData {
 				'description' => _('VMware datastore capacity statistics in bytes or in percentage from total. Returns integer for bytes; float for percentage'),
 				'value_type' => null
 			],
+			'vmware.datastore.tags.get[<url>,<uuid>]' => [
+				'description' => _('VMware datastore tags array, <url> - VMware service URL, <uuid> - VMware datastore uuid. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.datastore.write[<url>,<datastore>,<mode>]' => [
 				'description' => _('VMware datastore write statistics, <url> - VMware service URL, <datastore> - datastore name, <mode> - latency/maxlatency - average or maximum'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
 			],
+			'vmware.dc.alarms.get[<url>,<id>]' => [
+				'description' => _('VMware datacenter alarms data, returns JSON, <url> - VMware service URL, <id> - VMware datacenter id'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.dc.discovery[<url>]' => [
-				'description' => _('VMware datacenters and their IDs. Returns JSON'),
+				'description' => _('VMware datacenters and their IDs, <url> - VMware service URL. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.dc.tags.get[<url>,<id>]' => [
+				'description' => _('VMware datacenter tags array, <url> - VMware service URL, <id> - VMware datacenter id. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.dvswitch.discovery[<url>]' => [
+				'description' => _('VMware Distributed Virtual Switch, <url> - VMware service URL. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.dvswitch.fetchports.get[<url>,<filter>,<mode>]' => [
+				'description' => _('VMware FetchDVPorts wrapper, <url> - VMware service URL, <filter> - vmware data object DistributedVirtualSwitchPortCriteria, <mode> - state(default)/full. Returns JSON'),
 				'value_type' => ITEM_VALUE_TYPE_TEXT
 			],
 			'vmware.eventlog[<url>,<mode>]' => [
@@ -1265,10 +1362,18 @@ final class CItemData {
 				'description' => _('VMware service full name, <url> - VMware service URL'),
 				'value_type' => ITEM_VALUE_TYPE_STR
 			],
+			'vmware.hv.alarms.get[<url>,<uuid>]' => [
+				'description' => _('VMware hypervisor alarms data, returns JSON, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.hv.cluster.name[<url>,<uuid>]' => [
 				'description' => _('VMware hypervisor cluster name, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_STR
 			],
+			'vmware.hv.connectionstate[<url>,<uuid>]' => [
+				'description' => _('VMware hypervisor connection state, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.hv.cpu.usage.perf[<url>,<uuid>]' => [
 				'description' => _('CPU usage as a percentage during the interval, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
@@ -1341,6 +1446,14 @@ final class CItemData {
 				'description' => _('VMware hypervisor model, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_STR
 			],
+			'vmware.hv.hw.sensors.get[<url>,<uuid>]' => [
+				'description' => _('VMware hypervisor sensors value, <url> - VMware service URL, <uuid> - VMware hypervisor host name. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.hv.hw.serialnumber[<url>,<uuid>]' => [
+				'description' => _('VMware hypervisor serialnumber, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.hv.hw.uuid[<url>,<uuid>]' => [
 				'description' => _('VMware hypervisor BIOS UUID, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_STR
@@ -1361,10 +1474,18 @@ final class CItemData {
 				'description' => _('VMware hypervisor used memory size, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.hv.net.if.discovery[<url>,<uuid>]' => [
+				'description' => _('Discovery of VMware hypervisor network interfaces, <url> - VMware service URL, <uuid> - VMware hypervisor. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.hv.network.in[<url>,<uuid>,<mode>]' => [
 				'description' => _('VMware hypervisor network input statistics, <url> - VMware service URL, <uuid> - VMware hypervisor host name, <mode> - bps'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.hv.network.linkspeed[<url>,<uuid>,<ifname>]' => [
+				'description' => _('VMware hypervisor network interface speed, <url> - VMware service URL, <uuid> - VMware hypervisor host name, <ifname> - interface name'),
+				'value_type' => ITEM_VALUE_TYPE_UINT64
+			],
 			'vmware.hv.network.out[<url>,<uuid>,<mode>]' => [
 				'description' => _('VMware hypervisor network output statistics, <url> - VMware service URL, <uuid> - VMware hypervisor host name, <mode> - bps'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1377,6 +1498,10 @@ final class CItemData {
 				'description' => _('Power usage , <url> - VMware service URL, <uuid> - VMware hypervisor host name, <max> - Maximum allowed power usage'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
 			],
+			'vmware.hv.property[<url>,<uuid>,<prop>]' => [
+				'description' => _('VMware hypervisor property , <url> - VMware service URL, <uuid> - VMware hypervisor host name, <prop> - property path'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.hv.sensor.health.state[<url>,<uuid>]' => [
 				'description' => _('VMware hypervisor health state rollup sensor, <url> - VMware service URL, <uuid> - VMware hypervisor host name. Returns 0 - gray; 1 - green; 2 - yellow; 3 - red'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1389,6 +1514,10 @@ final class CItemData {
 				'description' => _('VMware hypervisor status, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => null
 			],
+			'vmware.hv.tags.get[<url>,<uuid>]' => [
+				'description' => _('VMware hypervisor tags array, <url> - VMware service URL, <uuid> - VMware hypervisor host name. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
 			'vmware.hv.uptime[<url>,<uuid>]' => [
 				'description' => _('VMware hypervisor uptime, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1401,14 +1530,34 @@ final class CItemData {
 				'description' => _('Number of virtual machines on VMware hypervisor, <url> - VMware service URL, <uuid> - VMware hypervisor host name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.rp.cpu.usage[<url>,<rpid>]' => [
+				'description' => _('CPU usage in hertz during the interval on VMware Resource Pool, <url> - VMware service URL, <rpid> - VMware resource pool id'),
+				'value_type' => ITEM_VALUE_TYPE_UINT64
+			],
+			'vmware.rp.memory[<url>,<rpid>,<mode>]' => [
+				'description' => _('Memory metrics of VMware Resource Pool, <url> - VMware service URL, <rpid> - VMware resource pool id, <mode> - consumed(default)/ballooned/overhead memory'),
+				'value_type' => ITEM_VALUE_TYPE_UINT64
+			],
 			'vmware.version[<url>]' => [
 				'description' => _('VMware service version, <url> - VMware service URL'),
 				'value_type' => ITEM_VALUE_TYPE_STR
 			],
+			'vmware.vm.alarms.get[<url>,<uuid>]' => [
+				'description' => _('VMware virtual machine alarms data, returns JSON, <url> - VMware service URL, <uuid> - VMware virtual machine name'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.vm.attribute[<url>,<uuid>,<name>]' => [
+				'description' => _('VMware virtual machine custom attribute value, <url> - VMware service URL, <uuid> - VMware virtual machine host name, <name> - custom attribute name'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.vm.cluster.name[<url>,<uuid>]' => [
 				'description' => _('VMware virtual machine name, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
 				'value_type' => ITEM_VALUE_TYPE_STR
 			],
+			'vmware.vm.consolidationneeded[<url>,<uuid>]' => [
+				'description' => _('VMware virtual machine disk requires consolidation, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.vm.cpu.latency[<url>,<uuid>]' => [
 				'description' => _('Percent of time the virtual machine is unable to run because it is contending for access to the physical CPU(s), <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
 				'value_type' => ITEM_VALUE_TYPE_FLOAT
@@ -1521,6 +1670,18 @@ final class CItemData {
 				'description' => _('VMware virtual machine power state, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.vm.property[<url>,<uuid>,<prop>]' => [
+				'description' => _('VMware virtual machine property, <url> - VMware service URL, <uuid> - VMware virtual machine host name, <prop> - property path'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.vm.snapshot.get[<url>,<uuid>]' => [
+				'description' => _('VMware virtual machine snapshot state, <url> - VMware service URL, <uuid> - VMware virtual machine host name. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.vm.state[<url>,<uuid>]' => [
+				'description' => _('VMware virtual machine state, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.vm.storage.committed[<url>,<uuid>]' => [
 				'description' => _('VMware virtual machine committed storage space, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
 				'value_type' => null
@@ -1549,6 +1710,14 @@ final class CItemData {
 				'description' => _('Average number of outstanding write requests to the virtual disk during the collection interval, <url> - VMware service URL, <uuid> - VMware virtual machine host name, <instance> - disk device instance'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
+			'vmware.vm.tags.get[<url>,<uuid>]' => [
+				'description' => _('VMware virtual machine tags array, <url> - VMware service URL, <uuid> - VMware virtual machine host name. Returns JSON'),
+				'value_type' => ITEM_VALUE_TYPE_TEXT
+			],
+			'vmware.vm.tools[<url>,<uuid>,<mode>]' => [
+				'description' => _('VMware virtual machine tools state, <url> - VMware service URL, <uuid> - VMware virtual machine host name, <mode> - version or status'),
+				'value_type' => ITEM_VALUE_TYPE_STR
+			],
 			'vmware.vm.uptime[<url>,<uuid>]' => [
 				'description' => _('VMware virtual machine uptime, <url> - VMware service URL, <uuid> - VMware virtual machine host name'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1605,26 +1774,6 @@ final class CItemData {
 				'description' => _('Startup time of Zabbix server, Unix timestamp.'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
-			'zabbix[history]' => [
-				'description' => _('Number of values stored in table HISTORY.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
-			'zabbix[history_log]' => [
-				'description' => _('Number of values stored in table HISTORY_LOG.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
-			'zabbix[history_str]' => [
-				'description' => _('Number of values stored in table HISTORY_STR.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
-			'zabbix[history_text]' => [
-				'description' => _('Number of values stored in table HISTORY_TEXT.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
-			'zabbix[history_uint]' => [
-				'description' => _('Number of values stored in table HISTORY_UINT.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
 			'zabbix[host,,items]' => [
 				'description' => _('Number of enabled items on the host.'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
@@ -1638,7 +1787,7 @@ final class CItemData {
 				'value_type' => null
 			],
 			'zabbix[host,<type>,available]' => [
-				'description' => _('Returns availability of a particular type of checks on the host. Value of this item corresponds to availability icons in the host list. Valid types are: agent, snmp, ipmi, jmx.'),
+				'description' => _('Returns availability of a particular type of checks on the host. Value of this item corresponds to availability icons in the host list. Valid types are: agent, active_agent, snmp, ipmi, jmx.'),
 				'value_type' => null
 			],
 			'zabbix[host,discovery,interfaces]' => [
@@ -1682,7 +1831,7 @@ final class CItemData {
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
 			'zabbix[queue,<from>,<to>]' => [
-				'description' => _('Number of items in the queue which are delayed by from to to seconds, inclusive.'),
+				'description' => _('Number of items in the queue which are delayed by from to seconds, inclusive.'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
 			],
 			'zabbix[rcache,<cache>,<mode>]' => [
@@ -1705,14 +1854,6 @@ final class CItemData {
 				'description' => _('Trend function cache statistics. Valid parameters are: all, hits, phits, misses, pmisses, items, pitems and requests.'),
 				'value_type' => null
 			],
-			'zabbix[trends]' => [
-				'description' => _('Number of values stored in table TRENDS.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
-			'zabbix[trends_uint]' => [
-				'description' => _('Number of values stored in table TRENDS_UINT.'),
-				'value_type' => ITEM_VALUE_TYPE_UINT64
-			],
 			'zabbix[triggers]' => [
 				'description' => _('Number of triggers in Zabbix database.'),
 				'value_type' => ITEM_VALUE_TYPE_UINT64
