@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,6 +21,11 @@
 
 class CPieGraphDraw extends CGraphDraw {
 
+	const DEFAULT_HEADER_PADDING_TOP = 30;
+
+	const GRAPH_WIDTH_MIN = 20;
+	const GRAPH_HEIGHT_MIN = 20;
+
 	public function __construct($type = GRAPH_TYPE_PIE) {
 		parent::__construct($type);
 		$this->background = false;
@@ -35,7 +40,20 @@ class CPieGraphDraw extends CGraphDraw {
 	/* PRE CONFIG: ADD / SET / APPLY
 	/********************************************************************************************************/
 	public function addItem($itemid, $calc_fnc = CALC_FNC_AVG, $color = null, $type = null) {
-		$items = CMacrosResolverHelper::resolveItemNames([get_item_by_itemid($itemid)]);
+		$items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_', 'units', 'value_type', 'valuemapid', 'history', 'trends'],
+			'itemids' => [$itemid],
+			'webitems' => true
+		]);
+
+		if (!$items) {
+			$items = API::ItemPrototype()->get([
+				'output' => ['itemid', 'hostid', 'name', 'key_', 'units', 'value_type', 'valuemapid', 'history',
+					'trends'
+				],
+				'itemids' => [$itemid]
+			]);
+		}
 
 		$this->items[$this->num] = reset($items);
 
@@ -50,15 +68,6 @@ class CPieGraphDraw extends CGraphDraw {
 		$this->num++;
 	}
 
-	public function set3DAngle($angle = 70) {
-		if (is_numeric($angle) && $angle < 85 && $angle > 10) {
-			$this->angle3d = (int) $angle;
-		}
-		else {
-			$this->angle3d = 70;
-		}
-	}
-
 	public function switchPie3D($type = false) {
 		if ($type) {
 			$this->type = $type;
@@ -84,31 +93,6 @@ class CPieGraphDraw extends CGraphDraw {
 		return $this->type;
 	}
 
-	public function switchPieExploded($type) {
-		if ($type) {
-			$this->type = $type;
-		}
-		else {
-			switch ($this->type) {
-				case GRAPH_TYPE_EXPLODED:
-					$this->type = GRAPH_TYPE_PIE;
-					break;
-				case GRAPH_TYPE_3D_EXPLODED:
-					$this->type = GRAPH_TYPE_3D;
-					break;
-				case GRAPH_TYPE_3D:
-					$this->type = GRAPH_TYPE_3D_EXPLODED;
-					break;
-				case GRAPH_TYPE_PIE:
-					$this->type = GRAPH_TYPE_EXPLODED;
-					break;
-				default:
-					$this->type = GRAPH_TYPE_PIE;
-			}
-		}
-		return $this->type;
-	}
-
 	protected function calc3dheight($height) {
 		$this->graphheight3d = (int) ($height / 20);
 	}
@@ -117,27 +101,29 @@ class CPieGraphDraw extends CGraphDraw {
 		$count *= $this->exploderad;
 		$anglemid = (int) (($anglestart + $angleend) / 2);
 
-		$y+= round($count * sin(deg2rad($anglemid)));
-		$x+= round($count * cos(deg2rad($anglemid)));
+		$y += round($count * sin(deg2rad($anglemid)));
+		$x += round($count * cos(deg2rad($anglemid)));
 
-		return [$x, $y];
+		return [(int) $x, (int) $y];
 	}
 
 	protected function calcExplodedRadius($sizeX, $sizeY, $count) {
 		$count *= $this->exploderad * 2;
 		$sizeX -= $count;
 		$sizeY -= $count;
-		return [$sizeX, $sizeY];
+
+		return [(int) $sizeX, (int) $sizeY];
 	}
 
 	protected function calc3DAngle($sizeX, $sizeY) {
 		$sizeY *= GRAPH_3D_ANGLE / 90;
-		return [$sizeX, round($sizeY)];
+
+		return [$sizeX, (int) round($sizeY)];
 	}
 
 	protected function selectData() {
 		$this->data = [];
-		$now = time(null);
+		$now = time();
 
 		if (isset($this->stime)) {
 			$this->from_time = $this->stime;
@@ -161,39 +147,38 @@ class CPieGraphDraw extends CGraphDraw {
 			$history = Manager::History()->getLastValues($lastValueItems);
 		}
 
-		$config = select_config();
 		$items = [];
 
 		for ($i = 0; $i < $this->num; $i++) {
-			$item = get_item_by_itemid($this->items[$i]['itemid']);
-			$type = $this->items[$i]['calc_type'];
+			$item = $this->items[$i];
+
 			$from_time = $this->from_time;
 			$to_time = $this->to_time;
 
 			$to_resolve = [];
 
 			// Override item history setting with housekeeping settings, if they are enabled in config.
-			if ($config['hk_history_global']) {
-				$item['history'] = timeUnitToSeconds($config['hk_history']);
+			if (CHousekeepingHelper::get(CHousekeepingHelper::HK_HISTORY_GLOBAL)) {
+				$item['history'] = timeUnitToSeconds(CHousekeepingHelper::get(CHousekeepingHelper::HK_HISTORY));
 			}
 			else {
 				$to_resolve[] = 'history';
 			}
 
-			if ($config['hk_trends_global']) {
-				$item['trends'] = timeUnitToSeconds($config['hk_trends']);
+			if (CHousekeepingHelper::get(CHousekeepingHelper::HK_TRENDS_GLOBAL)) {
+				$item['trends'] = timeUnitToSeconds(CHousekeepingHelper::get(CHousekeepingHelper::HK_TRENDS));
 			}
 			else {
 				$to_resolve[] = 'trends';
 			}
 
-			// Otherwise, resolve user macro and parse the string. If successfull, convert to seconds.
+			// Otherwise, resolve user macro and parse the string. If successful, convert to seconds.
 			if ($to_resolve) {
 				$item = CMacrosResolverHelper::resolveTimeUnitMacros([$item], $to_resolve)[0];
 
 				$simple_interval_parser = new CSimpleIntervalParser();
 
-				if (!$config['hk_history_global']) {
+				if (!CHousekeepingHelper::get(CHousekeepingHelper::HK_HISTORY_GLOBAL)) {
 					if ($simple_interval_parser->parse($item['history']) != CParser::PARSE_SUCCESS) {
 						show_error_message(_s('Incorrect value for field "%1$s": %2$s.', 'history',
 							_('invalid history storage period')
@@ -203,7 +188,7 @@ class CPieGraphDraw extends CGraphDraw {
 					$item['history'] = timeUnitToSeconds($item['history']);
 				}
 
-				if (!$config['hk_trends_global']) {
+				if (!CHousekeepingHelper::get(CHousekeepingHelper::HK_TRENDS_GLOBAL)) {
 					if ($simple_interval_parser->parse($item['trends']) != CParser::PARSE_SUCCESS) {
 						show_error_message(_s('Incorrect value for field "%1$s": %2$s.', 'trends',
 							_('invalid trend storage period')
@@ -214,19 +199,19 @@ class CPieGraphDraw extends CGraphDraw {
 				}
 			}
 
-			$this->data[$this->items[$i]['itemid']][$type]['last'] = isset($history[$item['itemid']])
+			$this->data[$this->items[$i]['itemid']]['last'] = isset($history[$item['itemid']])
 				? $history[$item['itemid']][0]['value'] : null;
-			$this->data[$this->items[$i]['itemid']][$type]['shift_min'] = 0;
-			$this->data[$this->items[$i]['itemid']][$type]['shift_max'] = 0;
-			$this->data[$this->items[$i]['itemid']][$type]['shift_avg'] = 0;
+			$this->data[$this->items[$i]['itemid']]['shift_min'] = 0;
+			$this->data[$this->items[$i]['itemid']]['shift_max'] = 0;
+			$this->data[$this->items[$i]['itemid']]['shift_avg'] = 0;
 
-			$item['source'] = ($item['trends'] == 0
-					|| ($item['history'] * SEC_PER_DAY > time() - ($from_time + $this->period / 2)))
-					? 'history' : 'trends';
+			$item['source'] = ($item['trends'] == 0 || ($item['history'] > time() - ($from_time + $this->period / 2)))
+				? 'history'
+				: 'trends';
 			$items[] = $item;
 		}
 
-		$results = Manager::History()->getGraphAggregation($items, $from_time, $to_time);
+		$results = Manager::History()->getGraphAggregationByWidth($items, $from_time, $to_time);
 		$i = 0;
 
 		foreach ($items as $item) {
@@ -235,10 +220,10 @@ class CPieGraphDraw extends CGraphDraw {
 				$this->dataFrom = $result['source'];
 
 				foreach ($result['data'] as $row) {
-					$this->data[$item['itemid']][$type]['min'] = $row['min'];
-					$this->data[$item['itemid']][$type]['max'] = $row['max'];
-					$this->data[$item['itemid']][$type]['avg'] = $row['avg'];
-					$this->data[$item['itemid']][$type]['clock'] = $row['clock'];
+					$this->data[$item['itemid']]['min'] = $row['min'];
+					$this->data[$item['itemid']]['max'] = $row['max'];
+					$this->data[$item['itemid']]['avg'] = $row['avg'];
+					$this->data[$item['itemid']]['clock'] = $row['clock'];
 				}
 				unset($result);
 			}
@@ -246,7 +231,7 @@ class CPieGraphDraw extends CGraphDraw {
 				$this->dataFrom = $item['source'];
 			}
 
-			switch ($this->items[$i]['calc_fnc']) {
+			switch ($item['calc_fnc']) {
 				case CALC_FNC_MIN:
 					$fncName = 'min';
 					break;
@@ -261,18 +246,18 @@ class CPieGraphDraw extends CGraphDraw {
 					$fncName = 'avg';
 			}
 
-			$item_value = empty($this->data[$item['itemid']][$type][$fncName])
+			$item_value = empty($this->data[$item['itemid']][$fncName])
 				? 0
-				: abs($this->data[$item['itemid']][$type][$fncName]);
+				: abs($this->data[$item['itemid']][$fncName]);
 
-			if ($type == GRAPH_ITEM_SUM) {
+			if ($item['calc_type'] == GRAPH_ITEM_SUM) {
 				$this->background = $i;
 				$graph_sum = $item_value;
 			}
 
 			$this->sum += $item_value;
 
-			$convertedUnit = strlen(convert_units([
+			$convertedUnit = strlen(convertUnits([
 				'value' => $item_value,
 				'units' => $item['units']
 			]));
@@ -297,7 +282,7 @@ class CPieGraphDraw extends CGraphDraw {
 		$functionNameXShift = 0;
 
 		foreach ($this->items as $item) {
-			$name = $displayHostName ? $item['hostname'].': '.$item['name_expanded'] : $item['name_expanded'];
+			$name = $displayHostName ? $item['hostname'].NAME_DELIMITER.$item['name'] : $item['name'];
 			$dims = imageTextSize($fontSize, 0, $name);
 
 			if ($dims['width'] > $functionNameXShift) {
@@ -307,6 +292,7 @@ class CPieGraphDraw extends CGraphDraw {
 
 		// display items
 		$i = 0;
+		$top_padding = $this->with_vertical_padding ? 10 : -(static::DEFAULT_TOP_BOTTOM_PADDING / 2);
 
 		foreach ($this->items as $item) {
 			$color = $this->getColor($item['color'], 0);
@@ -331,13 +317,13 @@ class CPieGraphDraw extends CGraphDraw {
 					$fncRealName = _('avg');
 			}
 
-			if (isset($this->data[$item['itemid']][$item['calc_type']])
-					&& isset($this->data[$item['itemid']][$item['calc_type']][$fncName])) {
-				$dataValue = $this->data[$item['itemid']][$item['calc_type']][$fncName];
+			if (isset($this->data[$item['itemid']])
+					&& isset($this->data[$item['itemid']][$fncName])) {
+				$dataValue = $this->data[$item['itemid']][$fncName];
 				$proc = ($this->sum == 0) ? 0 : ($dataValue * 100) / $this->sum;
 
 				$strValue = sprintf(_('Value').': %s ('.(round($proc) != round($proc, 2) ? '%0.2f' : '%0.0f').'%%)',
-					convert_units([
+					convertUnits([
 						'value' => $dataValue,
 						'units' => $this->items[$i]['units']
 					]),
@@ -360,7 +346,7 @@ class CPieGraphDraw extends CGraphDraw {
 				$this->shiftXleft + 15,
 				$this->sizeY + $shiftY + 14 * $i + 5,
 				$this->getColor($this->graphtheme['textcolor'], 0),
-				$displayHostName ? $item['hostname'].': '.$item['name_expanded'] : $item['name_expanded']
+				$displayHostName ? $item['hostname'].NAME_DELIMITER.$item['name'] : $item['name']
 			);
 
 			// function name
@@ -400,9 +386,9 @@ class CPieGraphDraw extends CGraphDraw {
 			imagefilledrectangle(
 				$this->im,
 				$shiftX - 10,
-				$this->shiftY + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 14 * $i,
 				$shiftX,
-				$this->shiftY + 10 + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 10 + 14 * $i,
 				$color
 			);
 
@@ -410,9 +396,9 @@ class CPieGraphDraw extends CGraphDraw {
 			imagerectangle(
 				$this->im,
 				$shiftX - 10,
-				$this->shiftY + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 14 * $i,
 				$shiftX,
-				$this->shiftY + 10 + 10 + 14 * $i,
+				$this->shiftY + $top_padding + 10 + 14 * $i,
 				$this->GetColor('Black No Alpha')
 			);
 
@@ -422,7 +408,7 @@ class CPieGraphDraw extends CGraphDraw {
 				$fontSize,
 				0,
 				$shiftX + 5,
-				$this->shiftY + 10 + 14 * $i + 10,
+				$this->shiftY + $top_padding + 14 * $i + 10,
 				$this->getColor($this->graphtheme['textcolor'], 0),
 				$strValue
 			);
@@ -463,19 +449,21 @@ class CPieGraphDraw extends CGraphDraw {
 		if ($this->type == GRAPH_TYPE_EXPLODED) {
 			list($sizeX, $sizeY) = $this->calcExplodedRadius($sizeX, $sizeY, count($values));
 		}
-		else {
-			$sizeX = (int) $sizeX * 0.95;
-			$sizeY = (int) $sizeY * 0.95;
-		}
 
-		$xc = $x = (int) $this->sizeX / 2 + $this->shiftXleft;
-		$yc = $y = (int) $this->sizeY / 2 + $this->shiftY;
+		$xc = $x = (int) ($this->sizeX / 2) + $this->shiftXleft;
+		$yc = $y = (int) ($this->sizeY / 2) + $this->shiftY;
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -543,15 +531,21 @@ class CPieGraphDraw extends CGraphDraw {
 
 		list($sizeX, $sizeY) = $this->calc3DAngle($sizeX, $sizeY);
 
-		$xc = $x = (int) $this->sizeX / 2 + $this->shiftXleft;
-		$yc = $y = (int) $this->sizeY / 2 + $this->shiftY;
+		$xc = $x = (int) ($this->sizeX / 2) + $this->shiftXleft;
+		$yc = $y = (int) ($this->sizeY / 2) + $this->shiftY;
 
 		// bottom angle line
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -559,6 +553,7 @@ class CPieGraphDraw extends CGraphDraw {
 			if ($this->type == GRAPH_TYPE_3D_EXPLODED) {
 				list($x, $y) = $this->calcExplodedCenter($anglestart, $angleend, $xc, $yc, count($values));
 			}
+
 			imagefilledarc(
 				$this->im,
 				$x,
@@ -588,7 +583,12 @@ class CPieGraphDraw extends CGraphDraw {
 		for ($i = $this->graphheight3d; $i > 0; $i--) {
 			$anglestart = 0;
 			$angleend = 0;
+
 			foreach ($values as $item => $value) {
+				if ($value == 0) {
+					continue;
+				}
+
 				$angleend += (int) (360 * $value / $sum) + 1;
 				$angleend = ($angleend > 360) ? 360 : $angleend;
 
@@ -620,9 +620,15 @@ class CPieGraphDraw extends CGraphDraw {
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -663,10 +669,13 @@ class CPieGraphDraw extends CGraphDraw {
 			$start_time = microtime(true);
 		}
 		set_image_header();
+		$this->calculateTopPadding();
 
 		$this->selectData();
+		if (hasErrorMessages()) {
+			show_messages();
+		}
 
-		$this->shiftY = 30;
 		$this->shiftYLegend = 20;
 		$this->shiftXleft = 10;
 		$this->shiftXright = 0;
@@ -679,27 +688,39 @@ class CPieGraphDraw extends CGraphDraw {
 
 		if ($this->drawLegend == 1) {
 			$this->sizeX -= $this->shiftXleft + $this->shiftXright + $this->shiftlegendright;
-			$this->sizeY -= $this->shiftY + $this->shiftYLegend + 12 * $this->num + 8;
+			$this->sizeY -= $this->shiftY + $this->shiftYLegend + 14 * $this->num + 8;
 		}
-		else {
+		elseif ($this->with_vertical_padding) {
 			$this->sizeX -= $this->shiftXleft * 2;
 			$this->sizeY -= $this->shiftY * 2;
 		}
 
+		if (!$this->with_vertical_padding) {
+			if ($this->drawLegend == 1) {
+				// Increase size of graph by sum of: 8px legend font size and 5px legend item bottom shift.
+				$this->sizeY += 13;
+			}
+			else {
+				// Remove y shift if only graph is rendered (no labels, header, vertical paddings).
+				$this->shiftY = 0;
+			}
+		}
+
 		$this->sizeX = min($this->sizeX, $this->sizeY);
 		$this->sizeY = min($this->sizeX, $this->sizeY);
 
+		if ($this->sizeX + $this->shiftXleft > $this->fullSizeX) {
+			$this->sizeX = $this->fullSizeX - $this->shiftXleft - $this->shiftXleft;
+			$this->sizeY = min($this->sizeX, $this->sizeY);
+		}
+
 		$this->calc3dheight($this->sizeY);
 
 		$this->exploderad = (int) $this->sizeX / 100;
 		$this->exploderad3d = (int) $this->sizeX / 60;
 
-		if (function_exists('ImageColorExactAlpha') && function_exists('ImageCreateTrueColor') && @imagecreatetruecolor(1, 1)) {
-			$this->im = imagecreatetruecolor($this->fullSizeX, $this->fullSizeY);
-		}
-		else {
-			$this->im = imagecreate($this->fullSizeX, $this->fullSizeY);
-		}
+		$this->im = imagecreatetruecolor($this->fullSizeX, $this->fullSizeY);
+
 		$this->initColors();
 		$this->drawRectangle();
 		$this->drawHeader();
@@ -707,9 +728,7 @@ class CPieGraphDraw extends CGraphDraw {
 		// for each metric
 		$values = [];
 		for ($i = 0; $i < $this->num; $i++) {
-			$type = $this->items[$i]['calc_type'];
-
-			$data = &$this->data[$this->items[$i]['itemid']][$type];
+			$data = &$this->data[$this->items[$i]['itemid']];
 
 			if (!isset($data)) {
 				continue;
@@ -730,9 +749,9 @@ class CPieGraphDraw extends CGraphDraw {
 					$fncName = 'avg';
 			}
 
-			$values[$i] = empty($this->data[$this->items[$i]['itemid']][$type][$fncName])
+			$values[$i] = empty($this->data[$this->items[$i]['itemid']][$fncName])
 				? 0
-				: abs($this->data[$this->items[$i]['itemid']][$type][$fncName]);
+				: abs($this->data[$this->items[$i]['itemid']][$fncName]);
 		}
 
 		switch ($this->type) {
@@ -755,16 +774,14 @@ class CPieGraphDraw extends CGraphDraw {
 
 		if ($debug_mode) {
 			$str = sprintf('%0.2f', microtime(true) - $start_time);
-			$str = _s('Data from %1$s. Generated in %2$s sec.', $this->dataFrom, $str);
-			$str_size = imageTextSize(6, 0, $str);
 			imageText(
 				$this->im,
 				6,
-				0,
-				$this->fullSizeX - $str_size['width'] - 5,
+				90,
+				$this->fullSizeX - 2,
 				$this->fullSizeY - 5,
 				$this->getColor('Gray'),
-				$str
+				_s('Data from %1$s. Generated in %2$s sec.', $this->dataFrom, $str)
 			);
 		}
 
