@@ -310,14 +310,14 @@ class CMacrosResolverGeneral {
 	 * @param bool   $types['functionids']                        Extract numeric macros. For example, "{12345}".
 	 * @param bool   $types['expr_macros']                        Extract expression macros.
 	 *                                                              For example, "{?func(/host/key, param)}".
-	 * @param bool   $types['expr_macros_host']                   Extract expression macros with with the ability to
+	 * @param bool   $types['expr_macros_host']                   Extract expression macros with the ability to
 	 *                                                              specify a {HOST.HOST} macro or an empty host name
 	 *                                                              instead of a hostname.
 	 *                                                              For example,
 	 *                                                                "{?func(/host/key, param)}",
 	 *                                                                "{?func(/{HOST.HOST}/key, param)}",
 	 *                                                                "{?func(//key, param)}".
-	 * @param bool   $types['expr_macros_host_n']                 Extract expression macros with with the ability to
+	 * @param bool   $types['expr_macros_host_n']                 Extract expression macros with the ability to
 	 *                                                              specify a {HOST.HOST<1-9>} macro or an empty host
 	 *                                                              name instead of a hostname.
 	 *                                                              For example,
@@ -649,14 +649,14 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	private function getItemKeyParameters($params_raw) {
+	public static function getItemKeyParameters($params_raw) {
 		$item_key_parameters = [];
 
 		foreach ($params_raw as $param_raw) {
 			switch ($param_raw['type']) {
 				case CItemKey::PARAM_ARRAY:
 					$item_key_parameters = array_merge($item_key_parameters,
-						$this->getItemKeyParameters($param_raw['parameters'])
+						self::getItemKeyParameters($param_raw['parameters'])
 					);
 					break;
 
@@ -686,7 +686,7 @@ class CMacrosResolverGeneral {
 
 		$item_key_parameters = [];
 		if ($item_key_parser->parse($key) == CParser::PARSE_SUCCESS) {
-			$item_key_parameters = $this->getItemKeyParameters($item_key_parser->getParamsRaw());
+			$item_key_parameters = self::getItemKeyParameters($item_key_parser->getParamsRaw());
 		}
 
 		return self::extractMacros($item_key_parameters, $types);
@@ -732,14 +732,14 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return string
 	 */
-	private function resolveItemKeyParamsMacros($key_chain, array $params_raw, array $macros, array $types) {
+	private static function resolveItemKeyParamsMacros($key_chain, array $params_raw, array $macros, array $types) {
 		foreach (array_reverse($params_raw) as $param_raw) {
 			$param = $param_raw['raw'];
 			$forced = false;
 
 			switch ($param_raw['type']) {
 				case CItemKey::PARAM_ARRAY:
-					$param = $this->resolveItemKeyParamsMacros($param, $param_raw['parameters'], $macros, $types);
+					$param = self::resolveItemKeyParamsMacros($param, $param_raw['parameters'], $macros, $types);
 					break;
 
 				case CItemKey::PARAM_QUOTED:
@@ -748,13 +748,7 @@ class CMacrosResolverGeneral {
 					// break; is not missing here
 
 				case CItemKey::PARAM_UNQUOTED:
-					$matched_macros = $this->getMacroPositions($param, $types);
-
-					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-						$param = substr_replace($param, $macros[$macro], $pos, strlen($macro));
-					}
-
-					$param = quoteItemKeyParam($param, $forced);
+					$param = quoteItemKeyParam(strtr($param, $macros), $forced);
 					break;
 			}
 
@@ -773,11 +767,11 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return string
 	 */
-	protected function resolveItemKeyMacros($key, array $macros, array $types) {
+	public static function resolveItemKeyMacros($key, array $macros, array $types) {
 		$item_key_parser = new CItemKey();
 
 		if ($item_key_parser->parse($key) == CParser::PARSE_SUCCESS) {
-			$key = $this->resolveItemKeyParamsMacros($key, $item_key_parser->getParamsRaw(), $macros, $types);
+			$key = self::resolveItemKeyParamsMacros($key, $item_key_parser->getParamsRaw(), $macros, $types);
 		}
 
 		return $key;
@@ -788,34 +782,30 @@ class CMacrosResolverGeneral {
 	 *
 	 * @param string $function	a trigger function
 	 * @param array  $macros	the list of macros (['{<MACRO>}' => '<value>', ...])
-	 * @param array  $types		the types of macros (see getMacroPositions() for more details)
 	 *
 	 * @return string
 	 */
-	protected function resolveFunctionMacros($function, array $macros, array $types) {
+	protected function resolveFunctionMacros($function, array $macros) {
 		$hist_function_parser = new CHistFunctionParser(['usermacros' => true, 'lldmacros' => true]);
 
 		if ($hist_function_parser->parse($function) == CParser::PARSE_SUCCESS) {
-			foreach (array_reverse($hist_function_parser->getParameters()) as $parameter) {
-				$param = $parameter['match'];
-				$forced = false;
+			foreach (array_reverse($hist_function_parser->getParameters(), true) as $i => $parameter) {
+				switch ($parameter['type']) {
+					case CHistFunctionParser::PARAM_TYPE_PERIOD:
+					case CHistFunctionParser::PARAM_TYPE_UNQUOTED:
+					case CHistFunctionParser::PARAM_TYPE_QUOTED:
+						$param = strtr($hist_function_parser->getParam($i), $macros);
 
-				if ($parameter['type'] == CHistFunctionParser::PARAM_TYPE_QUOTED) {
-					$param = CHistFunctionParser::unquoteParam($param);
-					$forced = true;
+						if ($parameter['type'] != CHistFunctionParser::PARAM_TYPE_PERIOD) {
+							$param = CExpressionParser::quoteString($param, true,
+								$parameter['type'] == CHistFunctionParser::PARAM_TYPE_QUOTED
+							);
+						}
+
+						$function = substr_replace($function, $param, $parameter['pos'], $parameter['length']);
+
+						break;
 				}
-
-				$matched_macros = $this->getMacroPositions($param, $types);
-
-				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-					$param = substr_replace($param, $macros[$macro], $pos, strlen($macro));
-				}
-
-				if ($parameter['type'] != CHistFunctionParser::PARAM_TYPE_PERIOD) {
-					$param = quoteFunctionParam($param, $forced);
-				}
-
-				$function = substr_replace($function, $param, $parameter['pos'], $parameter['length']);
 			}
 		}
 
