@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,11 +33,18 @@ class CLdapAuthValidator extends CValidator {
 		'search_attribute' => null
 	];
 
+	/**
+	 * Switch between more detailed or more generic error message mode.
+	 *
+	 * @var type
+	 */
+	protected $detailed_errors = false;
+
 	/**
 	 * Checks if the given user name and password are valid.
 	 *
 	 * The $value array must have the following attributes:
-	 * - user       - user name
+	 * - username   - user name
 	 * - password   - password
 	 *
 	 * @param array $value
@@ -49,7 +56,7 @@ class CLdapAuthValidator extends CValidator {
 
 		$ldap = new CLdap($this->conf);
 
-		$status = $ldap->checkPass($value['user'], $value['password']);
+		$status = $ldap->checkPass($value['username'], $value['password']);
 
 		if (!$status) {
 			$this->setError($ldap->error);
@@ -70,13 +77,16 @@ class CLdapAuthValidator extends CValidator {
 			CLdap::ERR_SERVER_UNAVAILABLE => _('Cannot connect to LDAP server.'),
 			CLdap::ERR_BIND_FAILED => _('Cannot bind to LDAP server.'),
 			CLdap::ERR_BIND_ANON_FAILED => _('Cannot bind anonymously to LDAP server.'),
-			CLdap::ERR_USER_NOT_FOUND => _('Login name or password is incorrect.'),
 			CLdap::ERR_OPT_PROTOCOL_FAILED => _('Setting LDAP protocol failed.'),
 			CLdap::ERR_OPT_TLS_FAILED => _('Starting TLS failed.'),
 			CLdap::ERR_OPT_REFERRALS_FAILED => _('Setting LDAP referrals to "Off" failed.'),
 			CLdap::ERR_OPT_DEREF_FAILED => _('Setting LDAP dereferencing mode failed.')
 		];
 
+		$messages[CLdap::ERR_USER_NOT_FOUND] = $this->detailed_errors
+			? _('Login name or password is incorrect.')
+			: _('Incorrect user name or password or account is temporarily blocked.');
+
 		return array_key_exists($error, $messages) ? $messages[$error] : '';
 	}
 
