@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -21,16 +21,141 @@
 
 class CWidgetFormSvgGraph extends CWidgetForm {
 
-	public function __construct($data) {
-		parent::__construct($data, WIDGET_SVG_GRAPH);
+	private const WIDGET_ITEM_PERCENTILE_MIN = 1;
+	private const WIDGET_ITEM_PERCENTILE_MAX = 100;
+
+	public function __construct($data, $templateid) {
+		parent::__construct($data, $templateid, WIDGET_SVG_GRAPH);
 
 		$this->data = self::convertDottedKeys($this->data);
 
-		/**
-		 * Data set tab.
-		 *
-		 * Contains single CWidgetFieldGraphDataSet field for data sets definition and configuration.
-		 */
+		$this->initDataSetFields();
+		$this->initDisplayingOptionsFields();
+		$this->initTimePeriodFields();
+		$this->initAxesFields();
+		$this->initLegendFields();
+		$this->initProblemsFields();
+		$this->initOverridesFields();
+	}
+
+	/**
+	 * Validate form fields.
+	 *
+	 * @param bool $strict  Enables more strict validation of the form fields.
+	 *                      Must be enabled for validation of input parameters in the widget configuration form.
+	 *
+	 * @throws Exception
+	 *
+	 * @return array
+	 */
+	public function validate($strict = false): array {
+		$errors = parent::validate($strict);
+
+		$number_parser_w_suffix = new CNumberParser(['with_size_suffix' => true, 'with_time_suffix' => true]);
+		$number_parser_wo_suffix = new CNumberParser();
+
+		// Percentiles
+		if ($this->fields['percentile_left']->getValue() == SVG_GRAPH_PERCENTILE_LEFT_ON) {
+			$percentile_left_value = $this->fields['percentile_left_value']->getValue();
+
+			if ($percentile_left_value !== '') {
+				$percentile_left_value_calculated =
+						$number_parser_wo_suffix->parse($percentile_left_value) == CParser::PARSE_SUCCESS
+					? $number_parser_wo_suffix->calcValue()
+					: null;
+
+				if ($percentile_left_value_calculated === null
+						|| $percentile_left_value_calculated < self::WIDGET_ITEM_PERCENTILE_MIN
+						|| $percentile_left_value_calculated > self::WIDGET_ITEM_PERCENTILE_MAX) {
+					$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Percentile line (left)'),
+						_s('value must be between "%1$s" and "%2$s"', self::WIDGET_ITEM_PERCENTILE_MIN,
+							self::WIDGET_ITEM_PERCENTILE_MAX
+						)
+					);
+				}
+			}
+		}
+
+		if ($this->fields['percentile_right']->getValue() == SVG_GRAPH_PERCENTILE_RIGHT_ON) {
+			$percentile_right_value = $this->fields['percentile_right_value']->getValue();
+
+			if ($percentile_right_value !== '') {
+				$percentile_right_value_calculated =
+						$number_parser_wo_suffix->parse($percentile_right_value) == CParser::PARSE_SUCCESS
+					? $number_parser_wo_suffix->calcValue()
+					: null;
+
+				if ($percentile_right_value_calculated === null
+						|| $percentile_right_value_calculated < self::WIDGET_ITEM_PERCENTILE_MIN
+						|| $percentile_right_value_calculated > self::WIDGET_ITEM_PERCENTILE_MAX) {
+					$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Percentile line (right)'),
+						_s('value must be between "%1$s" and "%2$s"', self::WIDGET_ITEM_PERCENTILE_MIN,
+							self::WIDGET_ITEM_PERCENTILE_MAX
+						)
+					);
+				}
+			}
+		}
+
+		// Test graph custom time period.
+		if ($this->fields['graph_time']->getValue() == SVG_GRAPH_CUSTOM_TIME) {
+			$errors = array_merge($errors, self::validateTimeSelectorPeriod($this->fields['time_from']->getValue(),
+				$this->fields['time_to']->getValue()
+			));
+		}
+
+		// Validate Min/Max values in Axes tab.
+		if ($this->fields['lefty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
+			$lefty_min =
+					$number_parser_w_suffix->parse($this->fields['lefty_min']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser_w_suffix->calcValue()
+				: '';
+
+			$lefty_max =
+					$number_parser_w_suffix->parse($this->fields['lefty_max']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser_w_suffix->calcValue()
+				: '';
+
+			if ($lefty_min !== '' && $lefty_max !== '' && $lefty_min >= $lefty_max) {
+				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Left Y').'/'._('Max'),
+					_('Y axis MAX value must be greater than Y axis MIN value')
+				);
+			}
+		}
+
+		if ($this->fields['righty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
+			$righty_min =
+					$number_parser_w_suffix->parse($this->fields['righty_min']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser_w_suffix->calcValue()
+				: '';
+
+			$righty_max =
+					$number_parser_w_suffix->parse($this->fields['righty_max']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser_w_suffix->calcValue()
+				: '';
+
+			if ($righty_min !== '' && $righty_max !== '' && $righty_min >= $righty_max) {
+				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Right Y').'/'._('Max'),
+					_('Y axis MAX value must be greater than Y axis MIN value')
+				);
+			}
+		}
+
+		return $errors;
+	}
+
+	/**
+	 * Check if widget configuration is set to use overridden time.
+	 *
+	 * @param array $fields  Widget configuration fields.
+	 *
+	 * @return bool
+	 */
+	public static function hasOverrideTime(array $fields): bool {
+		return array_key_exists('graph_time', $fields) && $fields['graph_time'] == SVG_GRAPH_CUSTOM_TIME;
+	}
+
+	private function initDataSetFields(): void {
 		$field_ds = (new CWidgetFieldGraphDataSet('ds', _('Data set')))->setFlags(CWidgetField::FLAG_NOT_EMPTY);
 
 		if (array_key_exists('ds', $this->data)) {
@@ -38,12 +163,10 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		}
 
 		$this->fields[$field_ds->getName()] = $field_ds;
+	}
 
-		/**
-		 * Display options tab.
-		 *
-		 * Used to select either data are loaded from History or Trends or turning automatic mode on.
-		 */
+	private function initDisplayingOptionsFields(): void {
+		// History data selection.
 		$field_data_source = (new CWidgetFieldRadioButtonList('source', _('History data selection'), [
 			SVG_GRAPH_DATA_SOURCE_AUTO => _x('Auto', 'history source selection method'),
 			SVG_GRAPH_DATA_SOURCE_HISTORY => _('History'),
@@ -58,16 +181,74 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 
 		$this->fields[$field_data_source->getName()] = $field_data_source;
 
-		/**
-		 * Time period tab.
-		 *
-		 * Contains fields for specifying widget time options.
-		 */
+		// Simple triggers.
+		$field_simple_triggers = new CWidgetFieldCheckBox('simple_triggers', _('Simple triggers'));
+
+		if (array_key_exists('simple_triggers', $this->data)) {
+			$field_simple_triggers->setValue($this->data['simple_triggers']);
+		}
+
+		$this->fields[$field_simple_triggers->getName()] = $field_simple_triggers;
+
+		// Working time.
+		$field_working_time = new CWidgetFieldCheckBox('working_time', _('Working time'));
+
+		if (array_key_exists('working_time', $this->data)) {
+			$field_working_time->setValue($this->data['working_time']);
+		}
+
+		$this->fields[$field_working_time->getName()] = $field_working_time;
+
+		// Percentile line left.
+		$field_percentile_left = new CWidgetFieldCheckBox('percentile_left', _('Percentile line (left)'));
+
+		if (array_key_exists('percentile_left', $this->data)) {
+			$field_percentile_left->setValue($this->data['percentile_left']);
+		}
+
+		$this->fields[$field_percentile_left->getName()] = $field_percentile_left;
+
+		// Percentile line left value.
+		$field_percentile_left_value = (new CWidgetFieldTextBox('percentile_left_value', null))
+			->setPlaceholder(_('value'))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
+
+		if ($field_percentile_left->getValue() != SVG_GRAPH_PERCENTILE_LEFT_ON) {
+			$field_percentile_left_value->setFlags(CWidgetField::FLAG_DISABLED);
+		}
+		elseif (array_key_exists('percentile_left_value', $this->data)) {
+			$field_percentile_left_value->setValue($this->data['percentile_left_value']);
+		}
+
+		$this->fields[$field_percentile_left_value->getName()] = $field_percentile_left_value;
+
+		// Percentile line right.
+		$field_percentile_right = new CWidgetFieldCheckBox('percentile_right', _('Percentile line (right)'));
+
+		if (array_key_exists('percentile_right', $this->data)) {
+			$field_percentile_right->setValue($this->data['percentile_right']);
+		}
+
+		$this->fields[$field_percentile_right->getName()] = $field_percentile_right;
+
+		// Percentile line right value.
+		$field_percentile_right_value = (new CWidgetFieldTextBox('percentile_right_value', null))
+			->setPlaceholder(_('value'))
+			->setWidth(ZBX_TEXTAREA_TINY_WIDTH);
+
+		if ($field_percentile_right->getValue() != SVG_GRAPH_PERCENTILE_RIGHT_ON) {
+			$field_percentile_right_value->setFlags(CWidgetField::FLAG_DISABLED);
+		}
+		elseif (array_key_exists('percentile_right_value', $this->data)) {
+			$field_percentile_right_value->setValue($this->data['percentile_right_value']);
+		}
+
+		$this->fields[$field_percentile_right_value->getName()] = $field_percentile_right_value;
+	}
+
+	private function initTimePeriodFields(): void {
 		// Checkbox to specify either relative dashboard time or widget's own time.
-		$field_graph_time = (new CWidgetFieldCheckBox('graph_time', _('Set custom time period')))
-			->setAction('jQuery("#time_from, #time_to, #time_from_calendar, #time_to_calendar")'.
-				'.prop("disabled", !jQuery(this).is(":checked"));'
-			);
+		$field_graph_time = new CWidgetFieldCheckBox('graph_time', _('Set custom time period'));
 
 		if (array_key_exists('graph_time', $this->data)) {
 			$field_graph_time->setValue($this->data['graph_time']);
@@ -76,7 +257,9 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$this->fields[$field_graph_time->getName()] = $field_graph_time;
 
 		// Date from.
-		$field_time_from = (new CWidgetFieldDatePicker('time_from', _('From')))->setDefault('now-1h');
+		$field_time_from = (new CWidgetFieldDatePicker('time_from', _('From'), false))
+			->setDefault('now-1h')
+			->setFlags(CWidgetField::FLAG_NOT_EMPTY);
 
 		if ($field_graph_time->getValue() != SVG_GRAPH_CUSTOM_TIME) {
 			$field_time_from->setFlags(CWidgetField::FLAG_DISABLED);
@@ -88,7 +271,9 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$this->fields[$field_time_from->getName()] = $field_time_from;
 
 		// Time to.
-		$field_time_to = (new CWidgetFieldDatePicker('time_to', _('To')))->setDefault('now');
+		$field_time_to = (new CWidgetFieldDatePicker('time_to', _('To'), false))
+			->setDefault('now')
+			->setFlags(CWidgetField::FLAG_NOT_EMPTY);
 
 		if ($field_graph_time->getValue() != SVG_GRAPH_CUSTOM_TIME) {
 			$field_time_to->setFlags(CWidgetField::FLAG_DISABLED);
@@ -98,16 +283,11 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		}
 
 		$this->fields[$field_time_to->getName()] = $field_time_to;
+	}
 
-		/**
-		 * Axes tab.
-		 *
-		 * Contains fields to specify options for graph axes.
-		 */
+	private function initAxesFields(): void {
 		// Show left Y axis.
-		$field_lefty = (new CWidgetFieldCheckBox('lefty', _('Left Y'), _('Show')))
-			->setDefault(SVG_GRAPH_AXIS_SHOW)
-			->setAction('onLeftYChange()');
+		$field_lefty = (new CWidgetFieldCheckBox('lefty', _('Left Y'), _('Show')))->setDefault(SVG_GRAPH_AXIS_SHOW);
 
 		if (array_key_exists('lefty', $this->data)) {
 			$field_lefty->setValue($this->data['lefty']);
@@ -149,8 +329,7 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$field_lefty_units = (new CWidgetFieldSelect('lefty_units', _('Units'), [
 			SVG_GRAPH_AXIS_UNITS_AUTO => _x('Auto', 'history source selection method'),
 			SVG_GRAPH_AXIS_UNITS_STATIC => _x('Static', 'history source selection method')
-		]))
-			->setDefault(SVG_GRAPH_AXIS_UNITS_AUTO);
+		]))->setDefault(SVG_GRAPH_AXIS_UNITS_AUTO);
 
 		if ($field_lefty->getValue() != SVG_GRAPH_AXIS_SHOW) {
 			$field_lefty_units->setFlags(CWidgetField::FLAG_DISABLED);
@@ -177,9 +356,7 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$this->fields[$field_lefty_static_units->getName()] = $field_lefty_static_units;
 
 		// Show right Y axis.
-		$field_righty = (new CWidgetFieldCheckBox('righty', _('Right Y'), _('Show')))
-			->setDefault(SVG_GRAPH_AXIS_SHOW)
-			->setAction('onRightYChange()');
+		$field_righty = (new CWidgetFieldCheckBox('righty', _('Right Y'), _('Show')))->setDefault(SVG_GRAPH_AXIS_SHOW);
 
 		if (array_key_exists('righty', $this->data)) {
 			$field_righty->setValue($this->data['righty']);
@@ -221,8 +398,7 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$field_righty_units = (new CWidgetFieldSelect('righty_units', _('Units'), [
 			SVG_GRAPH_AXIS_UNITS_AUTO => _x('Auto', 'history source selection method'),
 			SVG_GRAPH_AXIS_UNITS_STATIC => _x('Static', 'history source selection method')
-		]))
-			->setDefault(SVG_GRAPH_AXIS_UNITS_AUTO);
+		]))->setDefault(SVG_GRAPH_AXIS_UNITS_AUTO);
 
 		if ($field_righty->getValue() != SVG_GRAPH_AXIS_SHOW) {
 			$field_righty_units->setFlags(CWidgetField::FLAG_DISABLED);
@@ -256,18 +432,16 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		}
 
 		$this->fields[$field_axisx->getName()] = $field_axisx;
+	}
 
+	private function initLegendFields(): void {
 		/**
 		 * Legend tab.
 		 *
 		 * Contains check-box field to show/hide legend and field to specify number of lines in which legend is shown.
 		 */
-		// Show legend.
-		$field_legend = (new CWidgetFieldCheckBox('legend', _('Show legend')))
-			->setAction('jQuery("[name=legend_lines]").rangeControl('.
-				'jQuery(this).is(":checked") ? "enable" : "disable"'.
-			');')
-			->setDefault(SVG_GRAPH_LEGEND_TYPE_SHORT);
+
+		$field_legend = (new CWidgetFieldCheckBox('legend', _('Show legend')))->setDefault(SVG_GRAPH_LEGEND_ON);
 
 		if (array_key_exists('legend', $this->data)) {
 			$field_legend->setValue($this->data['legend']);
@@ -275,13 +449,25 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 
 		$this->fields[$field_legend->getName()] = $field_legend;
 
+		// Show legend statistic.
+		$field_legend_statistic = (new CWidgetFieldCheckBox('legend_statistic', _('Display min/max/avg')))
+			->setDefault(SVG_GRAPH_LEGEND_STATISTIC_OFF);
+
+		if ($field_legend->getValue() == SVG_GRAPH_LEGEND_OFF) {
+			$field_legend_statistic->setFlags(CWidgetField::FLAG_DISABLED);
+		}
+		if (array_key_exists('legend_statistic', $this->data)) {
+			$field_legend_statistic->setValue($this->data['legend_statistic']);
+		}
+
+		$this->fields[$field_legend_statistic->getName()] = $field_legend_statistic;
+
 		// Number of lines.
 		$field_legend_lines = (new CWidgetFieldRangeControl('legend_lines', _('Number of rows'),
 			SVG_GRAPH_LEGEND_LINES_MIN, SVG_GRAPH_LEGEND_LINES_MAX
-		))
-			->setDefault(SVG_GRAPH_LEGEND_LINES_MIN);
+		))->setDefault(SVG_GRAPH_LEGEND_LINES_MIN);
 
-		if ($field_legend->getValue() == SVG_GRAPH_LEGEND_TYPE_NONE) {
+		if ($field_legend->getValue() == SVG_GRAPH_LEGEND_OFF) {
 			$field_legend_lines->setFlags(CWidgetField::FLAG_DISABLED);
 		}
 		if (array_key_exists('legend_lines', $this->data)) {
@@ -290,23 +476,24 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 
 		$this->fields[$field_legend_lines->getName()] = $field_legend_lines;
 
-		/**
-		 * Problems tab.
-		 *
-		 * Contains fields to configure highlighted problem areas in graph.
-		 */
+		// Number of columns.
+		$field_legend_columns = (new CWidgetFieldRangeControl('legend_columns', _('Number of columns'),
+			SVG_GRAPH_LEGEND_COLUMNS_MIN, SVG_GRAPH_LEGEND_COLUMNS_MAX
+		))->setDefault(SVG_GRAPH_LEGEND_COLUMNS_MAX);
+
+		if ($field_legend_statistic->getValue() == SVG_GRAPH_LEGEND_STATISTIC_ON) {
+			$field_legend_columns->setFlags(CWidgetField::FLAG_DISABLED);
+		}
+		if (array_key_exists('legend_columns', $this->data)) {
+			$field_legend_columns->setValue($this->data['legend_columns']);
+		}
+
+		$this->fields[$field_legend_columns->getName()] = $field_legend_columns;
+	}
+
+	private function initProblemsFields(): void {
 		// Checkbox: Selected items only.
-		$field_show_problems = (new CWidgetFieldCheckBox('show_problems', _('Show problems')))
-			->setAction(
-				'var on = jQuery(this).is(":checked"),'.
-					'widget = jQuery(this).closest(".ui-widget");'.
-				'jQuery("#graph_item_problems, #problem_name, #problemhosts_select")'.
-					'.prop("disabled", !on);'.
-				'jQuery("#problemhosts_").multiSelect(on ? "enable" : "disable");'.
-				'jQuery("[name^=\"severities[\"]", widget).prop("disabled", !on);'.
-				'jQuery("[name=\"evaltype\"]", widget).prop("disabled", !on);'.
-				'jQuery("input, button", jQuery("#tags_table_tags", widget)).prop("disabled", !on);'
-			);
+		$field_show_problems = new CWidgetFieldCheckBox('show_problems', _('Show problems'));
 
 		if (array_key_exists('show_problems', $this->data)) {
 			$field_show_problems->setValue($this->data['show_problems']);
@@ -393,12 +580,9 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		}
 
 		$this->fields[$field_tags->getName()] = $field_tags;
+	}
 
-		/**
-		 * Overrides tab.
-		 *
-		 * Contains single field for override configuration.
-		 */
+	private function initOverridesFields(): void {
 		$field_or = (new CWidgetFieldGraphOverride('or', _('Overrides')))->setFlags(CWidgetField::FLAG_NOT_EMPTY);
 
 		if (array_key_exists('or', $this->data)) {
@@ -416,97 +600,36 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 	 *
 	 * @return array
 	 */
-	private static function validateTimeSelectorPeriod($from, $to) {
+	private static function validateTimeSelectorPeriod(string $from, string $to): array {
 		$errors = [];
 		$ts = [];
+		$ts['now'] = time();
 		$range_time_parser = new CRangeTimeParser();
 
 		foreach (['from' => $from, 'to' => $to] as $field => $value) {
 			$range_time_parser->parse($value);
-			$ts[$field] = $range_time_parser->getDateTime($field === 'from')->getTimestamp();
+			$ts[$field] = $range_time_parser
+				->getDateTime($field === 'from')
+				->getTimestamp();
 		}
 
 		$period = $ts['to'] - $ts['from'] + 1;
+		$range_time_parser->parse('now-'.CSettingsHelper::get(CSettingsHelper::MAX_PERIOD));
+		$max_period = 1 + $ts['now'] - $range_time_parser
+			->getDateTime(true)
+			->getTimestamp();
 
 		if ($period < ZBX_MIN_PERIOD) {
 			$errors[] = _n('Minimum time period to display is %1$s minute.',
 				'Minimum time period to display is %1$s minutes.', (int) (ZBX_MIN_PERIOD / SEC_PER_MIN)
 			);
 		}
-		elseif ($period > ZBX_MAX_PERIOD) {
+		elseif ($period > $max_period) {
 			$errors[] = _n('Maximum time period to display is %1$s day.',
-				'Maximum time period to display is %1$s days.', (int) (ZBX_MAX_PERIOD / SEC_PER_DAY)
+				'Maximum time period to display is %1$s days.', (int) round($max_period / SEC_PER_DAY)
 			);
 		}
 
 		return $errors;
 	}
-
-	/**
-	 * Validate form fields.
-	 *
-	 * @param bool $strict  Enables more strict validation of the form fields.
-	 *                      Must be enabled for validation of input parameters in the widget configuration form.
-	 *
-	 * @return bool
-	 */
-	public function validate($strict = false) {
-		$errors = parent::validate($strict);
-
-		// Test graph custom time period.
-		if ($this->fields['graph_time']->getValue() == SVG_GRAPH_CUSTOM_TIME) {
-			$errors = array_merge($errors, self::validateTimeSelectorPeriod($this->fields['time_from']->getValue(),
-				$this->fields['time_to']->getValue()
-			));
-		}
-
-		$number_parser = new CNumberParser(['with_suffix' => true]);
-
-		// Validate Min/Max values in Axes tab.
-		if ($this->fields['lefty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
-			$lefty_min = $number_parser->parse($this->fields['lefty_min']->getValue()) == CParser::PARSE_SUCCESS
-				? $number_parser->calcValue()
-				: '';
-
-			$lefty_max = $number_parser->parse($this->fields['lefty_max']->getValue()) == CParser::PARSE_SUCCESS
-				? $number_parser->calcValue()
-				: '';
-
-			if ($lefty_min !== '' && $lefty_max !== '' && $lefty_min >= $lefty_max) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Left Y').'/'._('Max'),
-					_('Y axis MAX value must be greater than Y axis MIN value')
-				);
-			}
-		}
-
-		if ($this->fields['righty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
-			$righty_min = $number_parser->parse($this->fields['righty_min']->getValue()) == CParser::PARSE_SUCCESS
-				? $number_parser->calcValue()
-				: '';
-
-			$righty_max = $number_parser->parse($this->fields['righty_max']->getValue()) == CParser::PARSE_SUCCESS
-				? $number_parser->calcValue()
-				: '';
-
-			if ($righty_min !== '' && $righty_max !== '' && $righty_min >= $righty_max) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Right Y').'/'._('Max'),
-					_('Y axis MAX value must be greater than Y axis MIN value')
-				);
-			}
-		}
-
-		return $errors;
-	}
-
-	/**
-	 * Check if widget configuration is set to use overridden time.
-	 *
-	 * @param array $fields                Widget configuration fields.
-	 * @param int   $fields['graph_time']  (optional)
-	 *
-	 * @return bool
-	 */
-	public static function hasOverrideTime($fields) {
-		return (array_key_exists('graph_time', $fields) && $fields['graph_time'] == SVG_GRAPH_CUSTOM_TIME);
-	}
 }
