@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ function ipmiAuthTypes($type = null) {
 		IPMI_AUTHTYPE_DEFAULT => _('Default'),
 		IPMI_AUTHTYPE_NONE => _('None'),
 		IPMI_AUTHTYPE_MD2 => _('MD2'),
-		IPMI_AUTHTYPE_MD5 => _('MD5'),
+		IPMI_AUTHTYPE_MD5 => 'MD5',
 		IPMI_AUTHTYPE_STRAIGHT => _('Straight'),
 		IPMI_AUTHTYPE_OEM => _('OEM'),
 		IPMI_AUTHTYPE_RMCP_PLUS => _('RMCP+')
@@ -477,16 +477,36 @@ function hostInterfaceTypeNumToName($type) {
 	return $name;
 }
 
-function get_hostgroup_by_groupid($groupid) {
-	$groups = DBfetch(DBselect('SELECT g.* FROM hstgrp g WHERE g.groupid='.zbx_dbstr($groupid)));
-
-	if ($groups) {
-		return $groups;
+/**
+ * Returns the host interface as a string of the host's IP address (or DNS name) and port number.
+ *
+ * @param array|null $interface
+ * @param int    $interface['useip']  Interface use IP or DNS. INTERFACE_USE_DNS or INTERFACE_USE_IP.
+ * @param string $interface['ip']     Interface IP.
+ * @param string $interface['dns']    Interface DNS.
+ * @param string $interface['port']   Interface port.
+ *
+ * @return string
+ */
+function getHostInterface(?array $interface): string {
+	if ($interface === null) {
+		return '';
 	}
 
-	error(_s('No host groups with groupid "%s".', $groupid));
+	if ($interface['type'] == INTERFACE_TYPE_AGENT_ACTIVE) {
+		return _('Active checks');
+	}
 
-	return false;
+	if ($interface['useip'] == INTERFACE_USE_IP) {
+		$ip_or_dns = (filter_var($interface['ip'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) !== false)
+			? '['.$interface['ip'].']'
+			: $interface['ip'];
+	}
+	else {
+		$ip_or_dns = $interface['dns'];
+	}
+
+	return $ip_or_dns.':'.$interface['port'];
 }
 
 function get_host_by_itemid($itemids) {
@@ -527,206 +547,12 @@ function get_host_by_hostid($hostid, $no_error_message = 0) {
 	}
 
 	if ($no_error_message == 0) {
-		error(_s('No host with hostid "%s".', $hostid));
+		error(_s('No host with host ID "%1$s".', $hostid));
 	}
 
 	return false;
 }
 
-function updateHostStatus($hostids, $status) {
-	zbx_value2array($hostids);
-
-	$hostIds = [];
-	$oldStatus = ($status == HOST_STATUS_MONITORED ? HOST_STATUS_NOT_MONITORED : HOST_STATUS_MONITORED);
-
-	$db_hosts = DBselect(
-		'SELECT h.hostid,h.host,h.status'.
-		' FROM hosts h'.
-		' WHERE '.dbConditionInt('h.hostid', $hostids).
-			' AND h.status='.zbx_dbstr($oldStatus)
-	);
-	while ($host = DBfetch($db_hosts)) {
-		$hostIds[] = $host['hostid'];
-
-		$host_new = $host;
-		$host_new['status'] = $status;
-		add_audit_ext(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST, $host['hostid'], $host['host'], 'hosts', $host, $host_new);
-		info(_s('Updated status of host "%1$s".', $host['host']));
-	}
-
-	return DB::update('hosts', [
-		'values' => ['status' => $status],
-		'where' => ['hostid' => $hostIds]
-	]);
-}
-
-/**
- * Get parent templates for each given application.
- *
- * @param array  $applications                     An array of applications.
- * @param string $applications[]['applicationid']  ID of an applications.
- * @param array  $applications[]['templateids]     IDs of parent template applications.
- *
- * @return array
- */
-function getApplicationParentTemplates(array $applications) {
-	$parent_applicationids = [];
-	$data = [
-		'links' => [],
-		'templates' => []
-	];
-
-	foreach ($applications as $application) {
-		foreach ($application['templateids'] as $parent_applicationid) {
-			$parent_applicationids[$parent_applicationid] = true;
-			$data['links'][$application['applicationid']][] = ['applicationid' => $parent_applicationid];
-		}
-	}
-
-	if (!$parent_applicationids) {
-		return $data;
-	}
-
-	$all_parent_applicationids = [];
-	$hostids = [];
-
-	while ($parent_applicationids) {
-		$db_applications = API::Application()->get([
-			'output' => ['applicationid', 'hostid', 'templateids'],
-			'applicationids' => array_keys($parent_applicationids)
-		]);
-
-		$all_parent_applicationids += $parent_applicationids;
-		$parent_applicationids = [];
-
-		foreach ($db_applications as $db_application) {
-			$data['templates'][$db_application['hostid']] = [];
-			$hostids[$db_application['applicationid']] = $db_application['hostid'];
-
-			foreach ($db_application['templateids'] as $parent_applicationid) {
-				if (!array_key_exists($parent_applicationid, $all_parent_applicationids)) {
-					$parent_applicationids[$parent_applicationid] = true;
-				}
-			}
-
-			if (!array_key_exists($db_application['applicationid'], $data['links'])) {
-				foreach ($db_application['templateids'] as $parent_applicationid) {
-					$data['links'][$db_application['applicationid']][] = ['applicationid' => $parent_applicationid];
-				}
-			}
-		}
-	}
-
-	foreach ($data['links'] as $applicationid => &$parent_applications) {
-		foreach ($parent_applications as &$parent_application) {
-			$parent_application['hostid'] = array_key_exists($parent_application['applicationid'], $hostids)
-				? $hostids[$parent_application['applicationid']]
-				: 0;
-		}
-		unset($parent_application);
-	}
-	unset($parent_applications);
-
-	$db_templates = $data['templates']
-		? API::Template()->get([
-			'output' => ['name'],
-			'templateids' => array_keys($data['templates']),
-			'preservekeys' => true
-		])
-		: [];
-
-	$rw_templates = $db_templates
-		? API::Template()->get([
-			'output' => [],
-			'templateids' => array_keys($db_templates),
-			'editable' => true,
-			'preservekeys' => true
-		])
-		: [];
-
-	$data['templates'][0] = [];
-
-	foreach ($data['templates'] as $hostid => &$template) {
-		$template = array_key_exists($hostid, $db_templates)
-			? [
-				'hostid' => $hostid,
-				'name' => $db_templates[$hostid]['name'],
-				'permission' => array_key_exists($hostid, $rw_templates) ? PERM_READ_WRITE : PERM_READ
-			]
-			: [
-				'hostid' => $hostid,
-				'name' => _('Inaccessible template'),
-				'permission' => PERM_DENY
-			];
-	}
-	unset($template);
-
-	return $data;
-}
-
-/**
- * Auxiliary function for makeApplicationTemplatePrefix().
- *
- * @param string $applicationid
- * @param array  $parent_templates
- *
- * @return array
- */
-function getTopLevelTemplates($applicationid, array $parent_templates) {
-	$templates = [];
-
-	foreach ($parent_templates['links'][$applicationid] as $parent_application) {
-		if (!array_key_exists($parent_application['applicationid'], $parent_templates['links'])) {
-			$templates[] = $parent_templates['templates'][$parent_application['hostid']];
-		}
-		else {
-			$templates = array_merge($templates,
-				getTopLevelTemplates($parent_application['applicationid'], $parent_templates)
-			);
-		}
-	}
-
-	return $templates;
-}
-
-/**
- * Returns a template prefix for selected application.
- *
- * @param string $applicationid
- * @param array  $parent_templates  The list of the templates, prepared by getApplicationParentTemplates() function.
- *
- * @return array|null
- */
-function makeApplicationTemplatePrefix($applicationid, array $parent_templates) {
-	if (!array_key_exists($applicationid, $parent_templates['links'])) {
-		return null;
-	}
-
-	$templates = getTopLevelTemplates($applicationid, $parent_templates);
-	CArrayHelper::sort($templates, ['name']);
-
-	$list = [];
-
-	foreach ($templates as $template) {
-		if ($template['permission'] == PERM_READ_WRITE) {
-			$name = (new CLink(CHtml::encode($template['name']),
-				(new CUrl('applications.php'))->setArgument('hostid', $template['hostid'])
-			))->addClass(ZBX_STYLE_LINK_ALT);
-		}
-		else {
-			$name = new CSpan(CHtml::encode($template['name']));
-		}
-
-		$list[] = $name->addClass(ZBX_STYLE_GREY);
-		$list[] = ', ';
-	}
-
-	array_pop($list);
-	$list[] = NAME_DELIMITER;
-
-	return $list;
-}
-
 /**
  * Get parent templates for each given host prototype.
  *
@@ -838,10 +664,11 @@ function getHostPrototypeParentTemplates(array $host_prototypes) {
  *
  * @param string $host_prototypeid
  * @param array  $parent_templates  The list of the templates, prepared by getHostPrototypeParentTemplates() function.
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array|null
  */
-function makeHostPrototypeTemplatePrefix($host_prototypeid, array $parent_templates) {
+function makeHostPrototypeTemplatePrefix($host_prototypeid, array $parent_templates, bool $provide_links) {
 	if (!array_key_exists($host_prototypeid, $parent_templates['links'])) {
 		return null;
 	}
@@ -852,10 +679,11 @@ function makeHostPrototypeTemplatePrefix($host_prototypeid, array $parent_templa
 
 	$template = $parent_templates['templates'][$parent_templates['links'][$host_prototypeid]['parent_hostid']];
 
-	if ($template['permission'] == PERM_READ_WRITE) {
+	if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 		$name = (new CLink(CHtml::encode($template['name']),
 			(new CUrl('host_prototypes.php'))
 				->setArgument('parent_discoveryid', $parent_templates['links'][$host_prototypeid]['lld_ruleid'])
+				->setArgument('context', 'template')
 		))->addClass(ZBX_STYLE_LINK_ALT);
 	}
 	else {
@@ -870,21 +698,23 @@ function makeHostPrototypeTemplatePrefix($host_prototypeid, array $parent_templa
  *
  * @param string $host_prototypeid
  * @param array  $parent_templates  The list of the templates, prepared by getHostPrototypeParentTemplates() function.
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array
  */
-function makeHostPrototypeTemplatesHtml($host_prototypeid, array $parent_templates) {
+function makeHostPrototypeTemplatesHtml($host_prototypeid, array $parent_templates, bool $provide_links) {
 	$list = [];
 
 	while (array_key_exists($host_prototypeid, $parent_templates['links'])) {
 		$template = $parent_templates['templates'][$parent_templates['links'][$host_prototypeid]['parent_hostid']];
 
-		if ($template['permission'] == PERM_READ_WRITE) {
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 			$name = new CLink(CHtml::encode($template['name']),
 				(new CUrl('host_prototypes.php'))
 					->setArgument('form', 'update')
 					->setArgument('parent_discoveryid', $parent_templates['links'][$host_prototypeid]['lld_ruleid'])
 					->setArgument('hostid', $parent_templates['links'][$host_prototypeid]['hostid'])
+					->setArgument('context', 'template')
 			);
 		}
 		else {
@@ -903,73 +733,6 @@ function makeHostPrototypeTemplatesHtml($host_prototypeid, array $parent_templat
 	return $list;
 }
 
-/**
- * Get host ids of hosts which $groupids can be unlinked from.
- * if $hostids is passed, function will check only these hosts.
- *
- * @param array $groupids
- * @param array $hostids
- *
- * @return array
- */
-function getUnlinkableHostIds(array $groupIds, array $hostIds) {
-	if (!$hostIds) {
-		return [];
-	}
-
-	$dbResult = DBselect(
-		'SELECT hg.hostid'.
-		' FROM hosts_groups hg'.
-		' WHERE '.dbConditionInt('hg.groupid', $groupIds, true).
-			' AND '.dbConditionInt('hg.hostid', $hostIds).
-		' GROUP BY hg.hostid'
-	);
-
-	$unlinkableHostIds = [];
-	while ($dbRow = DBfetch($dbResult)) {
-		$unlinkableHostIds[] = $dbRow['hostid'];
-	}
-
-	return $unlinkableHostIds;
-}
-
-function getDeletableHostGroupIds(array $groupIds) {
-	// selecting the list of hosts linked to the host groups
-	$dbResult = DBselect(
-		'SELECT hg.hostid'.
-		' FROM hosts_groups hg'.
-		' WHERE '.dbConditionInt('hg.groupid', $groupIds)
-	);
-
-	$linkedHostIds = [];
-	while ($dbRow = DBfetch($dbResult)) {
-		$linkedHostIds[] = $dbRow['hostid'];
-	}
-
-	// the list of hosts which can be unlinked from the host groups
-	$hostIds = getUnlinkableHostIds($groupIds, $linkedHostIds);
-
-	$dbResult = DBselect(
-		'SELECT g.groupid'.
-		' FROM hstgrp g'.
-		' WHERE g.internal='.ZBX_NOT_INTERNAL_GROUP.
-			' AND '.dbConditionInt('g.groupid', $groupIds).
-			' AND NOT EXISTS ('.
-				'SELECT NULL'.
-				' FROM hosts_groups hg'.
-				' WHERE g.groupid=hg.groupid'.
-					($hostIds ? ' AND '.dbConditionInt('hg.hostid', $hostIds, true) : '').
-			')'
-	);
-
-	$deletableGroupIds = [];
-	while ($dbRow = DBfetch($dbResult)) {
-		$deletableGroupIds[$dbRow['groupid']] = $dbRow['groupid'];
-	}
-
-	return $deletableGroupIds;
-}
-
 function isTemplate($hostId) {
 	$dbHost = DBfetch(DBselect('SELECT h.status FROM hosts h WHERE h.hostid='.zbx_dbstr($hostId)));
 
@@ -983,10 +746,15 @@ function isTemplate($hostId) {
  *   array(
  *       '{$MACRO}' => array(
  *           'macro' => '{$MACRO}',
+ *           'parent_host' => array(                <- optional
+ *               'value' => 'parent host level value',
+ *               'type' => 0,
+ *               'description' => ''
+ *           ),
  *           'template' => array(                   <- optional
  *               'value' => 'template-level value'
  *               'templateid' => 10001,
- *               'name' => 'Template OS Linux'
+ *               'name' => 'Template OS Linux by Zabbix agent'
  *           ),
  *           'global' => array(                     <- optional
  *               'value' => 'global-level value'
@@ -994,24 +762,29 @@ function isTemplate($hostId) {
  *       )
  *   )
  *
- * @param array $hostids
+ * @param array     $hostids        Host or template ids.
+ * @param int|null  $parent_hostid  Parent host id of host prototype.
  *
  * @return array
  */
-function getInheritedMacros(array $hostids) {
+function getInheritedMacros(array $hostids, ?int $parent_hostid = null): array {
 	$user_macro_parser = new CUserMacroParser();
 
 	$all_macros = [];
 	$global_macros = [];
 
 	$db_global_macros = API::UserMacro()->get([
-		'output' => ['macro', 'value'],
+		'output' => ['macro', 'value', 'description', 'type'],
 		'globalmacro' => true
 	]);
 
 	foreach ($db_global_macros as $db_global_macro) {
 		$all_macros[$db_global_macro['macro']] = true;
-		$global_macros[$db_global_macro['macro']] = $db_global_macro['value'];
+		$global_macros[$db_global_macro['macro']] = [
+			'value' => getMacroConfigValue($db_global_macro),
+			'description' => $db_global_macro['description'],
+			'type' => $db_global_macro['type']
+		];
 	}
 
 	// hostid => array('name' => name, 'macros' => array(macro => value), 'templateids' => array(templateid))
@@ -1023,7 +796,7 @@ function getInheritedMacros(array $hostids) {
 		$db_templates = API::Template()->get([
 			'output' => ['name'],
 			'selectParentTemplates' => ['templateid'],
-			'selectMacros' => ['macro', 'value'],
+			'selectMacros' => ['macro', 'value', 'description', 'type'],
 			'templateids' => $templateids,
 			'preservekeys' => true
 		]);
@@ -1045,7 +818,11 @@ function getInheritedMacros(array $hostids) {
 			 */
 			foreach ($db_template['macros'] as $dbMacro) {
 				if (array_key_exists($dbMacro['macro'], $all_macros)) {
-					$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+					$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+						'value' => getMacroConfigValue($dbMacro),
+						'description' => $dbMacro['description'],
+						'type' => $dbMacro['type']
+					];
 					$all_macros[$dbMacro['macro']] = true;
 				}
 				else {
@@ -1054,7 +831,11 @@ function getInheritedMacros(array $hostids) {
 					$tpl_context = $user_macro_parser->getContext();
 
 					if ($tpl_context === null) {
-						$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+						$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+							'value' => getMacroConfigValue($dbMacro),
+							'description' => $dbMacro['description'],
+							'type' => $dbMacro['type']
+						];
 						$all_macros[$dbMacro['macro']] = true;
 					}
 					else {
@@ -1072,7 +853,11 @@ function getInheritedMacros(array $hostids) {
 									$all_macros[$global_macro]
 								);
 
-								$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+								$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+									'value' => getMacroConfigValue($dbMacro),
+									'description' => $dbMacro['description'],
+									'type' => $dbMacro['type']
+								];
 								$all_macros[$dbMacro['macro']] = true;
 								$global_macros[$dbMacro['macro']] = $global_value;
 
@@ -1081,7 +866,11 @@ function getInheritedMacros(array $hostids) {
 						}
 
 						if (!$match_found) {
-							$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+							$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+								'value' => getMacroConfigValue($dbMacro),
+								'description' => $dbMacro['description'],
+								'type' => $dbMacro['type']
+							];
 							$all_macros[$dbMacro['macro']] = true;
 						}
 					}
@@ -1099,17 +888,38 @@ function getInheritedMacros(array $hostids) {
 		}
 	} while ($templateids);
 
-	$all_macros = array_keys($all_macros);
 	$all_templates = [];
 	$inherited_macros = [];
+	$parent_host_macros = [];
+
+	if ($parent_hostid !== null) {
+		$parent_host_macros = API::UserMacro()->get([
+			'output' => ['macro', 'type', 'value', 'description'],
+			'hostids' => [$parent_hostid]
+		]);
+
+		$parent_host_macros = array_column($parent_host_macros, null, 'macro');
+		$all_macros += array_fill_keys(array_keys($parent_host_macros), true);
+	}
+
+	$all_macros = array_keys($all_macros);
 
 	// resolving
 	foreach ($all_macros as $macro) {
 		$inherited_macro = ['macro' => $macro];
 
-		if (array_key_exists($macro, $global_macros)) {
+		if (array_key_exists($macro, $parent_host_macros)) {
+			$inherited_macro['parent_host'] = [
+				'value' => getMacroConfigValue($parent_host_macros[$macro]),
+				'description' => $parent_host_macros[$macro]['description'],
+				'type' => $parent_host_macros[$macro]['type']
+			];
+		}
+		elseif (array_key_exists($macro, $global_macros)) {
 			$inherited_macro['global'] = [
-				'value' => $global_macros[$macro]
+				'value' => $global_macros[$macro]['value'],
+				'description' => $global_macros[$macro]['description'],
+				'type' => $global_macros[$macro]['type']
 			];
 		}
 
@@ -1121,10 +931,12 @@ function getInheritedMacros(array $hostids) {
 			foreach ($templateids as $templateid) {
 				if (array_key_exists($templateid, $hosts) && array_key_exists($macro, $hosts[$templateid]['macros'])) {
 					$inherited_macro['template'] = [
-						'value' => $hosts[$templateid]['macros'][$macro],
+						'value' => $hosts[$templateid]['macros'][$macro]['value'],
+						'description' => $hosts[$templateid]['macros'][$macro]['description'],
 						'templateid' => $hosts[$templateid]['templateid'],
 						'name' => $hosts[$templateid]['name'],
-						'rights' => PERM_READ
+						'rights' => PERM_READ,
+						'type' => $hosts[$templateid]['macros'][$macro]['type']
 					];
 
 					if (!array_key_exists($hosts[$templateid]['templateid'], $all_templates)) {
@@ -1178,13 +990,19 @@ function getInheritedMacros(array $hostids) {
  *   array(
  *       '{$MACRO}' => array(
  *           'macro' => '{$MACRO}',
- *           'type' => 0x03,						<- MACRO_TYPE_INHERITED, MACRO_TYPE_HOSTMACRO or MACRO_TYPE_BOTH
+ *           'type' => 0,                           <- ZBX_MACRO_TYPE_TEXT or ZBX_MACRO_TYPE_SECRET
+ *           'inherited_type' => 0x03,              <- ZBX_PROPERTY_INHERITED, ZBX_PROPERTY_OWN or ZBX_PROPERTY_BOTH
  *           'value' => 'effective value',
  *           'hostmacroid' => 7532,                 <- optional
+ *           'parent_host' => array(                <- optional
+ *               'value' => 'parent host value',
+ *               'type' => 0,
+ *               'description' => ''
+ *           ),
  *           'template' => array(                   <- optional
  *               'value' => 'template-level value'
  *               'templateid' => 10001,
- *               'name' => 'Template OS Linux'
+ *               'name' => 'Template OS Linux by Zabbix agent'
  *           ),
  *           'global' => array(                     <- optional
  *               'value' => 'global-level value'
@@ -1192,19 +1010,27 @@ function getInheritedMacros(array $hostids) {
  *       )
  *   )
  *
- * @param array $host_macros		the list of host macros
- * @param array $inherited_macros	the list of inherited macros (the output of the getInheritedMacros() function)
+ * @param array $host_macros       The list of host macros.
+ * @param array $inherited_macros  The list of inherited macros (the output of the getInheritedMacros() function).
  *
  * @return array
  */
-function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
+function mergeInheritedMacros(array $host_macros, array $inherited_macros): array {
 	$user_macro_parser = new CUserMacroParser();
+	$inherit_order = ['parent_host', 'template', 'global'];
 
 	foreach ($inherited_macros as &$inherited_macro) {
-		$inherited_macro['type'] = MACRO_TYPE_INHERITED;
-		$inherited_macro['value'] = array_key_exists('template', $inherited_macro)
-			? $inherited_macro['template']['value']
-			: $inherited_macro['global']['value'];
+		[$inherited_level] = array_values(array_intersect($inherit_order, array_keys($inherited_macro)));
+		$inherited_macro['inherited_type'] = ZBX_PROPERTY_INHERITED;
+		$inherited_macro['inherited_level'] = $inherited_level;
+		$inherited_macro['value'] = $inherited_macro[$inherited_level]['value'];
+		$inherited_macro['type'] = $inherited_macro[$inherited_level]['type'];
+		$inherited_macro['description'] = $inherited_macro[$inherited_level]['description'];
+
+		// Secret macro value cannot be inherited.
+		if ($inherited_macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+			unset($inherited_macro['value']);
+		}
 	}
 	unset($inherited_macro);
 
@@ -1213,6 +1039,11 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 	 * checking for contexts, since {$MACRO:} is the same as {$MACRO:""}.
 	 */
 	foreach ($host_macros as &$host_macro) {
+		// Secret macro value cannot be inherited.
+		if ($host_macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+			unset($inherited_macros[$host_macro['macro']]['value']);
+		}
+
 		if (array_key_exists($host_macro['macro'], $inherited_macros)) {
 			$host_macro = array_merge($inherited_macros[$host_macro['macro']], $host_macro);
 			unset($inherited_macros[$host_macro['macro']]);
@@ -1227,7 +1058,7 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 				$hst_context = $user_macro_parser->getContext();
 
 				if ($hst_context === null) {
-					$host_macro['type'] = 0x00;
+					$host_macro['inherited_type'] = 0x00;
 				}
 				else {
 					$match_found = false;
@@ -1249,16 +1080,16 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 					}
 
 					if (!$match_found) {
-						$host_macro['type'] = 0x00;
+						$host_macro['inherited_type'] = 0x00;
 					}
 				}
 			}
 			else {
-				$host_macro['type'] = 0x00;
+				$host_macro['inherited_type'] = 0x00;
 			}
 		}
 
-		$host_macro['type'] |= MACRO_TYPE_HOSTMACRO;
+		$host_macro['inherited_type'] |= ZBX_PROPERTY_OWN;
 	}
 	unset($host_macro);
 
@@ -1278,11 +1109,11 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
  */
 function cleanInheritedMacros(array $macros) {
 	foreach ($macros as $idx => $macro) {
-		if (array_key_exists('type', $macro) && !($macro['type'] & MACRO_TYPE_HOSTMACRO)) {
+		if (array_key_exists('inherited_type', $macro) && !($macro['inherited_type'] & ZBX_PROPERTY_OWN)) {
 			unset($macros[$idx]);
 		}
 		else {
-			unset($macros[$idx]['type'], $macros[$idx]['inherited']);
+			unset($macros[$idx]['inherited_type'], $macros[$idx]['inherited']);
 		}
 	}
 
@@ -1331,32 +1162,7 @@ function isReadableTemplates(array $templateids) {
 }
 
 /**
- * Check if user has read permissions for hosts or templates.
- *
- * @param array $hostids
- *
- * @return bool
- */
-function isReadableHostTemplates(array $hostids) {
-	$count = API::Host()->get([
-		'countOutput' => true,
-		'hostids' => $hostids
-	]);
-
-	if ($count == count($hostids)) {
-		return true;
-	}
-
-	$count += API::Template()->get([
-		'countOutput' => true,
-		'templateids' => $hostids
-	]);
-
-	return ($count == count($hostids));
-}
-
-/**
- * Check if user has read permissions for hosts or templates.
+ * Check if user has write permissions for hosts or templates.
  *
  * @param array $hostids
  *
@@ -1381,3 +1187,82 @@ function isWritableHostTemplates(array $hostids) {
 
 	return ($count == count($hostids));
 }
+
+function getAddNewInterfaceSubmenu() {
+	return [
+		'main_section' => [
+			'items' => [
+				"javascript:hostInterfaceManager.addAgent();" => _('Agent'),
+				"javascript:hostInterfaceManager.addSnmp();" => _('SNMP'),
+				"javascript:hostInterfaceManager.addJmx();" => _('JMX'),
+				"javascript:hostInterfaceManager.addIpmi();" => _('IPMI')
+			]
+		]
+	];
+}
+
+function renderInterfaceHeaders() {
+	return (new CDiv())
+		->addClass(implode(' ', [ZBX_STYLE_HOST_INTERFACE_CONTAINER, ZBX_STYLE_HOST_INTERFACE_CONTAINER_HEADER]))
+		->addItem(
+			(new CDiv())
+				->addClass(implode(' ', [ZBX_STYLE_HOST_INTERFACE_ROW, ZBX_STYLE_HOST_INTERFACE_ROW_HEADER]))
+				->addItem([
+					(new CDiv())->addClass(ZBX_STYLE_HOST_INTERFACE_CELL),
+					(new CDiv(_('Type')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_TYPE
+						])
+					),
+					(new CDiv(_('IP address')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_IP
+						])
+					),
+					(new CDiv(_('DNS name')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_DNS
+						])
+					),
+					(new CDiv(_('Connect to')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_USEIP
+						])
+					),
+					(new CDiv(_('Port')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_PORT
+						])
+					),
+					(new CDiv(_('Default')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_ACTION
+						])
+					)
+				])
+		);
+}
+
+function getHostDashboards(string $hostid, array $dashboard_fields = []): array {
+	$dashboard_fields = array_merge($dashboard_fields, ['dashboardid']);
+	$dashboard_fields = array_keys(array_flip($dashboard_fields));
+
+	$templateids = CApiHostHelper::getParentTemplates([$hostid])[1];
+
+	return API::TemplateDashboard()->get([
+		'output' => $dashboard_fields,
+		'templateids' => $templateids,
+		'preservekeys' => true
+	]);
+}
+
+/**
+ * Return macro value to display in the list of inherited macros.
+ *
+ * @param array $macro
+ *
+ * @return string
+ */
+function getMacroConfigValue(array $macro): string {
+	return ($macro['type'] == ZBX_MACRO_TYPE_SECRET) ? ZBX_SECRET_MASK : $macro['value'];
+}
