@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,17 +23,23 @@
  * @var CView $this
  */
 
-$widget = (new CWidget())->setTitle(_('Host inventory'));
+$widget = (new CWidget())
+	->setTitle(_('Host inventory'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::INVENTORY_HOST_LIST));
 
 // Make an inventory field dropdown.
-$inventoryFieldsComboBox = new CComboBox('filter_field', $data['filter']['field']);
+$inventory_field_select = (new CSelect('filter_field'))
+	->setValue($data['filter']['field'])
+	->setFocusableElementId('label-field');
+
 foreach ($data['host_inventories'] as $inventoryField) {
-	$inventoryFieldsComboBox->addItem($inventoryField['db_field'], $inventoryField['title']);
+	$inventory_field_select->addOption(new CSelectOption($inventoryField['db_field'], $inventoryField['title']));
 }
 
 // filter
 $widget->addItem(
-	(new CFilter(new CUrl('hostinventories.php')))
+	(new CFilter())
+		->setResetUrl(new CUrl('hostinventories.php'))
 		->setProfile($data['profileIdx'])
 		->setActiveTab($data['active_tab'])
 		->addFilterTab(_('Filter'), [
@@ -50,19 +56,21 @@ $widget->addItem(
 								'srcfld1' => 'groupid',
 								'dstfrm' => 'zbx_filter',
 								'dstfld1' => 'filter_groups_',
-								'real_hosts' => 1,
+								'with_hosts' => true,
 								'enrich_parent_groups' => true
 							]
 						]
 					]))->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
 				)
-				->addRow(_('Field'), [
-					$inventoryFieldsComboBox,
+				->addRow(new CLabel(_('Field'), 'label-field'), [
+					$inventory_field_select,
 					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					new CComboBox('filter_exact', $data['filter']['exact'], null, [
-						0 => _('contains'),
-						1 => _('equals')
-					]),
+					(new CSelect('filter_exact'))
+						->setValue($data['filter']['exact'])
+						->addOptions(CSelect::createOptionsFromArray([
+							0 => _('contains'),
+							1 => _('equals')
+						])),
 					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
 					(new CTextBox('filter_field_value', $data['filter']['fieldValue']))
 						->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
@@ -86,17 +94,13 @@ $table = (new CTableInfo())
 	]);
 
 foreach ($this->data['hosts'] as $host) {
-	$hostGroups = [];
-	foreach ($host['groups'] as $group) {
-		$hostGroups[] = $group['name'];
-	}
-	natsort($hostGroups);
-	$hostGroups = implode(', ', $hostGroups);
+	$hostgroups = array_column($host['hostgroups'], 'name');
+	natsort($hostgroups);
 
 	$row = [
 		(new CLink($host['name'], (new CUrl('hostinventories.php'))->setArgument('hostid', $host['hostid'])))
 			->addClass($host['status'] == HOST_STATUS_NOT_MONITORED ? ZBX_STYLE_RED : null),
-		$hostGroups,
+		implode(', ', $hostgroups),
 		zbx_str2links($host['inventory']['name']),
 		zbx_str2links($host['inventory']['type']),
 		zbx_str2links($host['inventory']['os']),
