@@ -176,7 +176,8 @@ $overviewFormList->addRow(_('Monitoring'),
 // configuration
 if ($data['allowed_ui_conf_hosts'] && $data['rwHost']) {
 	$hostLink = (new CLink(_('Host')))
-		->onClick('view.editHost({hostid:\''.$data['host']['hostid'].'\'})');
+		->setAttribute('data-hostid', $data['host']['hostid'])
+		->onClick('view.editHost({hostid: this.dataset.hostid});');
 	$itemsLink = new CLink(_('Items'),
 		(new CUrl('items.php'))
 			->setArgument('filter_set', '1')
@@ -264,7 +265,7 @@ $web_layout_mode = CViewHelper::loadLayoutMode();
 	->setWebLayoutMode($web_layout_mode)
 	->setControls((new CList())->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode])))
 	->addItem((new CForm())
-		->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+		->setAttribute('aria-labelledby', ZBX_STYLE_PAGE_TITLE)
 		->addItem($hostInventoriesTab)
 	)
 	->show();
