@@ -47,9 +47,6 @@ if (CAuthenticationHelper::get(CAuthenticationHelper::SAML_AUTH_ENABLED) == ZBX_
 	redirect($redirect_to->toString());
 }
 
-require_once __DIR__.'/vendor/php-saml/_toolkit_loader.php';
-require_once __DIR__.'/vendor/xmlseclibs/xmlseclibs.php';
-
 use OneLogin\Saml2\Auth;
 use OneLogin\Saml2\Utils;
 
@@ -277,13 +274,13 @@ echo (new CView('general.warning', [
 	'header' => _('You are not logged in'),
 	'messages' => array_column(get_and_clear_messages(), 'message'),
 	'buttons' => [
-		(new CButton('login', _('Login')))->onClick(
-			'document.location = '.json_encode(
+		(new CButton('login', _('Login')))
+			->setAttribute('data-url',
 				$redirect_to
 					->setArgument('request', $request)
 					->getUrl()
-			).';'
-		)
+			)
+			->onClick('document.location = this.dataset.url;')
 	],
 	'theme' => getUserTheme(CWebUser::$data)
 ]))->getOutput();
