@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ class ZSelect extends HTMLElement {
 		this._options_map = new Map();
 
 		this._option_template = "#{label}";
+		this._selected_option_template = "#{label}";
 
 		this._highlighted_index = -1;
 		this._preselected_index = -1;
@@ -128,6 +129,11 @@ class ZSelect extends HTMLElement {
 			this.removeAttribute('option-template');
 		}
 
+		if (this.hasAttribute('selected-option-template')) {
+			this._selected_option_template = this.getAttribute('selected-option-template');
+			this.removeAttribute('selected-option-template');
+		}
+
 		if (this.hasAttribute('data-options')) {
 			const options = JSON.parse(this.getAttribute('data-options'));
 
@@ -353,7 +359,9 @@ class ZSelect extends HTMLElement {
 		const option = this.getOptionByIndex(index);
 
 		if (option) {
-			this._button.innerText = option.label;
+			this._button.innerHTML = new Template(this._selected_option_template).evaluate(
+				Object.assign({label: option.label.trim()}, option.extra || {})
+			);
 			this._input.disabled = this.hasAttribute('disabled');
 		}
 		else {
