@@ -401,11 +401,19 @@ var hintBox = {
 
 			var $hint_box = $target.next('.hint-box').empty();
 
-			if (resp.messages) {
-				$hint_box.append(resp.messages);
+			if ('error' in resp) {
+				const message_box = makeMessageBox('bad', resp.error.messages, resp.error.title, false, true);
+
+				$hint_box.append(message_box);
 			}
-			if (resp.data) {
-				$hint_box.append(resp.data);
+			else {
+				if (resp.messages) {
+					$hint_box.append(resp.messages);
+				}
+
+				if (resp.data) {
+					$hint_box.append(resp.data);
+				}
 			}
 
 			hintBox.displayHint(e, $target);
@@ -486,13 +494,20 @@ var hintBox = {
 
 		jQuery(appendTo).append(box);
 
-		var removeHandler = function() {
-			hintBox.deleteHint(target);
-		};
+		target.observer = new MutationObserver(() => {
+			const node = target instanceof Node ? target : target[0];
 
-		jQuery(target)
-			.off('remove', removeHandler)
-			.on('remove', removeHandler);
+			if (document.body.contains(node)) {
+				return;
+			}
+
+			hintBox.deleteHint(target);
+		})
+
+		target.observer.observe(document, {
+			childList: true,
+			subtree: true
+		})
 
 		return box;
 	},
@@ -615,12 +630,18 @@ var hintBox = {
 			delete target.hintBoxItem;
 
 			if (target.isStatic) {
-				if (jQuery(target).data('return-control') !== 'undefined') {
+				if (jQuery(target).data('return-control') !== undefined) {
 					jQuery(target).data('return-control').focus();
 				}
 				delete target.isStatic;
 			}
 		}
+
+		if (target.observer !== undefined) {
+			target.observer.disconnect();
+
+			delete target.observer;
+		}
 	},
 
 	getUniqueId: function() {
