@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,20 +18,8 @@
 **/
 
 
-jQuery(function($) {
-	var ZBX_STYLE_CLASS = 'multiselect-control',
-		KEY = {
-			ARROW_DOWN: 40,
-			ARROW_LEFT: 37,
-			ARROW_RIGHT: 39,
-			ARROW_UP: 38,
-			BACKSPACE: 8,
-			DELETE: 46,
-			ENTER: 13,
-			ESCAPE: 27,
-			TAB: 9
-		};
-
+(function($) {
+	var ZBX_STYLE_CLASS = 'multiselect-control';
 	const MS_ACTION_POPUP = 0;
 	const MS_ACTION_AUTOSUGGEST = 1;
 
@@ -257,6 +245,47 @@ jQuery(function($) {
 			});
 
 			return ret;
+		},
+
+		/**
+		 * @return HTMLElement
+		 */
+		getSelectButton: function() {
+			var ret = null;
+
+			this.each(function() {
+				var $obj = $(this);
+
+				if ($obj.data('multiSelect') !== undefined) {
+					ret = $obj.data('multiSelect').select_button[0];
+
+					return false;
+				}
+			});
+
+			return ret;
+		},
+
+		/**
+		 * @param array entries  IDs to mark disabled.
+		 */
+		setDisabledEntries: function (entries) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms_parameters = $obj.data('multiSelect');
+
+				if (typeof ms_parameters === 'undefined') {
+					return;
+				}
+
+				const link = new Curl(ms_parameters.options.url, false);
+				link.setArgument('disabledids', entries);
+
+				ms_parameters.options.url = link.getUrl();
+				ms_parameters.options.popup.parameters.disableids = entries;
+
+				$obj.data('multiSelect', ms_parameters);
+			});
 		}
 	};
 
@@ -311,7 +340,6 @@ jQuery(function($) {
 				},
 				placeholder: t('type here to search'),
 				data: [],
-				only_hostid: 0,
 				excludeids: [],
 				addNew: false,
 				defaultValue: null,
@@ -349,7 +377,8 @@ jQuery(function($) {
 						 * In such case the "focusout" event (IE) of the search input should not be processed.
 						 */
 						available_false_click: false
-					}
+					},
+					select_button: null
 				};
 
 			ms.values.available_div.on('mousedown', 'li', function() {
@@ -411,34 +440,40 @@ jQuery(function($) {
 				});
 			}
 
-			if (ms.options.popup.parameters != null) {
-				if (typeof ms.options.popup.parameters['only_hostid'] !== 'undefined') {
-					ms.options.only_hostid = ms.options.popup.parameters['only_hostid'];
-				}
-
-				var popup_button = $('<button>', {
-						type: 'button',
-						'class': 'btn-grey',
-						text: ms.options.labels['Select']
-					});
-
-				if (ms.options.disabled) {
-					popup_button.prop('disabled', true);
-				}
-
-				popup_button.on('click', function(event) {
-					var popup_options = ms.options.popup.parameters;
-
-					if (ms.options.popup.filter_preselect_fields) {
-						popup_options = jQuery.extend(popup_options, getFilterPreselectField($obj, MS_ACTION_POPUP));
-					}
-
-					// Click used instead focus because in patternselect listen only click.
-					$('input[type="text"]', $obj).click();
-					return PopUp('popup.generic', popup_options, null, event.target);
+			if (ms.options.custom_select || ms.options.popup.parameters !== undefined) {
+				ms.select_button = $('<button>', {
+					type: 'button',
+					'class': 'btn-grey',
+					text: ms.options.labels['Select']
 				});
 
-				$obj.after($('<div>', {'class': 'multiselect-button'}).append(popup_button));
+				if (ms.options.disabled) {
+					ms.select_button.prop('disabled', true);
+				}
+
+				if (ms.options.popup.parameters !== undefined) {
+					ms.select_button.on('click', function(event) {
+						var parameters = ms.options.popup.parameters;
+
+						if (ms.options.popup.filter_preselect_fields) {
+							parameters = jQuery.extend(parameters, getFilterPreselectField($obj, MS_ACTION_POPUP));
+						}
+
+						if (typeof parameters['disable_selected'] !== 'undefined' && parameters['disable_selected']) {
+							parameters['disableids'] = Object.keys(ms.values.selected);
+						}
+
+						// Click used instead focus because in patternselect only click is listened for.
+						$('input[type="text"]', $obj).click();
+
+						return PopUp('popup.generic', parameters, {
+							dialogue_class: 'modal-popup-generic',
+							trigger_element: event.target
+						});
+					});
+				}
+
+				$obj.after($('<div>', {'class': 'multiselect-button'}).append(ms.select_button));
 			}
 		});
 	};
@@ -452,28 +487,39 @@ jQuery(function($) {
 	 * @return {object}
 	 */
 	function getFilterPreselectField($obj, action) {
-		var ms = $obj.data('multiSelect'),
-			preselect_options = ms.options[(action == MS_ACTION_AUTOSUGGEST) ? 'autosuggest' : 'popup'] || null,
-			ret = {};
+		const ms = $obj.data('multiSelect');
+		const preselect_options = ms.options[(action == MS_ACTION_AUTOSUGGEST) ? 'autosuggest' : 'popup'] || null;
+		const ret = {};
 
 		if (!preselect_options) {
 			return ret;
 		}
 
 		if (typeof preselect_options.filter_preselect_fields.hosts !== 'undefined') {
-			var hosts = $('#' + preselect_options.filter_preselect_fields.hosts).multiSelect('getData');
+			const hosts = $('#' + preselect_options.filter_preselect_fields.hosts).multiSelect('getData');
+
 			if (hosts.length != 0) {
 				ret.hostid = hosts[0].id;
 			}
 		}
 
 		if (typeof preselect_options.filter_preselect_fields.hostgroups !== 'undefined') {
-			var host_groups = $('#' + preselect_options.filter_preselect_fields.hostgroups).multiSelect('getData');
+			const host_groups = $('#' + preselect_options.filter_preselect_fields.hostgroups).multiSelect('getData');
+
 			if (host_groups.length != 0) {
 				ret.groupid = host_groups[0].id;
 			}
 		}
 
+		if (typeof preselect_options.filter_preselect_fields.templategroups !== 'undefined') {
+			const template_groups = $('#' + preselect_options.filter_preselect_fields.templategroups)
+				.multiSelect('getData');
+
+			if (template_groups.length != 0) {
+				ret.templategroupid = template_groups[0].id;
+			}
+		}
+
 		return ret;
 	}
 
@@ -492,11 +538,11 @@ jQuery(function($) {
 			})
 				.on('keyup', function(e) {
 					switch (e.which) {
-						case KEY.ARROW_DOWN:
-						case KEY.ARROW_LEFT:
-						case KEY.ARROW_RIGHT:
-						case KEY.ARROW_UP:
-						case KEY.ESCAPE:
+						case KEY_ARROW_DOWN:
+						case KEY_ARROW_LEFT:
+						case KEY_ARROW_RIGHT:
+						case KEY_ARROW_UP:
+						case KEY_ESCAPE:
 							return false;
 					}
 
@@ -570,13 +616,19 @@ jQuery(function($) {
 				})
 				.on('keydown', function(e) {
 					switch (e.which) {
-						case KEY.TAB:
-						case KEY.ESCAPE:
-							hideAvailable($obj);
+						case KEY_TAB:
+						case KEY_ESCAPE:
+							var $available = ms.values.available_div;
+
+							if ($available.parent().is(document.body)) {
+								hideAvailable($obj);
+								e.stopPropagation();
+							}
+
 							cleanSearchInput($obj);
 							break;
 
-						case KEY.ENTER:
+						case KEY_ENTER:
 							if ($input.val() !== '') {
 								var $selected = $('li.suggest-hover', ms.values.available_div);
 
@@ -589,7 +641,7 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_LEFT:
+						case KEY_ARROW_LEFT:
 							if ($input.val() === '') {
 								var $collection = $('.selected li', $obj);
 
@@ -605,7 +657,7 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_RIGHT:
+						case KEY_ARROW_RIGHT:
 							if ($input.val() === '') {
 								var $collection = $('.selected li', $obj);
 
@@ -621,13 +673,13 @@ jQuery(function($) {
 							}
 							break;
 
-						case KEY.ARROW_UP:
-						case KEY.ARROW_DOWN:
+						case KEY_ARROW_UP:
+						case KEY_ARROW_DOWN:
 							var $collection = $('li', ms.values.available_div.filter(':visible')),
 								$selected = $collection.filter('.suggest-hover').removeClass('suggest-hover');
 
 							if ($selected.length) {
-								$selected = (e.which == KEY.ARROW_UP)
+								$selected = (e.which == KEY_ARROW_UP)
 									? ($selected.is(':first-child') ? $collection.last() : $selected.prev())
 									: ($selected.is(':last-child') ? $collection.first() : $selected.next());
 
@@ -639,8 +691,8 @@ jQuery(function($) {
 
 							return false;
 
-						case KEY.BACKSPACE:
-						case KEY.DELETE:
+						case KEY_BACKSPACE:
+						case KEY_DELETE:
 							if ($input.val() === '') {
 								var $selected = $('.selected li.selected', $obj);
 
@@ -651,7 +703,7 @@ jQuery(function($) {
 									if (typeof item.disabled === 'undefined' || !item.disabled) {
 										var aria_text = sprintf(t('Removed, %1$s'), $selected.data('label'));
 
-										$selected = (e.which == KEY.BACKSPACE)
+										$selected = (e.which == KEY_BACKSPACE)
 											? ($selected.is(':first-child') ? $selected.next() : $selected.prev())
 											: ($selected.is(':last-child') ? $selected.prev() : $selected.next());
 
@@ -676,10 +728,10 @@ jQuery(function($) {
 										$aria_live.text(aria_text);
 									}
 									else {
-										$aria_live.text(t('Can not be removed'));
+										$aria_live.text(t('Cannot be removed'));
 									}
 								}
-								else if (e.which == KEY.BACKSPACE) {
+								else if (e.which == KEY_BACKSPACE) {
 									/*
 									 * Pressing Backspace on empty input field should select last element in
 									 * multiselect. For next Backspace press to be able to remove it.
@@ -838,7 +890,6 @@ jQuery(function($) {
 		var ms = $obj.data('multiSelect'),
 			is_new = item.isNew || false,
 			prefix = item.prefix || '',
-			search = ms.values.search.replace(/[.+?^${}()|[\]\\]/g, '\\$&').replace(/[*]/g, '\\\*?'),
 			$li = $('<li>', {
 				'data-id': item.id,
 				'data-label': prefix + item.name
@@ -856,11 +907,38 @@ jQuery(function($) {
 		}
 
 		// Highlight matched.
-		$li
-			.append(item.name.replace(new RegExp(search, 'gi'), function(match) {
-				return '<span' + (!is_new ? ' class="suggest-found"' : '') + '>' + match + '</span>';
-			}))
-			.toggleClass('suggest-new', is_new);
+		if (ms.values.search !== item.name) {
+			var text = item.name.toLowerCase(),
+				search = ms.values.search.toLowerCase().replace(/[*]+/g, ''),
+				start = 0,
+				end = 0;
+
+			while (search !== '' && text.indexOf(search, end) > -1) {
+				end = text.indexOf(search, end);
+
+				if (end > start) {
+					$li.append(document.createTextNode(item.name.substring(start, end)));
+				}
+
+				$li.append($('<span>', {
+					class: !is_new ? 'suggest-found' : '',
+					text: item.name.substring(end, end + search.length)
+				})).toggleClass('suggest-new', is_new);
+
+				end += search.length;
+				start = end;
+			}
+
+			if (end < item.name.length) {
+				$li.append(document.createTextNode(item.name.substring(end, item.name.length)));
+			}
+		}
+		else {
+			$li.append($('<span>', {
+				class: !is_new ? 'suggest-found' : '',
+				text: item.name
+			})).toggleClass('suggest-new', is_new);
+		}
 
 		$('ul', ms.values.available_div).append($li);
 	}
@@ -1192,7 +1270,7 @@ jQuery(function($) {
 	}
 
 	function isSearchFieldVisible($obj) {
-		return !$obj.hasClass('.search-disabled');
+		return !$obj.hasClass('search-disabled');
 	}
 
 	function getLimit($obj) {
@@ -1202,4 +1280,4 @@ jQuery(function($) {
 			? ms.options.limit + objectSize(ms.values.selected) + ms.options.excludeids.length + 1
 			: null;
 	}
-});
+})(jQuery);
